/***************************************************************************
 *																		 *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or	 *
 *   (at your option) any later version.								   *
 *																		 *
 ***************************************************************************/

// TCL module includes
#include "tcl_scripting.h"
#include "tclloaddialog.h"

// Kadu includes
#include "debug.h"
#include "misc.h"

// QT includes
#include <qpushbutton.h>
#include <qlabel.h>
#include <qlistbox.h>
#include <qlayout.h>
#include <qfont.h>
#include <qmessagebox.h>

TclLoadDialog::TclLoadDialog()
{
	kdebugf();
	setCaption(tr("TCL Scripts"));
	setMinimumWidth(250);
	setMinimumHeight(300);
	QFont labFont("helvetica", 12, QFont::Bold);

	// Close
	QPushButton* btn_close = new QPushButton(this);
	btn_close->setText(tr("Close"));
	btn_close->setDefault(FALSE);

	// Load
	QPushButton* btn_load = new QPushButton(this);
	btn_load->setText(tr("Load"));
	btn_load->setDefault(TRUE);

	// Unload
	QPushButton *btn_unload = new QPushButton(this);
	btn_unload->setText(tr("Unload"));

	// Loaded
	QLabel* loaded_label = new QLabel(this);
	loaded_label->setText(tr("Loaded scripts"));
	loaded_label->setFont(labFont);
	loaded_list = new QListBox(this);
	loaded_list->setMinimumWidth(150);
	loaded_list->setMinimumHeight(200);
	loaded_list->setAutoScroll(TRUE);

	// Local
	QLabel* local_label = new QLabel(this);
	local_label->setText(tr("Local scripts"));
	local_label->setFont(labFont);
	local_list = new QListBox(this);
	local_list->setMinimumWidth(150);
	local_list->setMinimumHeight(90);
	local_list->setAutoScroll(TRUE);

	// Global
	QLabel* global_label = new QLabel(this);
	global_label->setText(tr("Global scripts"));
	global_label->setFont(labFont);
	global_list = new QListBox(this);
	global_list->setMinimumWidth(150);
	global_list->setMinimumHeight(90);
	global_list->setAutoScroll(TRUE);

	// Others
	QPushButton* btn_others = new QPushButton(this);
	btn_others->setText(tr("Other scripts"));

	// Others
	QPushButton* btn_reload = new QPushButton(this);
	btn_reload->setText(tr("Reload"));

	// Layout
	QGridLayout *g = new QGridLayout(this, 1, 1);

	g->addMultiCellWidget(btn_load, 0, 0, 0, 9);
	g->addMultiCellWidget(btn_unload, 0, 0, 11, 20);
 
	g->addMultiCellWidget(local_label, 1, 1, 0, 9);
	g->addMultiCellWidget(local_list, 2, 10, 0, 9);
	g->addMultiCellWidget(global_label, 11, 11, 0, 9);
	g->addMultiCellWidget(global_list, 12, 19, 0, 9);
	g->addMultiCellWidget(loaded_label, 1, 1, 11, 20);
	g->addMultiCellWidget(loaded_list, 2, 19, 11, 20);
 
	g->addMultiCellWidget(btn_others, 20, 20, 0, 9);
	g->addMultiCellWidget(btn_reload, 20, 20, 11, 20);
	g->addMultiCellWidget(btn_close, 21, 21, 0, 20);

	resize(250, 350);
	
	// Slots	
	connect(btn_close, SIGNAL(clicked()), this, SLOT(done()));
	connect(btn_others, SIGNAL(clicked()), this, SLOT(others()));
	connect(btn_load, SIGNAL(clicked()), this, SLOT(load()));
	connect(btn_unload, SIGNAL(clicked()), this, SLOT(unload()));
	connect(btn_reload, SIGNAL(clicked()), this, SLOT(reload()));

	connect(local_list, SIGNAL(doubleClicked(QListBoxItem*)), this, SLOT(loadLocal(QListBoxItem*)));
	connect(global_list, SIGNAL(doubleClicked(QListBoxItem*)), this, SLOT(loadGlobal(QListBoxItem*)));
	connect(loaded_list, SIGNAL(doubleClicked(QListBoxItem*)), this, SLOT(unload(QListBoxItem*)));

	// Paths
	localPath = ggPath("");
	globalPath = dataPath("kadu/modules/data/tcl_scripting/scripts/");

	// Making local and global scripts list
	QStringList locals = QDir(localPath).entryList(),
				globals = QDir(globalPath).entryList();
	for ( uint i = 0; i < locals.count(); i++ )
	{
		if (locals[i].find(".tcl", FALSE) > -1 && locals[i].compare("default.tcl"))
		{
			local_list->insertItem(locals[i]);
		}
	}
	for ( uint i = 0; i < globals.count(); i++ )
	{
		if (globals[i].find(".tcl", FALSE) > -1)
		{
			global_list->insertItem(globals[i]);
		}
	}
}

void TclLoadDialog::load(int list_type)
{
	kdebugf();
	QString item, file;
	if (list_type == 0)
	{
		if (local_list->currentItem() > -1)
		{
			item = local_list->currentText();
			file = localPath + item;
			list_type = 1;
		}
		else if (global_list->currentItem() > -1)
		{
			item = global_list->currentText();
			file = globalPath + item;
			list_type = 2;
		}
		else
		{
			return;
		}
	}
	else
	{
		switch (list_type)
		{
			case 1:
				item = local_list->currentText();
				file = localPath + item;
				break;
			case 2:
				item = global_list->currentText();
				file = globalPath + item;
				break;
		}
	}

	if (loaded(item))
	{
		QMessageBox::critical( 0, tr("Error"), tr("Script %1 is already loaded.").arg(item),
			QMessageBox::Ok, QMessageBox::NoButton, QMessageBox::NoButton);

		return;
	}

	TclSession* tsess = new TclSession(FALSE);

	bool ret = tsess->load(file);

	if (ret == TCL_ERROR)
	{
		tsess->error(QString::fromUtf8(Tcl_GetStringResult(tsess->tclInt())), TRUE);
		delete tsess;
	}
	else
	{
		loaded_list->insertItem(item);
		switch (list_type)
		{
			case 1:
			{
				local_list->removeItem(local_list->currentItem());
				break;
			}
			case 2:
			{
				global_list->removeItem(global_list->currentItem());
				break;
			}
		}
	}
}

void TclLoadDialog::loadLocal(QListBoxItem*)
{
	kdebugf();
	load(1);
}

void TclLoadDialog::loadGlobal(QListBoxItem*)
{
	kdebugf();
	load(2);
}

void TclLoadDialog::unload(QListBoxItem* it)
{
	kdebugf();
	if (loaded_list->currentItem() == -1)
		return;

	QString item = loaded_list->currentText(), script;
	for ( TclSessions::Iterator it = tclSess.begin(); it != tclSess.end(); it++ )
	{
		script = QStringList::split("/", it.data()->scriptName() ).last();
		if (script == item)
		{
			loaded_list->removeItem(loaded_list->currentItem());
			delete it.data();
			break;
		}
	}
	QFile localScripts(QString("%1%2").arg(localPath).arg(item));
	QFile globalScripts(QString("%1%2").arg(globalPath).arg(item));
	if (localScripts.exists())
	{
		bool sw = FALSE;
		for ( uint i = 0; i < local_list->count(); i++ )
		{
			if (!local_list->text(i).compare(item))
			{
				sw = TRUE;
				break;
			}
		}
		if (sw)
		{
			if (globalScripts.exists())
			{
				global_list->insertItem(item);
				return;
			}
		}
		local_list->insertItem(item);
	}
	else if (globalScripts.exists())
	{
		global_list->insertItem(item);
	}
}

void TclLoadDialog::reload()
{
	kdebugf();
	if (!loaded_list->selectedItem())
		return;

	QString item = loaded_list->currentText(), script, file;
	for ( TclSessions::Iterator it = tclSess.begin(); it != tclSess.end(); it++ )
	{
		file = it.data()->scriptName();
		script = QStringList::split("/", file).last();
		if (script == item)
		{
			delete it.data();

			TclSession* tsess = new TclSession(FALSE);
			bool ret = tsess->load(file);
			if (ret == TCL_ERROR)
			{
				loaded_list->removeItem(loaded_list->currentItem());
				QFile localScripts(QString("%1%2").arg(localPath).arg(item));
				QFile globalScripts(QString("%1%2").arg(globalPath).arg(item));
				if (localScripts.exists())
				{
					bool sw = FALSE;
					for ( uint i = 0; i < local_list->count(); i++ )
					{
						if (!local_list->text(i).compare(item))
						{
							sw = TRUE;
							break;
						}
					}
					if (sw)
					{
						if (globalScripts.exists())
						{
							global_list->insertItem(item);
							return;
						}
					}
					local_list->insertItem(item);
				}
				else if (globalScripts.exists())
				{
					global_list->insertItem(item);
				}
				tclSess[0]->error(QString::fromUtf8(Tcl_GetStringResult(tsess->tclInt())), TRUE);
				delete tsess;
			}
			break;
		}
	}
}

void TclLoadDialog::done()
{
	kdebugf();
	close();
}

void TclLoadDialog::others()
{
	kdebugf();
	QFileDialog *fd = new QFileDialog();
	fd->setCaption(tr("Choose file to load"));
	fd->setMode(QFileDialog::ExistingFile);
	fd->setFilter(tr("TCL scripts (%1)").arg("*.tcl *.TCL"));
	fd->show();
	if ( fd->exec() == QDialog::Accepted )
	{
		QString file = fd->selectedFile(), item = QStringList::split("/", file).last();
		if (loaded(item))
		{
			QMessageBox::critical( 0, tr("Error"), tr("Script %1 is already loaded.").arg(item),
				QMessageBox::Ok, QMessageBox::NoButton, QMessageBox::NoButton);

			return;
		}

		TclSession* tsess = new TclSession(FALSE);

		bool ret = tsess->load(file.latin1());

		if (ret == TCL_ERROR)
		{
			tsess->error(QString::fromUtf8(Tcl_GetStringResult(tsess->tclInt())), TRUE);
			delete tsess;
		}
		else
		{
			loaded_list->insertItem(QStringList::split("/", file ).last());
		}
	}
}

QListBox* TclLoadDialog::list()
{
	kdebugf();
	return loaded_list;
}

QListBox* TclLoadDialog::lList()
{
	kdebugf();
	return local_list;
}

QListBox* TclLoadDialog::gList()
{
	kdebugf();
	return global_list;
}

bool TclLoadDialog::loaded(QString script)
{
	kdebugf();
	bool l = FALSE;
	for ( uint i = 0; i < loaded_list->count(); i++ )
	{
		if (loaded_list->text(i) == script)
		{
			l = TRUE;
			break;
		}
	}
	return l;
}

void TclLoadDialog::keyPressEvent(QKeyEvent* e)
{
	kdebugf();
	switch (e->key())
	{
		case Qt::Key_Delete:
			unload();
			break;
		case Qt::Key_Insert:
			load();
			break;
	}
}

