/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// TCL module includes
#include "tcl_scripting.h"
#include "tcltimer.h"
#include "tcllistbox.h"
#include "tclstring.h"
#include "tclmisc.h"

// QT includes
#include <qmenubar.h>
#include <qinputdialog.h>
#include <qmessagebox.h>
#include <qregexp.h>

// Kadu includes
#include "ignore.h"
#include "config_file.h"
#include "config_dialog.h"
#include "status.h"
#include "misc.h"
#include "message_box.h"
#include "emoticons.h"
#include "search.h"
#include "modules.h"
#include "history.h"
#include "gadu.h"

// External modules
#include "../sound/sound.h"
#include "../docking/docking.h"
#include "../notify/notify.h"

/* Tcl Command quit
SYNTAX: quit ?opis?
DESCRIPTION:
Zapisuje ustawienia i list kontaktw, nastepnie zamyka program. Opcjonalnie mona
poda opis. W takim wypadku przed zamkniciem programu ustawiany jest status
'Niedostpny z opisem' wraz z podanym opisem.
*/
int Tcl_CmdQuit(void* clientData, Tcl_Interp *interp, int argc, const char *argv[])
{
	if (argc > 2)
	{
		Tcl_WrongNumArgs(interp, 0, NULL, "quit ?description?");
		return TCL_ERROR;
	}
	if (argc > 1)
	{
		gadu->status().setOffline(argv[1]);
	}
	else
	{
		gadu->status().setOffline();
	}
	qApp->exit(0);
	return TCL_OK;
}

/* Tcl Command server
SYNTAX: server komenda ?argumenty?
DESCRIPTION:
Komenda server suy do obsugi wszelkich operacji zwizanych z poczeniem z serwerem GG.

SUBSYNTAX: server connect
DESCRIPTION:
czy Kadu z serwerem GG i ustawia status 'Dostpny'.

SUBSYNTAX: server disconnect
DESCRIPTION:
Odcza Kadu od serwera GG.

SUBSYNTAX: server current
DESCRIPTION:
Zwraca adres IP serwera, z ktrym uytkownik jest aktualnie poczony, lub z ktrym
si czy. Jeli jest to HUB, to zwracany jest acuch "hub".
*/
int Tcl_CmdServer(void* clientData, Tcl_Interp *interp, int argc, const char *argv[])
{
	if (argc < 2)
	{
		Tcl_WrongNumArgs(interp, 0, NULL, "server command ?arguments?");
		return TCL_ERROR;
	}
	QStringList cmds = QStringList::split(" ", "connect disconnect current");
	switch (cmds.findIndex(argv[1]))
	{
		case 0:
		{
			if (argc != 2)
			{
				Tcl_WrongNumArgs(interp, 0, NULL, "server connect");
				return TCL_ERROR;
			}
			gadu->status().setOnline();
			break;
		}
		case 1:
		{
			if (argc != 2)
			{
				Tcl_WrongNumArgs(interp, 0, NULL, "server disconnect");
				return TCL_ERROR;
			}
			gadu->status().setOffline();
			break;
		}
		case 2:
		{
			if (argc != 2)
			{
				Tcl_WrongNumArgs(interp, 0, NULL, "server current");
				return TCL_ERROR;
			}
			if (gadu->activeServer())
				tclMisc->setTclResult(interp, gadu->activeServer()->toString());
			else
				tclMisc->setTclResult(interp, "hub");

			break;
		}
		default:
			tclMisc->setTclResult(interp, QObject::tr("Command should be one of following: %1").arg(cmds.join(", ")));
			return TCL_ERROR;
	}
	return TCL_OK;
}

/* Tcl Command msgbox
SYNTAX: msgbox typ ?tytu? wiadomo ?opcje?

DESCRIPTION:
Wywietla okno z wiadomoci (w ktrej mona uywa znacznikw HTML), o podanym tytule.
Typ moe przyj jedn z siedmiu wartoci: info, warning, critical, msg, ask, status, oraz progress.

SUBSYNTAX: msgbox information/warning/critical tytu wiadomo
DESCRIPTION:
Standardowe okna dialogowe biblioteki QT. Zawieraj odpowiedni ikonk, oraz mona dla nich
zdefiniowa tytu okna.

SUBSYNTAX: msgbox msg wiadomo
DESCRIPTION:
Najprostsze okno informacyjne z podan <I>wiadomoci</I>, oraz przyciskiem "OK".

SUBSYNTAX: msgbox ask wiadomo
DESCRIPTION:
Okno z <I>wiadomoci</I> jako pytaniem, na ktre uytkownik moe odpowiedzie
poprzez kliknicie na jednym z 2 przyciskw - "tak" lub "nie". Dokonany wybr decyduje
o zwrconej przez komend wartoci - 1 (tak) lub 0 (nie).

SUBSYNTAX: msgbox status wiadomo ?-close?
DESCRIPTION:
Okno informujce uytkownika o wykonywanej aktualnie jakiej czynnoci. Nie ma w nim
adnych przyciskw, a zamykane jest przez t sam komend (z t sam <I>wiadomoci</I>)
z opcj -close, lub odpowiedni funkcj menadera okien.

SUBSYNTAX: msgbox progress wiadomo ?kroki? ?-progress warto? ?-close?
DESCRIPTION:
Informuje uytkownika o wykonywanej czynnoci, jednoczenie pokazujc stopie
zaawansowania w przebiegu tej czynnoci. Przy tworzeniu okna wymagany jest argument <I>krokw</I>,
ktry decyduje, ile krokw jest potrzebnych do wykonania czynnoci. Kadorazowo, gdy owa czynno
przejdzie do nastpnego kroku, naley uruchomi t komend z argumentem tej samej <I>wiadomoci</I>,
pomijajc argument <I>krokw</I>, za to podajc opcj -progress i odpowiedni dla niej warto,
ktra okrela z kolei, w ktrym <I>kroku</I> aktualnie jest wykonywana czynno. Po zakoczeniu
pracy okno naley zamkn t sam komend z opcj -close. Dodatkowo w oknie tym jest przycisk
"anuluj", ktry umoliwia przerwanie wykonywania czynnoci przez uytkownika. Wywoywane jest wtedy
zdarzenie <B>progresscancel</B> (patrz: komenda <a href=on.html></B>on<B></a>), w implementacji
ktrego powinna zosta przerwana wykonywana czynno.
*/
int Tcl_CmdMessageBox(void* clientData, Tcl_Interp *interp, int argc, const char *argv[])
{
	if (argc < 3)
	{
		Tcl_WrongNumArgs(interp, 0, NULL, "msgbox type ?caption? message ?options?");
		return TCL_ERROR;
	}
	QStringList types = QStringList::split(" ", "critical information warning msg ask status progress");
	switch (types.findIndex(QString(argv[1])))
	{
		case 0:
		{
			if (argc != 4)
			{
				Tcl_WrongNumArgs(interp, 0, NULL, "msgbox critical caption message");
				return TCL_ERROR;
			}
			QMessageBox::critical( 0, QString::fromUtf8(argv[2]), QString::fromUtf8(argv[3]) );
			break;
		}
		case 1:
		{
			if (argc != 4)
			{
				Tcl_WrongNumArgs(interp, 0, NULL, "msgbox information caption message");
				return TCL_ERROR;
			}
			QMessageBox::information( 0, QString::fromUtf8(argv[2]), QString::fromUtf8(argv[3]) );
			break;
		}
		case 2:
		{
			if (argc != 4)
			{
				Tcl_WrongNumArgs(interp, 0, NULL, "msgbox warning caption message");
				return TCL_ERROR;
			}
			QMessageBox::warning( 0, QString::fromUtf8(argv[2]), QString::fromUtf8(argv[3]) );
			break;
		}
		case 3:
		{
			if (argc != 3)
			{
				Tcl_WrongNumArgs(interp, 0, NULL, "msgbox msg message");
				return TCL_ERROR;
			}
			MessageBox::msg(QString::fromUtf8(argv[2]));
			break;
		}
		case 4:
		{
			if (argc != 3)
			{
				Tcl_WrongNumArgs(interp, 0, NULL, "msgbox ask message");
				return TCL_ERROR;
			}
			Tcl_SetResult(interp, (char*)QString::number(
				MessageBox::ask(QString::fromUtf8(argv[2]))
				).latin1(), TCL_VOLATILE);
			break;
		}
		case 5:
		{
			if (argc < 3 || argc > 4)
			{
				Tcl_WrongNumArgs(interp, 0, NULL, "msgbox status message ?-close?");
				return TCL_ERROR;
			}
			if (argc == 3)
			{
				MessageBox::status(QString::fromUtf8(argv[2]));
			}
			else
			{
				if (QString::compare(argv[3], "-close"))
				{
					Tcl_WrongNumArgs(interp, 0, NULL, "msgbox status message ?-close?");
					return TCL_ERROR;
				}
				MessageBox::close(QString::fromUtf8(argv[2]));
			}
			break;
		}
		case 6:
		{
			if (argc < 3 || argc > 5)
			{
				Tcl_WrongNumArgs(interp, 0, NULL, "msgbox progress message ?steps? ?-progress value? ?-close?");
				return TCL_ERROR;
			}
			if (argc == 5)
			{
				bool ok;
				int val = QString(argv[4]).toInt(&ok);
				if (QString::compare(argv[3], "-progress") || !ok)
				{
					Tcl_WrongNumArgs(interp, 0, NULL, "msgbox progress message -progress value");
					return TCL_ERROR;
				}
				MessageBox::progress(QString::fromUtf8(argv[2]), val);
			}
			else if (argc == 4)
			{
				if (!QString::compare(argv[3], "-close"))
				{
					MessageBox::close(QString::fromUtf8(argv[2]));
				}
				else
				{
					bool ok;
					int val = QString(argv[3]).toInt(&ok);
					if (!ok)
					{
						Tcl_WrongNumArgs(interp, 0, NULL, "msgbox progress message ?steps? ?-progress value? ?-close?");
						return TCL_ERROR;
					}
					int sessID = *((int*) clientData);
					TclSession* tsess = tclSess[sessID];
					MessageBox::progress(QString::fromUtf8(argv[2]), (QObject*)tsess->tclEvtHnd(), SLOT(msgBoxCancel()), val);
				}
			}
			else
			{
				MessageBox::progress(QString::fromUtf8(argv[2]));
			}
			break;
		}
		default:
		{
			QString err = QString("type should be one of: %1").arg(types.join(", "));
			Tcl_WrongNumArgs(interp, 0, NULL, err.local8Bit());
			return TCL_ERROR;
		}
	}
	return TCL_OK;
}

/* Tcl Command inputdialog
GLOBALSYNTAX: inputdialog typ tytu wiadomo ?argumenty?

DESCRIPTION:
Wywietla okno z polem do wpisania danych przez uytkownika. Tytu okna bdzie taki,
jak podany przez uytkownika. Nad polem pojawi si podana wiadomo (mona w niej uywa znacznikw HTML).
Okno ma 2 przyciski - OK, oraz Anuluj. Jeli naciniety zostanie przycisk Anuluj, to komenda ta zwrci pusty acuch,
niezalenie od wpisanej danej w polu, natomiast jeli przycisk OK zostanie nacinity,
wtedy zwrcona zostanie zawarto pola.

SUBSYNTAX: inputdialog text tytu wiadomo ?domylnyTekst ?typWywietlania??

DESCRIPTION:
Opcjonalnie mona poda <I>domylny tekst</I>, oraz <I>rodzaj wywietlania</I>, ktry moe przyj
jedn z trzech wartoci: <B>normal</B>, <B>noecho</B>, <B>password</B>, gdzie pierwszy jest uywany
domylnie, drugi powoduje, e pomimo i uytkownik wpisuje tekst, w polu nie pokazuj si adne znaki.
Wreszcie trzecia z moliwoci wywietla kady wpisany znak jako gwiazdk.

SUBSYNTAX: inputdialog item tytu wiadomo lista ?pocztkowaPozycja ?edytowalny??

DESCRIPTION:
Dodatkowym wymaganym argumentem jest lista sw, ktre zo si na liste pozycji do wyboru.
Opcjonalny argument to <I>pocztkowa pozycja</I>, ktr stanowi liczba wiksza, lub rwna 0.
Warto rwna 0 oznacza, e pierwsze sowo z listy bdzie domylnie wybrane, kade nastpne sowo
oznaczone jest kolejn liczb. Kolejnym opcjonalnym argumentem jest ?edytowalno? pola. Domylnie
jego warto jest rwna TRUE, co oznacza, e oprcz sw z podanej listy, uytkownik moe poda
swoje wasne. Jeli argument ten zostanie ustawiony na warto negatywn (tj. false, no, off, 0),
to takiej moliwoci nie bdzie.

SUBSYNTAX: inputdialog integer tytu wiadomo ?wartoDomylna ?minimum ?maksimum ?krok????

DESCRIPTION:
Jako pierwszy argument opcjonalny mona poda <I>domyln warto</I> pola. Komenda dotyczy typu danych integer,
w zwizku z tym musi to by warto liczbowa, cakowita. Kolejne argumenty opcjonalne to <I>minimalna</I> i <I>maksymalna</I>
warto, ktr bdzie mg poda uytkownik. Ostatni z argumentw okrela, jak duy ma by <I>krok</I>, czyli o ile
ma si zwikszy warto liczby w polu, po klikniciu na strzak wzrostu, a ile zmale, gdy na strzak zmniejszania.
Warto uywa tego dialogu, kiedy zaley nam, aby uytkownik mg wpisywa tylko liczby cakowite (ew. z rzdanego zakresu).

SUBSYNTAX: inputdialog double tytu wiadomo ?wartoDomylna ?minimum ?maksimum ?precyzja????

DESCRIPTION:
Okno dialogowe podobne do inputDialog integer, tyle e tutaj mona (a nawet trzeba) poda liczbe rzeczywista,
w postaci 'xxx.xxxxxx', gdzie liczba cyfr przed przecinkiem jest nieograniczona*, a po przecinku jest okrelona przez ostatni
z argumentw opcjonalnych. Podana precyzja dotyczy tylko wartoci, ktr moe wpisa uytkownik. Bez wzgldu na warto
<I>precyzji</I> zwrcona liczba bdzie miaa dokadno 6 miejsc po przecinku.
*/
int Tcl_CmdInputDialog(void* clientData, Tcl_Interp *interp, int argc, const char *argv[])
{
	if (argc < 4)
	{
		Tcl_WrongNumArgs(interp, 0, NULL, "inputdialog type caption message ?arguments?");
		return TCL_ERROR;
	}
	QStringList types = QStringList::split(" ", "text item integer double");
	QString str = argv[1];
	switch (types.findIndex(str))
	{
		case -1:
		{
			Tcl_WrongNumArgs(interp, 0, NULL, "valid types: text, item, integer, double");
			return TCL_ERROR;
			break;
		}
		case 0:
		{
			if (argc > 6)
			{
				Tcl_WrongNumArgs(interp, 0, NULL, "inputdialog text caption message ?defaultText ?echoType??");
				return TCL_ERROR;
			}
			QString def = "";
			QLineEdit::EchoMode type = QLineEdit::Normal;
			bool ok;
			if (argc > 4)
			{
				def = QString::fromUtf8(argv[4]);
				if (argc > 5)
				{
					QString tmp = argv[5];
					if (!tmp.compare("normal"))
					{
						type = QLineEdit::Normal;
					}
					else if (!tmp.compare("noecho"))
					{
						type = QLineEdit::NoEcho;
					}
					else if (!tmp.compare("password"))
					{
						type = QLineEdit::Password;
					}
					else
					{
						Tcl_WrongNumArgs(interp, 0, NULL, "inputdialog text caption message ?defaultText ?echoType??");
						return TCL_ERROR;
					}
				}
			}
			QString text = QInputDialog::getText(QString::fromUtf8(argv[2]), QString::fromUtf8(argv[3]), type, def, &ok );
			if ( ok && !text.isEmpty() )
				tclMisc->setTclResult(interp, text);

			break;
		}
		case 1:
		{
			QString syntax = "inputdialog item caption message list ?current ?editable??";
			if (argc < 5)
			{
				Tcl_WrongNumArgs(interp, 0, NULL, syntax.local8Bit());
				return TCL_ERROR;
			}
			int cnt = 1;
			bool edit = 1, conv, ok;
			if (argc > 5)
			{
				cnt = QString(argv[5]).toInt(&conv);
				if (!conv)
				{
					Tcl_WrongNumArgs(interp, 0, NULL, syntax.local8Bit());
					return TCL_ERROR;
				}
				if (argc > 6)
				{
					if (argc > 7)
					{
						Tcl_WrongNumArgs(interp, 0, NULL, syntax.local8Bit());
						return TCL_ERROR;
					}
					int tmp;
					Tcl_GetBoolean(interp, argv[6], &tmp);
					edit = tmp;
				}
				else
				{
					edit = 1;
				}
			}
			QStringList lst = QStringList::split(" ", QString::fromUtf8(argv[4]));

			QString text = QInputDialog::getItem(
				QString::fromUtf8(argv[2]), QString::fromUtf8(argv[3]), lst, cnt, edit, &ok);

			if ( ok && !text.isEmpty() )
				tclMisc->setTclResult(interp, text);

			break;
		}
		case 2:
		{
			QString syntax = "inputDialog integer caption message ?default ?min ?max ?step????";
			int def = 0;
			int min = -2147483647;
			int max = 2147483647;
			int step = 1;
			bool ok;
			if (argc > 4)
			{
				def = QString(argv[4]).toInt();
				if (argc > 5)
				{
					min = QString(argv[5]).toInt();
					if (argc > 6)
					{
						max = QString(argv[6]).toInt();
						if (argc > 7)
						{
							step = QString(argv[7]).toInt();
							if (argc > 8)
							{
								Tcl_WrongNumArgs(interp, 0, NULL, syntax.local8Bit());
								return TCL_ERROR;
							}
						}
					}
				}
			}
			int res = QInputDialog::getInteger(QString::fromUtf8(argv[2]), QString::fromUtf8(argv[3]), def, min, max, step, &ok );
			if ( ok )
				Tcl_SetResult(interp, (char*)QString::number(res).latin1(), TCL_VOLATILE);

			break;
		}
		case 3:
		{
			QString syntax = "inputDialog double caption message ?default ?min ?max ?precision????";
			int def = 0;
			int min = -2147483647;
			int max = 2147483647;
			int precision = 1;
			bool ok;
			if (argc > 4)
			{
				def = QString(argv[4]).toInt();
				if (argc > 5)
				{
					min = QString(argv[5]).toInt();
					if (argc > 6)
					{
						max = QString(argv[6]).toInt();
						if (argc > 7)
						{
							precision = QString(argv[7]).toInt();
							if (argc > 8)
							{
								Tcl_WrongNumArgs(interp, 0, NULL, syntax.local8Bit());
								return TCL_ERROR;
							}
						}
					}
				}
			}
			double res = QInputDialog::getDouble(QString::fromUtf8(argv[2]), QString::fromUtf8(argv[3]), def, min, max, precision, &ok );
			if ( ok )
				Tcl_SetResult(interp, (char*)QString::number(res, 'f').latin1(), TCL_VOLATILE);

			break;
		}
	}
	return TCL_OK;
}

/* Tcl Command filedialog
SYNTAX: filedialog typ tytu ?nazwaFiltra maskaFiltra ?nazwaFiltra maskaFiltra ?...???
DESCRIPTION:
Wywietla okno przegldania plikw. Typ moe przyj jedn z trzech wartoci: 1) <B>open</B>, 2) <B>save</B>, lub 3) <B>getdir</B>,
ktre oznaczaj odopwiednio, e wybrany plik: 1) musi istnie, 2) moe, ale nie musi istnie, 3) Musi istnie i musi to by katalog.
Dodatkowo mona poda filtry, ktre pomog w wyszukiwaniu plikw. <I>Nazwa filtru</I> moe przyj warto np: "Obrazki", lub
"Pliki konfiguracyjne". Nie ma to wikszego znaczenia, bdzie to wycznie dla informacji uytkownika. <I>Maska filtra</I> natomiast
okrela jakie typy plikw powinny by wywietlane. Jest to lista sw oddzielonych spacjami, np: "*.jpg *.png", lub "*.cfg".
Dla kadej <I>nazwy filtra</I> musi by podana jako nastpna <I>maska</I> dla tego filtra. Jeli aden filtr nie zostanie
podany, domylnie zostanie uyty "Wszystkie pliki" "*".

EXAMPLE:
set file [filedialog open "Wybierz plik" "Pliki rdowe C/C++" "*.c *.cpp" "Skrypty TCL" "*.tcl"]
ENDOFEXAMPLE
*/
int Tcl_CmdFileDialog(void* clientData, Tcl_Interp *interp, int argc, const char *argv[])
{
	if (argc < 3 || !(argc % 2))
	{
		Tcl_WrongNumArgs(interp, 0, NULL, "filedialog type caption ?filtername filtermasks ?filtername filtermasks ?...???");
		return TCL_ERROR;
	}
	QString cdir = QDir::currentDirPath(), type = QString::fromUtf8(argv[1]),
			caption = QString::fromUtf8(argv[2]), tmp, res;
	QStringList types = QStringList::split(" ", "open save getdir"), filters;
	for ( int i = 3; i < argc; i++ )
	{
		tmp = argv[i];
		tmp.append(" (");
		i++;
		tmp.append(argv[i]);
		tmp.append(")");
		filters.append(tmp);
	}
	switch (types.findIndex(type))
	{
		case 0:
		{
			res = QFileDialog::getOpenFileName(QString::null, filters.join(";;"), 0, 0, caption);
			break;
		}
		case 1:
		{
			res = QFileDialog::getSaveFileName(QString::null, filters.join(";;"), 0, 0, caption);
			break;
		}
		case 2:
		{
			res = QFileDialog::getExistingDirectory(QString::null, 0, 0, caption, TRUE);
			break;
		}
		default:
		{
			tclMisc->setTclResult(interp, QObject::tr("Type must be one of following: %1").arg(types.join(", ")));
			return TCL_ERROR;
		}
	}

	if (res != QString::null)
	{
		tclMisc->setTclResult(interp, res);
	} else {
		Tcl_SetResult(interp, "", TCL_STATIC);
	}

	return TCL_OK;
}

/* Tcl Command user
GLOBALSYNTAX: user komenda ?argumenty?
DESCRIPTION:
Komenda suca zarzdzaniu list kontaktw.

SUBSYNTAX: user add ?-uin liczba -altname nazwa ?-firstname nazwa? ?-lastname nazwa? ?-nickname nazwa? ?-mobile numer? ?-blocking true/false? ?-offline true/false? ?-notify true/false? ?-group grupa? ?-email adres? ?-anonymous true/false??
DESCRIPTION:
Jeli adna z opcji nie jest podana, otwiera dialog dodawania nowego uytkownika. Jeli jakakolwiek opcja
jest podana, dodaje nowego uytkownika z podanymi, lub domylnymi parametrami. W tym przypadku rwnierz
<u>musz</u> by zdefiniowane 2 parametry: UIN (-uin) oraz nazwa alternatywna (-altname).<br><br>
<I>Nazwa alternatywna</I> (-altname) jest nazw wywietlan w oknie kontaktw.<br><br>
<I>UIN</I> (-uin) jest unikalnym identyfikatorem uytkownika GG.<br><br>
<I>Pseudonim</I> (-nickname) jest nazw zwykle identyczny z nazw alternatywn, lecz czasem
istnieje potrzeba zmiany nazwy wywietlanej na inn ni pseudonim, a mimo to, bdzie on pamitany
jako osobny parametr. Domylna warto to pusty acuch.<br><br>
<I>Blokowanie</I> (-blocking) wcza lub wycza blokowanie uytkownika. Domylna warto to FALSE.
Argument dla tej opcji jest dozwolony w formie TRUE/FALSE, jak rwnie 1/0.<br><br>
<I>Bd niedostpny</I> (-offline) dla tego uytkownika. Argument identyczny jak dla -blocking.
Domylnie jest FALSE.<br><br>
<I>Powiadamiaj o</I> (-notify) dostpnoci tego uytkownika. Argument identyczny jak dla -blocking.
Domylnie jest TRUE.<br><br>
<I>Anonimowy</I> (-anonymous) definiuje uytkownika tak, jak robi to Kadu automatycznie z uytkownikiem,
ktorego nie mamy na licie, a wysa on do nas jak wiadomo. Domylnie jest FALSE. Opcja ta musi wystpowa
razem z opcj -uin, inne opcje s wtedy ignorowane.<br><br>
<I>Grupa</I> (-group) do ktrej uytkownik ma przynalee. Domylnie jest to "Wszyscy".<br><br>
<I>Imi</I> (-firstname), <I>Nazwisko</I> (-lastname), <I>Numer telefonu</I> (-mobile),
oraz <I>E-Mail</I> (-email) s dobrze znane i nie wymagaj omawiania.<br>

SUBSYNTAX: user remove nazwaAlternatywna
DESCRIPTION:
Uswa uytkownika z listy, ktry jest znany pod podan <I>nazw alternatywn</I>. Jeli nie ma takiego uytkownika
na licie, to zwrcony zostaje bd.

SUBSYNTAX: user list
DESCRIPTION:
Zwraca list nazw alternatywnych wszystkich kontaktw, czyli tych widzianych w oknie kontaktw.

SUBSYNTAX: user uins
DESCRIPTION:
Zwraca list numerw identyfikacyjnych (UIN'w) wszystkich kontaktw.

SUBSYNTAX: user change staraNazwaAlternatywna ?-firstname nazwa? ?-lastname nazwa? ?-nickname nazwa? ?-altname nazwa? ?-mobile numer? ?-uin numer? ?-blocking true/false? ?-offline true/false? ?-notify true/false? ?-group grupa? ?-email adres?
DESCRIPTION:
Zmienia parametry zarejestrowanego uytkownika, skojaonego z nazw alternatywn, podan jako staraNazwaAlternatywna.
Jeli uytkownik o podanej nazwie alternatywnej nie jest zarejestrowany, zwracany jest bd. Wszystkie opcje tutaj spotkane
s opisane w komendzie user add.

SUBSYNTAX: user get UIN opcja
DESCRIPTION:
Zwraca warto parametru okrelonego <I>opcj</I> dla uytkownika o podanej <I>nazwie alternatywnej</I>.
Opcja moe by ktorkolwiek z dozwolonych dla komendy user add, oraz dodatkowo mona tutaj uywa
nastpujcych opcji:<br>
<B>-status</B> - zwraca aktualny status uytkownika<Br>
<B>-description</B> - zwraca opis danego kontaktu (o ile takowy ma aktualnie ustawiony).<br>
<B>-dns</B> - zwraca adres DNS (o ile jest ju gotowy [patrz zdarzenie <a href="on.html"><B>usldns</B></a>])<br>
<B>-ip</B> - zwraca adres IP<br>
<B>-port</B> - zwraca port z ktrego korzysta rozmwca ( = 1, gdy kontakt jest za NAT'em [maskarad])<br>
<B>-protocol</B> - zwraca wersj protokou uywanego przez rozmwc

SUBSYNTAX: user exists UIN/nazwaAlternatywna
DESCRIPTION:
Jeli uytkownik o podanym identyfikatorze <I>UIN</I> lub podanej <I>nazwie alternatywnej</I> jest zarejestrowany
to zwraca 1, w przeciwnym wypadku zwraca 0.
*/
int Tcl_CmdUser(void* clientData, Tcl_Interp *interp, int argc, const char *argv[])
{
	QString mainsyntax = "user command ?arguments?";
	QString addsyntax = "user add ?-uin number -altname name ?-firstname name? ?-lastname name?\n	 ?-nickname name? ?-mobile number? ?-blocking true/false?\n	 ?-offline true/false? ?-notify true/false? ?-group group?	 ?-email address? ?-anonymous true/false??";
	QString remsyntax = "user remove altName";
	QString chsyntax = "user change UIN ?-firstname name? ?-lastname name? ?-nickname name?\n	 ?-altname name? ?-mobile number? ?-uin number? ?-blocking true/false?\n	 ?-offline true/false? ?-notify true/false? ?-group group? ?-email address?";
	QString exsyntax = "user exists UIN/altName";
	QString getsyntax = "user get UIN option";
	QString getsyntax_opts = QObject::tr("option should be: -uin, -nickname, -altname, -firstname, -lastname, -mobile, -blocking, -offline, -notify, -group, -email, -anonymous");

	if (argc < 2)
	{
		Tcl_WrongNumArgs(interp, 0, NULL, mainsyntax.local8Bit());
		return TCL_ERROR;
	}
	bool rest, conv;
	QStringList cmds = QStringList::split(" ", "add remove list uins change exists get"),
				*tclUOpt = new QStringList(QStringList::split(" ", "-uin -altname -firstname -lastname -nickname -mobile -status -blocking -offline -notify -group -description -email -anonymous -dns -ip -port -protocol"));
	QString cmd = argv[1], uin, altNick, anon, str, arg, opt;
	UserListElement *nu, user, _nu;
	int id;
	uint uint_uin;
	int boolVal;
	QCString utf;

	switch (cmds.findIndex(cmd))
	{
		case 0:
		{
			if (argc > 2)
			{
				rest = argc % 2;
				if (rest)
				{
					Tcl_WrongNumArgs(interp, 0, NULL, addsyntax.local8Bit());
					delete tclUOpt;
					return TCL_ERROR;
				}
				else
				{
					nu = new UserListElement();
					int id;
					for (int cnt = 2; argc > cnt; cnt++)
					{
						str = argv[cnt];
						cnt++;
						arg = QString::fromUtf8(argv[cnt]);
						if (str.startsWith("-"))
						{
							id = tclUOpt->findIndex(str);
							switch (id)
							{
								case 0:
									nu->setUin(arg.toInt());
									uin = arg;
									break;
								case 1:
									nu->setAltNick(arg);
									altNick = arg;
									break;
								case 2:
									nu->setFirstName(arg);
									break;
								case 3:
									nu->setLastName(arg);
									break;
								case 4:
									nu->setNickName(arg);
									break;
								case 5:
									nu->setMobile(arg);
									break;
								case 7:
									if (Tcl_GetBoolean(interp, arg.latin1(), &boolVal) == TCL_ERROR)
									{
										str = QString(QObject::tr("Option %1 needs proper TCL boolean value")).arg(str);
										Tcl_SetResult(interp, (char*)str.latin1(), TCL_VOLATILE);
										delete tclUOpt;
										return TCL_ERROR;
									}
									nu->setBlocking((bool)boolVal);
									break;
								case 8:
									if (Tcl_GetBoolean(interp, arg.latin1(), &boolVal) == TCL_ERROR)
									{
										str = QString(QObject::tr("Option %1 needs proper TCL boolean value")).arg(str);
										Tcl_SetResult(interp, (char*)str.latin1(), TCL_VOLATILE);
										delete tclUOpt;
										return TCL_ERROR;
									}
									nu->setOfflineTo((bool)boolVal);
									break;
								case 9:
									if (Tcl_GetBoolean(interp, arg.latin1(), &boolVal) == TCL_ERROR)
									{
										str = QString(QObject::tr("Option %1 needs proper TCL boolean value")).arg(str);
										Tcl_SetResult(interp, (char*)str.latin1(), TCL_VOLATILE);
										delete tclUOpt;
										return TCL_ERROR;
									}
									nu->setNotify((bool)boolVal);
									break;
								case 10:
									nu->setGroup(arg);
									break;
								case 12:
									nu->setEmail(arg);
									break;
								case 13:
									if (Tcl_GetBoolean(interp, arg.latin1(), &boolVal) == TCL_ERROR)
									{
										str = QString(QObject::tr("Option %1 needs proper TCL boolean value")).arg(str);
										Tcl_SetResult(interp, (char*)str.latin1(), TCL_VOLATILE);
										delete tclUOpt;
										return TCL_ERROR;
									}
									nu->setAnonymous(boolVal);
									anon = arg;
									break;
								default:
									Tcl_WrongNumArgs(interp, 0, NULL, addsyntax.local8Bit());
									delete tclUOpt;
									return TCL_ERROR;
							}
						}
						else
						{
							Tcl_WrongNumArgs(interp, 0, NULL, addsyntax.local8Bit());
							delete tclUOpt;
							return TCL_ERROR;
						}
					}
					if (!uin.isNull())
					{
						if (!altNick.isNull())
						{
							userlist.addUser(*nu);
						}
						else if (!anon.isNull())
						{
							userlist.addAnonymous(uin.toInt());
						}
						else
						{
							Tcl_WrongNumArgs(interp, 0, NULL, addsyntax.local8Bit());
							delete tclUOpt;
							return TCL_ERROR;
						}
					}
					else
					{
						Tcl_WrongNumArgs(interp, 0, NULL, addsyntax.local8Bit());
						delete tclUOpt;
						return TCL_ERROR;
					}
				}
				delete nu;
			}
			else
			{
				kadu->addUserAction();
			}
			break;
		}
		case 1:
		{
			if (argc == 3)
			{
				altNick = argv[2];
				if (userlist.containsAltNick(altNick))
				{
					userlist.removeUser(altNick);
				}
				else
				{
					str = QString(QObject::tr("User %1 isn't registered")).arg(altNick);
					Tcl_SetResult(interp, (char*)str.latin1(), TCL_VOLATILE);
					delete tclUOpt;
					return TCL_ERROR;
				}
			}
			else
			{
				Tcl_WrongNumArgs(interp, 0, NULL, remsyntax.local8Bit());
				delete tclUOpt;
				return TCL_ERROR;
			}

			break;
		}
		case 2:
		{
			for (UserList::ConstIterator i = userlist.begin(); i != userlist.end(); i++)
			{
				user = (*i);
				tclMisc->appendTclResult(interp, user.altNick());
			}
			break;
		}
		case 3:
		{
			for (UserList::ConstIterator i = userlist.begin(); i != userlist.end(); i++)
			{
				user = (*i);
				Tcl_AppendElement(interp, (char*)QString::number(user.uin()).latin1());
			}
			break;
		}
		case 4:
		{
			if (argc > 2)
			{
				rest = argc % 2;
				if (!rest)
				{
					Tcl_WrongNumArgs(interp, 0, NULL, chsyntax.local8Bit());
					delete tclUOpt;
					return TCL_ERROR;
				}
				else
				{
					altNick = QString::fromUtf8(argv[2]);
					opt = argv[3];
					uint_uin = altNick.toUInt(&conv);

					bool searchingForAnon = false, foundUser = true;
					if (opt.startsWith("-") && tclUOpt->findIndex(opt) == 13 )
							searchingForAnon = true;

					if (conv == TRUE)
					{
						if (userlist.containsUin(uint_uin))
						{
							_nu = userlist.byUin(uint_uin);
						}
						else
						{
							if (userlist.containsAltNick(altNick))
							{
								_nu = userlist.byAltNick(altNick);
							}
							else
							{
								if ( searchingForAnon )
									foundUser = false;
								else
								{
									str = QString("User UIN isn't registered: %1\nwhile executing:\nuser change %2 %3").arg(altNick).arg(altNick).arg(opt);
									Tcl_SetResult(interp, (char *)str.latin1(), TCL_VOLATILE);
									delete tclUOpt;
									return TCL_ERROR;
								}
							}
						}
					}
					else
					{
						if (userlist.containsAltNick(altNick))
						{
							_nu = userlist.byAltNick(altNick);
						}
						else
						{
							if ( searchingForAnon )
							{
								  foundUser = false;
							}
							else
							{
								str = QString(("User UIN isn't registered: %1\nwhile executing:\nuser change %1 %2")).arg(altNick).arg(opt);
								Tcl_SetResult(interp, (char*)str.latin1(), TCL_VOLATILE);
								delete tclUOpt;
								return TCL_ERROR;
							}
						 }
					}
					altNick = _nu.altNick();

					for (int cnt = 3; argc > cnt; cnt++)
					{
						str = argv[cnt];
						cnt++;
						arg = QString::fromUtf8(argv[cnt]);
						if (str.startsWith("-"))
						{
							id = tclUOpt->findIndex(str);
							switch (id)
							{
								case 0:
									_nu.setUin(arg.toInt());
									break;
								case 1:
									_nu.setAltNick(arg);
									break;
								case 2:
									_nu.setFirstName(arg);
									break;
								case 3:
									_nu.setLastName(arg);
									break;
								case 4:
									_nu.setNickName(arg);
									break;
								case 5:
									_nu.setMobile(arg);
									break;
								case 7:
									if (Tcl_GetBoolean(interp, arg.latin1(), &boolVal) == TCL_ERROR)
									{
										str = QString(QObject::tr("Option %1 needs proper TCL boolean value")).arg(str);
										Tcl_SetResult(interp, (char*)str.latin1(), TCL_VOLATILE);
										delete tclUOpt;
										return TCL_ERROR;
									}
									_nu.setBlocking((bool)boolVal);
									break;
								case 8:
									if (Tcl_GetBoolean(interp, arg.latin1(), &boolVal) == TCL_ERROR)
									{
										str = QString(QObject::tr("Option %1 needs proper TCL boolean value")).arg(str);
										Tcl_SetResult(interp, (char*)str.latin1(), TCL_VOLATILE);
										delete tclUOpt;
										return TCL_ERROR;
									}
									_nu.setOfflineTo((bool)boolVal);
									break;
								case 9:
									if (Tcl_GetBoolean(interp, arg.latin1(), &boolVal) == TCL_ERROR)
									{
										str = QString(QObject::tr("Option %1 needs proper TCL boolean value")).arg(str);
										Tcl_SetResult(interp, (char*)str.latin1(), TCL_VOLATILE);
										delete tclUOpt;
										return TCL_ERROR;
									}
									_nu.setNotify((bool)boolVal);
									break;
								case 10:
									_nu.setGroup(arg);
									break;
								case 12:
									_nu.setEmail(arg);
									break;

								default:
									Tcl_WrongNumArgs(interp, 0, NULL, addsyntax.local8Bit());
									delete tclUOpt;
									return TCL_ERROR;
							}
						}
						else
						{
							Tcl_WrongNumArgs(interp, 0, NULL, chsyntax.local8Bit());
							delete tclUOpt;
							return TCL_ERROR;
						}
					}
					userlist.changeUserInfo(altNick, _nu);
				}
			}
			else
			{
				Tcl_WrongNumArgs(interp, 0, NULL, chsyntax.local8Bit());
				delete tclUOpt;
				return TCL_ERROR;
			}
			break;
		}
		case 5:
		{
			if (argc == 3)
			{
				altNick = QString::fromUtf8(argv[2]);
				uint_uin = altNick.toUInt(&conv);
				if (conv == TRUE)
				{
					if (userlist.containsUin(uint_uin))
					{
						Tcl_SetResult(interp, "1", TCL_STATIC);
					}
					else
					{
						if (userlist.containsAltNick(altNick))
						{
							Tcl_SetResult(interp, "1", TCL_STATIC);
						}
						else
						{
							Tcl_SetResult(interp, "0", TCL_STATIC);
						}
					}
				}
				else
				{
					if (userlist.containsAltNick(altNick))
					{
						Tcl_SetResult(interp, "1", TCL_STATIC);
					}
					else
					{
						Tcl_SetResult(interp, "0", TCL_STATIC);
					}
				}
			}
			else
			{
				Tcl_WrongNumArgs(interp, 0, NULL, exsyntax.local8Bit());
				delete tclUOpt;
				return TCL_ERROR;
			}
			break;
		}
		case 6:
		{
			if (argc == 4)
			{
				altNick = QString::fromUtf8(argv[2]);
				opt = argv[3];
				uint_uin = altNick.toUInt(&conv);

				bool searchingForAnon = false, foundUser = true;
				if (opt.startsWith("-") && tclUOpt->findIndex(opt) == 13 )
						searchingForAnon = true;

				if (conv == TRUE)
				{
					if (userlist.containsUin(uint_uin))
					{
						user = userlist.byUin(uint_uin);
					}
					else
					{
						if (userlist.containsAltNick(altNick))
						{
							user = userlist.byAltNick(altNick);
						}
						else
						{
							if ( searchingForAnon )
								foundUser = false;
							else
							{
								str = QString("User UIN isn't registered: %1\nwhile executing:\nuser get %2 %3").arg(altNick).arg(altNick).arg(opt);
								Tcl_SetResult(interp, (char *)str.latin1(), TCL_VOLATILE);
								delete tclUOpt;
								return TCL_ERROR;
							}
						}
					}
				}
				else
				{
					if (userlist.containsAltNick(altNick))
					{
						user = userlist.byAltNick(altNick);
					}
					else
					{
						if ( searchingForAnon )
							  foundUser = false;
						else
						{
							str = QString(("User UIN isn't registered: %1\nwhile executing:\nuser get %1 %2")).arg(altNick).arg(opt);
							Tcl_SetResult(interp, (char*)str.latin1(), TCL_VOLATILE);
							delete tclUOpt;
							return TCL_ERROR;
						}
					 }
				}
				if (opt.startsWith("-"))
				{
					id = tclUOpt->findIndex(opt);
					switch (id)
					{
						case 0:
							arg = QString::number(user.uin());
							break;
						case 1:
							arg = user.altNick();
							break;
						case 2:
							arg = user.firstName();
							break;
						case 3:
							arg = user.lastName();
							break;
						case 4:
							arg = user.nickName();
							break;
						case 5:
							arg = user.mobile();
							break;
						case 6:
							arg = tclMisc->KaduStatusToTclStatus(user.status().index());
							if (arg.isEmpty())
							{
								tclMisc->setTclResult(interp, QObject::tr("Unknown status type."));
								return TCL_ERROR;
							}
							break;
						case 7:
							arg = QString::number(user.blocking());
							break;
						case 8:
							arg = user.offlineTo();
							break;
						case 9:
							arg = QString::number(user.notify());
							break;
						case 10:
							arg = user.group();
							break;
						case 11:
							arg = user.status().description();
							break;
						case 12:
							arg = user.email();
							break;
						case 13:
							if ( foundUser )
								foundUser = user.isAnonymous();

							arg = QString::number(foundUser);
							break;
						case 14:
							arg = user.dnsName();
							break;
						case 15:
							arg = user.ip().toString();
							break;
						case 16:
							arg = QString::number(user.port());
							break;
						case 17:
							arg = QString::number(user.version());
							break;
						default:
							tclMisc->setTclResult(interp, getsyntax_opts);
							return TCL_ERROR;
					}
					tclMisc->setTclResult(interp, arg);
				}
				else
				{
					Tcl_WrongNumArgs(interp, 0, NULL, getsyntax.local8Bit());
					delete tclUOpt;
					return TCL_ERROR;
				}
			}
			else
			{
				Tcl_WrongNumArgs(interp, 0, NULL, getsyntax.local8Bit());
				delete tclUOpt;
				return TCL_ERROR;
			}
			break;
		}
		default:
			str = QString(QObject::tr("Command should be one of following: %1")).arg(cmds.join(", "));
			Tcl_WrongNumArgs(interp, 0, NULL, str.local8Bit());
			delete tclUOpt;
			return TCL_ERROR;
	}
	delete tclUOpt;
	return TCL_OK;
}

/* Tcl Command ignore
GLOBALSYNTAX: ignore komenda ?argumenty?
DESCRIPTION:
Komenda suca manipulowaniu list ignorowanych.

SUBSYNTAX: ignore add ?UIN ?UIN ?...??? ?lista UIN'w ?lista UIN'w ?...???
DESCRIPTION:
Dodaje podane UIN'y, lub listy UIN'w do listy ignorowanych.

SUBSYNTAX: ignore remove ?UIN ?UIN ?...??? ?lista UIN'w ?lista UIN'w ?...???
DESCRIPTION:
Usuwa podane UIN'y, lub listy UIN'w z listy ignorowanych.

SUBSYNTAX: ignore exists ?UIN ?UIN ?...??? ?lista UIN'w ?lista UIN'w ?...???
DESCRIPTION:
Sprawdza czy podany UIN, lub lista UIN'w jest dodana do listy ignorowanych. Jeli tak, zwraca 1, inaczej zwraca 0.

SUBSYNTAX: ignore list
DESCRIPTION:
Zwraca liste pozycji ignorowanych, w ktrej mog si znale pojedyncze UIN'y, jak i listy UIN'w.
*/
int Tcl_CmdIgnore(void* clientData, Tcl_Interp *interp, int argc, const char *argv[])
{
	if (argc < 2)
	{
		Tcl_WrongNumArgs(interp, 0, NULL, "ignore command ?arguments?");
		return TCL_ERROR;
	}
	QString cmd = argv[1], arg;
	QStringList cmds = QStringList::split(" ", "add remove exists list"), args;
	UinsList igList, ig_List, emptyList, uins;
	  bool conv;
	uint uin;
	uint i, j;
	Tcl_DString ds;

	switch (cmds.findIndex(cmd))
	{
		case 0:
		{
			for (int cnt = 2; cnt < argc; cnt++)
			{
				arg = argv[cnt];

				// Sprawdzanie, czy dany argument to lista UIN'ow czy pojedynczy UIN
				if (arg.find(" ") > -1)
				{
					args = QStringList::split(" ", arg);
					ig_List = emptyList;

					// Dla kazdego UIN'a z listy sprawdzanie, czy jest poprawnym UIN'em.
					for ( QStringList::Iterator it = args.begin(); it != args.end(); ++it )
					{
						uin = (*it).toUInt(&conv);
						if (conv)
						{
							if (!ig_List.findIndex(uin) > -1)
							{
								// Dodawanie UIN'a do listy, aby sie nie powtorzyl
								ig_List.append(uin);
							}
						}
						else
						{
							Tcl_WrongNumArgs(interp, 0, NULL, "ignore add ?UIN ?UIN ?...??? ?UINs list ?UINs list ?...???");
							return TCL_ERROR;
						}
					}
					if (!isIgnored(ig_List))
						addIgnored(ig_List);

				}
				else
				{

					// Sprawdzanie czy podany UIN jest poprawny.
					uin = arg.toUInt(&conv);
					if (conv)
					{
						if (!igList.findIndex(uin) > -1)
						{
							// Deklarowanie uins, aby dodawany UIN byl lista
							uins.append(uin);
							if (!isIgnored(uins))
								addIgnored(uins);
						}
					}
					else
					{
						Tcl_WrongNumArgs(interp, 0, NULL, "ignore add ?UIN ?UIN ?...??? ?UINs list ?UINs list ?...???");
						return TCL_ERROR;
					}
				}
			}
			break;
		}
		case 1:
		{
			for (int cnt = 2; cnt < argc; cnt++)
			{
				arg = argv[cnt];

				// Sprawdzanie, czy dany argument to lista UIN'ow czy pojedynczy UIN
				if (arg.find(" ") > -1)
				{
					args = QStringList::split(" ", arg);
					ig_List = emptyList;

					// Dla kazdego UIN'a z listy sprawdzanie, czy jest poprawnym UIN'em.
					for ( QStringList::Iterator it = args.begin(); it != args.end(); ++it )
					{
						uin = (*it).toUInt(&conv);
						if (conv)
						{
							if (!ig_List.findIndex(uin) > -1)
							{
								// Dodawanie UIN'a do listy, aby sie nie powtorzyl
								ig_List.append(uin);
							}
						}
						else
						{
							Tcl_WrongNumArgs(interp, 0, NULL, "ignore add ?UIN ?UIN ?...??? ?UINs list ?UINs list ?...???");
							return TCL_ERROR;
						}
					}
					if (ignored.findIndex(ig_List) > -1)
						delIgnored(ig_List);

				}
				else
				{
					uin = arg.toUInt(&conv);
					if (conv)
					{
						uins.append(uin);
						if (ignored.findIndex(uins) > -1)
							delIgnored(uins);

					}
					else
					{
						Tcl_WrongNumArgs(interp, 0, NULL, "ignore remove ?UIN ?UIN ?...??? ?UINs list ?UINs list ?...???");
						return TCL_ERROR;
					}
				}
			}
			break;
		}
		case 2:
		{
			if (argc > 3)
			{
				Tcl_WrongNumArgs(interp, 0, NULL, "ignore exists ?UIN? ?UINs list?");
				return TCL_ERROR;
			}
			else
			{
				arg = argv[2];
				uin = arg.toUInt(&conv);
				if (!conv)
				{
					Tcl_WrongNumArgs(interp, 0, NULL, "ignore exists ?UIN? ?UINs list?");
					return TCL_ERROR;
				}
				uins.append(uin);
				if (ignored.findIndex(uins) > -1)
				{
					Tcl_SetResult(interp, "1", TCL_STATIC);
				}
				else
				{
					Tcl_SetResult(interp, "0", TCL_STATIC);
				}
			}
			break;
		}
		case 3:
		{
			Tcl_DStringInit(&ds);
			for (i = 0; i < ignored.count(); i++)
			{
				Tcl_DStringStartSublist(&ds);
				for (j = 0; j < ignored[i].count(); j++)
					Tcl_DStringAppendElement(&ds, (char*)QString::number(ignored[i][j]).latin1() );

				Tcl_DStringEndSublist(&ds);
			}
			Tcl_DStringResult(interp, &ds);
			Tcl_DStringFree(&ds);
			break;
		}
		default:
			arg = QString(QObject::tr("Command should be one of following: %1")).arg(cmds.join(", "));
			Tcl_WrongNumArgs(interp, 0, NULL, arg.local8Bit());
			return TCL_ERROR;
	}
	return TCL_OK;
}

/* Tcl Command play
SYNTAX: play ?-force? plik
DESCRIPTION:
Odtwarza plik muzyczny. Jeli plik nie istnieje, nic nie zostanie odtworzone.
Jeli zostanie uyta opcja <B>-force</B>, to dwik zostanie odtworzony
bez wzgldu na to, czy dwieki w Kadu s wyczone, czy nie.
*/
int Tcl_CmdPlay(void* clientData, Tcl_Interp *interp, int argc, const char *argv[])
{
	if (argc > 3)
	{
		Tcl_WrongNumArgs(interp, 0, NULL, "play ?-force? file");
		return TCL_ERROR;
	}
	bool force = FALSE;
	int arg;
	if (argc == 3)
	{
		if (!QString::compare(argv[1], "-force"))
			force = TRUE;
		else
		{
			Tcl_WrongNumArgs(interp, 0, NULL, "play ?-force? file");
			return TCL_ERROR;
		}
		arg = 2;
	}
	else
		arg = 1;

	QString file = QString::fromUtf8(argv[arg]);

	notify->emitMessage("TCL", "Sound", file);
	return TCL_OK;
}

/* Tcl Command configfile
GLOBALSYNTAX: configfile komenda ?plik? grupa zmienna ?warto?
DESCRIPTION:
Suy do odczytu i zapisu zmiennych w pliku konfiguracyjnym Kadu,
lub gdy podany jest argument <I>pliku</I>, to zapis/odczyt dokonywany jest
do/z podanego pliku.

SUBSYNTAX: configfile read ?plik? grupa zmienna
DESCRIPTION:
Czyta z pliku konfiguracyjnego, spod podanej <I>grupy</I> ustawie, dan <I>zmienn</I>.

SUBSYNTAX: configfile write ?plik? grupa zmienna warto
DESCRIPTION:
Zapisuje w pliku konfiguracyjnym <I>zmienn</I>, pod podan <I>grup</I>,
przyporzdkowyjc jej dan warto.
*/
int Tcl_CmdConfigFile(void* clientData, Tcl_Interp *interp, int argc, const char *argv[])
{
	if (argc < 2)
	{
		Tcl_WrongNumArgs(interp, 0, NULL, "configfile command arguments");
		return TCL_ERROR;
	}
	QStringList cmds = QStringList::split(" ", "read write");
	QString act = argv[1], group, var, res, value;
	QCString utf;

	switch (cmds.findIndex(act))
	{
		case 0:
		{
			if (argc < 4 || argc > 5)
			{
				Tcl_WrongNumArgs(interp, 0, NULL, "configfile read ?file? group variable");
				return TCL_ERROR;
			}

			if (argc == 4)
			{
				group = QString::fromUtf8(argv[2]);
				var = QString::fromUtf8(argv[3]);
				res = config_file.readEntry(group, var);
			}
			else
			{
				group = QString::fromUtf8(argv[3]);
				var = QString::fromUtf8(argv[4]);
				ConfigFile config(QString::fromUtf8(argv[2]));
				res = config.readEntry(group, var);
			}
			tclMisc->setTclResult(interp, res);
			break;
		}
		case 1:
		{
			if (argc < 5 || argc > 6)
			{
				Tcl_WrongNumArgs(interp, 0, NULL, "configfile write ?file? group variable value");
				return TCL_ERROR;
			}
			if (argc == 4)
			{
				group = QString::fromUtf8(argv[2]);
				var = QString::fromUtf8(argv[3]);
				value = QString::fromUtf8(argv[4]);
				config_file.writeEntry(group, var, value);
			}
			else
			{
				group = QString::fromUtf8(argv[2]);
				var = QString::fromUtf8(argv[3]);
				value = QString::fromUtf8(argv[4]);
				ConfigFile config(QString::fromUtf8(argv[2]));
				config.writeEntry(group, var, value);
				config.sync();
			}
			break;
		}
		default:
			res = QString(QObject::tr("Command should be one of following: %1")).arg(cmds.join(", "));
			Tcl_WrongNumArgs(interp, 0, NULL, res.local8Bit());
			return TCL_ERROR;
	}
	return TCL_OK;
}

/* Tcl Command chat
GLOBALSYNTAX: chat komenda ?argumenty?
DESCRIPTION:
Komenda chat suy do operacji na oknach rozmw.

SUBSYNTAX: chat open UIN/UINy
DESCRIPTION:
Otwiera okno rozmowy/konferencij dla podanego <I>UINu</I> lub <I>UINw</I>. Jeli okno istnieje, zwracany
jest bd.

SUBSYNTAX: chat close UIN/UINy
DESCRIPTION:
Zamyka okno rozmowy/konferencji z podanym <I>UINem</I> lub <I>UINami</I>. Jeli okno nie istnieje, zwracany
jest bd.

SUBSYNTAX: chat exists UIN/UINy
<DD><br><p>
DESCRIPTION:
Zwraca 1, jeli okno rozmowy/konferencji z podanym <I>UINem</I> lub <I>UINami</I> istnieje, w przeciwnym wypadku
zwraca 0.

SUBSYNTAX: chat input UIN/UINy ?-html? ?warto?
DESCRIPTION:
Jeli podana jest warto, to pole wiadomoci w oknie rozmowy z podanym
<I>UINem</I>, bd <I>list UINw</I> (konferencja) bdzie t warto zawierao. Jeli zostanie
ona opuszczona, to aktualna zawarto tego pola zostanie zwrcona. Jeli okno rozmowy z podanym <I>UINem</I>,
czy <I>list UINw</I> nie istnieje, zostanie zwrcony bd. Jeli nie ma pewnoci, e okno istnieje, naley
posuy si komend chat exists. Gdy podana zostanie opcja -html, to w przypadku zwracania zawartoci
pola rozmowy, zwrcone te zostan tagi HTML (jak pogrubienie, etc.), a w przypadku nadawania nowej wartoci
temu polu, wszelkie tagi HTML zostan zinterpretowane, zamiast wstawione, jako zwyky tekst.
<B>WANE:</B> Jeli pobierasz tekst z pola edycji, zmieniasz w nim co, a nastpnie wstawiasz z powrotem,
to lepiej, aby operowa na tekcie pobranym i wstawianym z opcj -html, w przeciwnym wypadu znaki nowej linii
w polu edycji znikn (s one znakami HTML).

SUBSYNTAX: chat addtext UIN/UINy ?opcje? tekst
DESCRIPTION:
Dodaje podany <I>tekst</I> do okna rozmowy z <I>UINem</I>, a dokladniej
do listy wywietlanych wiadomoci.<br><br>
Opcje:<br>
<DT>
<P><DT><B>-formatted</B> <I>wysyajcy</I><DD>
Formatuje wiadomo tak, jakby nadesza ona od <I>wysyajcego</I>. Wystpuje specjalny przypadek,
gdy <I>wysyajcy</I> jest identyczny z nazw lokalnego uytkownika Kadu (odczytan z pliku konfiguracyjnego
- zmienna Nick z sekcji General) - wtedy wiadomo formatowana jest tak, jakby zostaa wysana
przez lokalnego uytkownika.<br>
<P><DT><B>-addhistory</B><DD>
Powoduje zapisanie dodawanej wiadomoci w historii rozmowy z podanymi
<I>UINami</I>. Mona uywa znacznikw HTML.<br>
<P><DT><B>-color</B> <I>kolor</I><DD>
Ustawia kolor czcionki dodawanego tekstu na <I>kolor</I>. Domylnie jest to czarny dla wszystkich przypadkw.<br>
<P><DT><B>-bgcolor</B> <I>kolor</I><DD>
Ustawia kolor ta dodawanego tekstu na <I>kolor</I>. Domylnie jest to biay dla
niesformatowanego tekstu. Dla sformatowanego domylnie przyjmuje kolor odpowiedni
dla wiadomoci wysanej lub odebranej.<br><br>

SUBSYNTAX: chat clear UIN/UINy
DESCRIPTION:
Czyci wszystkie wiadomoci z okna rozmowy z <I>UINem</I>.

SUBSYNTAX: chat list
DESCRIPTION:
Zwraca list UINw, z ktrymi otwarte s okna rozmw.

SUBSYNTAX: chat curpos UIN/UINy ?pozX ?pozY??
DESCRIPTION:
Jeli podane s <I>pozycje X i Y</I>, to ustawia kursor w oknie rozmowy z <I>UINem</I> bd
<I>UINami</I> na podan pozycj. Jeli natomiast argumenty te nie s podane, zwraca aktualn
pozycj kursora, w postaci listy 2 wyrazw.<br>Dopuszczalne s specjalne indeksy pozycji, jak:
<B>end</B>, <B>lineend</B>, <B>linestart</B>, <B>lineup</B>, <B>linedown</B>, <B>nextword</B>,
oraz <B>prevword</B>. Wystpuj one jako pojedyncze indeksy (nie podaje si wtedy argumentu
<I>pozY</I>). Oto ich opis:<br>
<B>end</B> przesunie kursor na koniec tekstu<br>
<B>lineend</B> przesunie kursor na konie bierzcej linii<br>
<B>linestart</B> przesunie kursor na pocztek bierzcej linii<br>
<B>nextword</B> przesunie kursor na pocztek nastpnego sowa<br>
<B>prevword</B> przesunie kursor na pocztek poprzedniego sowa, lub bierzcego sowa, jeli
pocztkowo nie by na pocztku tego sowa<br><br>
Jest jeszcze jedna metoda okrelania pozycji. Mona to zrobi za pomoc przesuwania kursora
relatywnie, wzgldem bierzcego miejsca. Robi si to za pomoc podawania indeksw w postaci
<B>+N</B>, oraz <B>-N</B>, gdzie N to liczba naturalna, oznaczajca liczb wierszy do przesunicia
w lewo, prawo, gr, czy d. Mona poda obydwa argumenty pozycji, tj.
<code>chat curpos 1234567 +5 -2</code> co spowoduje przesunicie kursora o 5 znakw w prawo,
oraz o 2 linie w gr. Mona te poda tylko pierwszy argument, aby przesun kursor o dan liczb
znakw w lewo, bd prawo, pomijajc linie. Mona te jako pierwszy argument <I>pozX</I> poda
zwyk, dowoln liczb, natomiast za argument <I>pozY</I> poda <B>+/-N</B> aby przesun kursor
jedynie w <B>N</B> linii w gr bd, d, poniewa w sytuacji gdy ktrykolwiek z indeksw pozycji
ma posta <B>+/-N</B>, to drugi argument musi mie t sam posta, lub w przeciwnym wypadku nie bdzie
w ogle brany pod uwag.

EXAMPLE:
chat curpos 1234567 10 3
chat curpos 1234567 endline
chat curpos 1234567 -5 +1
chat curpos 1234567 +1
chat curpos 1234567 0 +1
chat curpos 1234567 prevword
ENDOFEXAMPLE

SUBSYNTAX: chat selection UIN/UINy ?startPozX startPozY koniecPozX koniecPozY?
DESCRIPTION:
Jeli podane s <I>pozycje pocztkowe X i Y</I> oraz <I>kocowe X i Y</I> to zaznaczony zostaje
tekst w oknie rozmowy z <I>UINem</I> bd <I>UINami</I>, poczwszy od pozycji okrelonej dwoma
pierwszymi argumentami, a skonczywszy na pozycji okrelonej ostatnimi dwoma. Jeli <I>pozycje</I>
nie zostan podane, zwracana jest 4-wyrazowa lista zawierajca ww. pozycje dla bierzcego tekstu
zaznaczonego. Jeli aden tekst nie jest zaznaczony, wszystkie zwrcone liczby bda mie warto -1.

SUBSYNTAX: chat getselection UIN/UINy
DESCRIPTION:
Jeli jest zaznaczony jakikolwiek tekst w oknie rozmowy z <I>UINem</I> bd <I>UINami</I>,
to zostaje on zwrcony. W przeciwnym wypadku pusty acuch jest zwracany.

SUBSYNTAX: chat raise UIN/UINy
DESCRIPTION:
Przywouje otwarte oknorozmowy  na pierwszy plan, oraz sprawia e tytu okna miga si, a do momentu,
a uytkownik kliknie na tym oknie. Jeli okno nie istnieje, zwraca bd.

SUBSYNTAX: chat minimize UIN/UINy
DESCRIPTION:
Minimalizuje okno rozmowy. Jeli okno nie istnieje, zwraca bd.

SUBSYNTAX: chat restore UIN/UINy
DESCRIPTION:
Przywraca okno rozmowy uprzednio zminimalizowane. Jeli okno nie istnieje, zwraca bd.

SUBSYNTAX: chat maximize UIN/UINy
DESCRIPTION:
Maksymalizuje rozmiar okna rozmowy. Jeli okno nie istnieje, zwraca bd.

SUBSYNTAX: chat fixemotspath
DESCRIPTION:
Zwraca sciezke wskazujaca na katalog glowny /. Komenda ta jest potrzebna, gdy chcemy doda jaki
obrazek przy pomocy komendy <B>chat addtext</B>, jako e uycie tam <font face=courier>
&lt;img src="..."&gt;</font> spowoduje prb odczytu podanego obrazka spod cieki
/katalog/instalacji/kadu/share/themes/emoticons/styl-emotikon/. Komenda <B>chat fixemotspath</B>
zwraca odpowiedni ancuch <font face=courier>../../..</font> (powtrzone odpowiedni liczb razy),
ktry uyty jako pocztek cieki dla obrazka, umoliwia podanie cieki do obrazka od katalogu
gwnego.

EXAMPLE:
chat addtext 35234332 "&lt;img src=\"[chat fixemotspath]/home/user/.gg/blablabla.png\"&gt;&lt;br&gt;"
ENDOFEXAMPLE
*/
int Tcl_CmdChat(void* clientData, Tcl_Interp *interp, int argc, const char *argv[])
{
	if (argc < 2)
	{
		Tcl_WrongNumArgs(interp, 0, NULL, "chat command ?arguments?");
		return TCL_ERROR;
	}
	QStringList acts = QStringList::split(" ", "input open close exists list curpos selection addtext raise minimize restore maximize fixemotspath clear getselection addbutton delbutton"), lst;
	QString act = argv[1], uin;
	UinsList uins;
	uint i, j;
	int x, y, x2, y2;
	QString noSuchWin = QObject::tr("No such chat window."),
			winAlrEx = QObject::tr("Chat window already exists.");

	switch (acts.findIndex(act))
	{
		case 0:
		{
			if (argc < 3 || argc > 6)
			{
				Tcl_WrongNumArgs(interp, 0, NULL, "chat input UIN/UINsList ?-html? ?value?");
				return TCL_ERROR;
			}
			uin = argv[2];
			if (uin.find(" ") > -1)
			{
				QStringList list = QStringList::split(" ", uin);
				for ( QStringList::iterator it = list.begin(); it != list.end(); it++ )
				{
					uins.append((*it).toUInt());
				}
			}
			else
			{
				uins.append(uin.toUInt());
			}
			if (chat_manager->findChatByUins(uins))
			{
				bool put = FALSE, html = FALSE;
				int textArg = 3;
				if (argc > 3)
				{
					if (argc == 5)
					{
						if (QString::compare(argv[3], "-html"))
						{
							Tcl_WrongNumArgs(interp, 0, NULL, "chat input UIN/UINsList ?-html? ?value?");
							return TCL_ERROR;
						}
						html = TRUE;
						put = TRUE;
						textArg++;
					}
					else
					{
						if (!QString::compare(argv[3], "-html"))
							html = TRUE;
						else
							put = TRUE;
					}
				}

				if (put)
				{
					HtmlDocument doc, putDoc;
					Chat* c = chat_manager->findChatByUins(uins);
					QString start, end;

					doc.parseHtml(c->edit->text());
					int i, tostop = doc.countElements() - 5;
					if (doc.countElements() > 0)
					{
						for ( i = 0; i < 7; i++ )
							start.append(doc.elementText(i));

						for ( i = tostop; i < doc.countElements(); i++ )
							end.append(doc.elementText(i));

						if (html)
						{
							start.append(QString::fromUtf8(argv[textArg]));
							start.append(end);
							putDoc.parseHtml(start);
						}
						else
						{
							putDoc.addTag(start);
							putDoc.addText(QString::fromUtf8(argv[textArg]));
							putDoc.addTag(end);
						}
					}
					else
					{
						if (html)
							putDoc.parseHtml(QString::fromUtf8(argv[textArg]));
						else
							putDoc.addText(QString::fromUtf8(argv[textArg]));
					}
					chat_manager->findChatByUins(uins)->edit->setText(putDoc.generateHtml());
				}
				else
				{
					HtmlDocument doc;
					Chat* c = chat_manager->findChatByUins(uins);
					QCString str;
					Tcl_DString ds;
					Tcl_DStringInit(&ds);
					Tcl_Obj* result;

					doc.parseHtml(c->edit->text());
					int stop = doc.countElements() - 5;
					if (html)
					{
						for ( int i = 7; i < stop; i++ )
						{
							if (!doc.isTagElement(i))
								str = doc.elementText(i).replace("<", "&lt;").replace(">", "&gt;").utf8();
							else
								str = doc.elementText(i).utf8();

							result = Tcl_NewStringObj((const char* const)str, str.length());
							Tcl_DStringAppend(&ds, Tcl_GetString(result), str.length());
						}
					}
					else
					{
						for ( int i = 7; i < stop; i++ )
						{
							if (!doc.isTagElement(i))
							{
								str = doc.elementText(i).utf8();
								result = Tcl_NewStringObj((const char* const)str, str.length());
								Tcl_DStringAppend(&ds, Tcl_GetString(result), str.length());
							}
						}
					}
					Tcl_DStringResult(interp, &ds);
					Tcl_DStringFree(&ds);
				}
			}
			else
			{
				tclMisc->setTclResult(interp, noSuchWin);
				return TCL_ERROR;
			}
			break;
		}
		case 1:
		{
			if (argc != 3)
			{
				Tcl_WrongNumArgs(interp, 0, NULL, "chat open UIN/UINsList");
				return TCL_ERROR;
			}
			uin = argv[2];
			if (uin.find(" ") > -1)
			{
				printf("a1\n");
				QStringList list = QStringList::split(" ", uin);
				printf("a2\n");
				for ( uint i = 0; i < list.count(); i++ )
				{
				printf("a3: %s\n", (char*)list[i].latin1());
					uins.append((UinType)list[i].toUInt());
				printf("a4\n");
				}
			}
			else
			{
				printf("a-1\n");
				uins.append(uin.toUInt());
			}
				printf("a5\n");
			if (chat_manager->findChatByUins(uins))
			{
				printf("a6\n");
				tclMisc->setTclResult(interp, winAlrEx);
				printf("a7\n");
				return TCL_ERROR;
			}
			else
			{
				printf("a8\n");
				chat_manager->openPendingMsgs(uins);
				// , QDateTime::currentDateTime().toTime_t()
				printf("a9\n");
			}
			break;
		}
		case 2:
		{
			if (argc != 3)
			{
				Tcl_WrongNumArgs(interp, 0, NULL, "chat close UIN/UINsList");
				return TCL_ERROR;
			}
			uin = argv[2];
			if (uin.find(" ") > -1)
			{
				QStringList list = QStringList::split(" ", uin);
				for ( QStringList::iterator it = list.begin(); it != list.end(); it++ )
				{
					uins.append((*it).toUInt());
				}
			}
			else
			{
				uins.append(uin.toUInt());
			}

			if (chat_manager->findChatByUins(uins))
			{
				chat_manager->findChatByUins(uins)->close(TRUE);
			}
			else
			{
				tclMisc->setTclResult(interp, noSuchWin);
				return TCL_ERROR;
			}
			break;
		}
		case 3:
		{
			if (argc != 3)
			{
				Tcl_WrongNumArgs(interp, 0, NULL, "chat exists UIN/UINsList");
				return TCL_ERROR;
			}
			uin = argv[2];
			if (uin.find(" ") > -1)
			{
				QStringList list = QStringList::split(" ", uin);
				for ( QStringList::iterator it = list.begin(); it != list.end(); it++ )
				{
					uins.append((*it).toUInt());
				}
			}
			else
			{
				uins.append(uin.toUInt());
			}
			if (chat_manager->findChatByUins(uins))
			{
				Tcl_SetResult(interp, "1", TCL_STATIC);
			}
			else
			{
				Tcl_SetResult(interp, "0", TCL_STATIC);
			}
			break;
		}
		case 4:
		{
			if (argc != 2)
			{
				Tcl_WrongNumArgs(interp, 0, NULL, "chat list");
				return TCL_ERROR;
			}
			for ( i = 0; chat_manager->chats().count() > i; i++ )
			{
				lst.clear();
				for ( j = 0; chat_manager->chats()[i]->uins().count() > j; j++ )
				{
					lst.append(QString::number( chat_manager->chats()[i]->uins()[j] ));
				}
				Tcl_AppendElement(interp, lst.join(" ").local8Bit());
			}
			break;
		}
		case 5:
		{
			if (argc < 3 || argc > 5)
			{
				Tcl_WrongNumArgs(interp, 0, NULL, "chat curpos UIN/UINsList ?posX ?posY??");
				return TCL_ERROR;
			}
			uin = argv[2];
			if (uin.find(" ") > -1)
			{
				QStringList list = QStringList::split(" ", uin);
				for ( QStringList::iterator it = list.begin(); it != list.end(); it++ )
				{
					uins.append((*it).toUInt());
				}
			}
			else
			{
				uins.append(uin.toUInt());
			}
			if (chat_manager->findChatByUins(uins))
			{
				QString tmp;
				if (argc > 3)
				{
					if (argc == 4)
					{
						QTextEdit::CursorAction pos = tclMisc->tclIndexToQtIndex(argv[3]);
						if (pos == QTextEdit::MoveHome)
						{
							QString xpos(argv[3]);
							bool conv;
							uint chars = xpos.mid(1).toUInt(&conv);
							if (!conv)
							{
								tclMisc->setTclResult(interp, QObject::tr("Wrong position index '%1'.").arg(xpos));
								return TCL_ERROR;
							}
							if (xpos.startsWith("+"))
							{
								for ( uint i = 0; i < chars; i++ )
									chat_manager->findChatByUins(uins)->edit->moveCursor(QTextEdit::MoveForward, 0);

								return TCL_OK;
							}
							else if (xpos.startsWith("-"))
							{
								for ( uint i = 0; i < chars; i++ )
									chat_manager->findChatByUins(uins)->edit->moveCursor(QTextEdit::MoveBackward, 0);

								return TCL_OK;
							}
							else
							{
								tclMisc->setTclResult(interp, QObject::tr("Wrong position index '%1'.").arg(xpos));
								return TCL_ERROR;
							}
						}
						else
							chat_manager->findChatByUins(uins)->edit->moveCursor(pos, 0);
					}
					else
					{
						bool conv, notCoords = FALSE;
						QString xpos(argv[3]), ypos(argv[4]);

						x = xpos.toInt(&conv);
						if (xpos.startsWith("+") || xpos.startsWith("-"))
						{
							notCoords = TRUE;
							uint chars = xpos.mid(1).toUInt(&conv);
							if (!conv)
							{
								tclMisc->setTclResult(interp, QObject::tr("Wrong position index '%1'.").arg(xpos));
								return TCL_ERROR;
							}
							if (xpos.startsWith("+"))
							{
								for ( uint i = 0; i < chars; i++ )
									chat_manager->findChatByUins(uins)->edit->moveCursor(QTextEdit::MoveForward, 0);
							}
							else if (xpos.startsWith("-"))
							{
								for ( uint i = 0; i < chars; i++ )
									chat_manager->findChatByUins(uins)->edit->moveCursor(QTextEdit::MoveBackward, 0);
							}
							else
							{
								tclMisc->setTclResult(interp, QObject::tr("Wrong position index '%1'.").arg(xpos));
								return TCL_ERROR;
							}
						}

						y = ypos.toInt(&conv);
						if (ypos.startsWith("+") || ypos.startsWith("-"))
						{
							notCoords = TRUE;
							uint chars = ypos.mid(1).toUInt(&conv);
							if (!conv)
							{
								tclMisc->setTclResult(interp, QObject::tr("Wrong position index '%1'.").arg(ypos));
								return TCL_ERROR;
							}
							if (ypos.startsWith("+"))
							{
								for ( uint i = 0; i < chars; i++ )
									chat_manager->findChatByUins(uins)->edit->moveCursor(QTextEdit::MoveDown, 0);
							}
							else if (ypos.startsWith("-"))
							{
								for ( uint i = 0; i < chars; i++ )
									chat_manager->findChatByUins(uins)->edit->moveCursor(QTextEdit::MoveUp, 0);
							}
							else
							{
								tclMisc->setTclResult(interp, QObject::tr("Wrong position index '%1'.").arg(ypos));
								return TCL_ERROR;
							}
						}

						if (!notCoords)
							chat_manager->findChatByUins(uins)->edit->setCursorPosition(y, x);
					}
				}
				else
				{
					chat_manager->findChatByUins(uins)->edit->getCursorPosition(&y, &x);
					Tcl_AppendElement(interp, (char*)QString::number(x).latin1());
					Tcl_AppendElement(interp, (char*)QString::number(y).latin1());
				}
			}
			else
			{
				tclMisc->setTclResult(interp, noSuchWin);
				return TCL_ERROR;
			}
			break;
		}
		case 6:
		{
			if (argc != 3 && argc != 7)
			{
				Tcl_WrongNumArgs(interp, 0, NULL, "chat selection UIN/UINsList ?startPosX startPosY endPosX endPosY?");
				return TCL_ERROR;
			}
			uin = argv[2];
			if (uin.find(" ") > -1)
			{
				QStringList list = QStringList::split(" ", uin);
				for ( QStringList::iterator it = list.begin(); it != list.end(); it++ )
				{
					uins.append((*it).toUInt());
				}
			}
			else
			{
				uins.append(uin.toUInt());
			}
			if (chat_manager->findChatByUins(uins))
			{
				QString tmp;
				if (argc == 7)
				{
					if (!Tcl_GetInt(interp, argv[3], &x))
						return TCL_ERROR;

					if (!Tcl_GetInt(interp, argv[4], &y))
						return TCL_ERROR;

					if (!Tcl_GetInt(interp, argv[5], &x2))
						return TCL_ERROR;

					if (!Tcl_GetInt(interp, argv[6], &y2))
						return TCL_ERROR;

					chat_manager->findChatByUins(uins)->edit->setSelection(y, x, y2, x2);
				}
				else
				{
					chat_manager->findChatByUins(uins)->edit->getSelection(&y, &x, &y2, &x2);
					Tcl_AppendElement(interp, (char*)QString::number(x).latin1());
					Tcl_AppendElement(interp, (char*)QString::number(y).latin1());
					Tcl_AppendElement(interp, (char*)QString::number(x2).latin1());
					Tcl_AppendElement(interp, (char*)QString::number(y2).latin1());
				}
			}
			else
			{
				tclMisc->setTclResult(interp, noSuchWin);
				return TCL_ERROR;
			}
			break;
		}
		case 7:
		{
			if (argc < 4)
			{
				Tcl_WrongNumArgs(interp, 0, NULL, "chat addtext UIN ?-formatted sender? ?-addhistory? text");
				return TCL_ERROR;
			}
			uin = argv[2];
			if (uin.find(" ") > -1)
			{
				QStringList list = QStringList::split(" ", uin);
				for ( QStringList::iterator it = list.begin(); it != list.end(); it++ )
				{
					uins.append((*it).toUInt());
				}
			}
			else
			{
				uins.append(uin.toUInt());
			}
			if (chat_manager->findChatByUins(uins))
			{
				QString tmp, sender;
				bool formated = FALSE, addhistory = FALSE,
					 background = FALSE, foreground = FALSE;
				QColor bg, fg;
				uint arg = 3, optc = argc-1;
				if (argc > 4)
				{
					QStringList opts = QStringList::split(" ", "-formatted -addhistory -bgcolor -color");
					for ( uint i = 3; i < optc; i++ )
					{
						switch (opts.findIndex(argv[i]))
						{
							case 0:
							{
								formated = TRUE;
								i++;
								arg++;
								if (i >= optc)
								{
									tclMisc->setTclResult(interp, QObject::tr("-formatted option needs a value argument."));
									return TCL_ERROR;
								}
								sender = QString::fromUtf8(argv[i]);
								break;
							}
							case 1:
							{
								addhistory = TRUE;
								break;
							}
							case 2:
							{
								background = TRUE;
								i++;
								arg++;
								if (i >= optc)
								{
									tclMisc->setTclResult(interp, QObject::tr("-bgcolor option needs a value argument."));
									return TCL_ERROR;
								}
								bg = argv[i];
								break;
							}
							case 3:
							{
								foreground = TRUE;
								i++;
								arg++;
								if (i >= optc)
								{
									tclMisc->setTclResult(interp, QObject::tr("-color option needs a value argument."));
									return TCL_ERROR;
								}
								fg = argv[i];
								break;
							}
							default:
								tclMisc->setTclResult(interp, QObject::tr("Option should be one of following: %1").arg(opts.join(" ")));
								return TCL_ERROR;
						}
						arg++;
					}
				}

				tmp = QString::fromUtf8(argv[arg]);
				ChatMessage *tmpmsg;
				QValueList<ChatMessage*> messages;

				if (addhistory)
				{
					if (config_file.readBoolEntry("General","Logging"))
						history.appendMessage(uins, uins[0], tmp, TRUE);
				}

				if (formated)
				{
					QString myNick = config_file.readEntry("General", "Nick");
					if (!sender.compare("*") || !sender.compare(myNick))
						tmpmsg = new ChatMessage(myNick, tmp, TRUE, QDateTime::currentDateTime());
					else
						tmpmsg = new ChatMessage(sender, tmp, FALSE, QDateTime::currentDateTime());

					chat_manager->findChatByUins(uins)->formatMessage(*tmpmsg);
				}
				else
					tmpmsg = new ChatMessage(tmp);

				if (background)
					tmpmsg->backgroundColor = bg;

				if (foreground)
					tmpmsg->textColor = fg;

				messages.append(tmpmsg);
				chat_manager->findChatByUins(uins)->scrollMessages(messages);
			}
			else
			{
				tclMisc->setTclResult(interp, noSuchWin);
				return TCL_ERROR;
			}
			break;
		}
		case 8:
		{
			if (argc != 3)
			{
				Tcl_WrongNumArgs(interp, 0, NULL, "chat raise UIN/UINy");
				return TCL_ERROR;
			}
			uin = argv[2];
			if (uin.find(" ") > -1)
			{
				QStringList list = QStringList::split(" ", uin);
				for ( QStringList::iterator it = list.begin(); it != list.end(); it++ )
				{
					uins.append((*it).toUInt());
				}
			}
			else
			{
				uins.append(uin.toUInt());
			}
			if (chat_manager->findChatByUins(uins))
			{
				chat_manager->findChatByUins(uins)->raise();
			}
			else
			{
				tclMisc->setTclResult(interp, noSuchWin);
				return TCL_ERROR;
			}
			break;
		}
		case 9:
		{
			if (argc != 3)
			{
				Tcl_WrongNumArgs(interp, 0, NULL, "chat minimize UIN/UINy");
				return TCL_ERROR;
			}
			uin = argv[2];
			if (uin.find(" ") > -1)
			{
				QStringList list = QStringList::split(" ", uin);
				for ( QStringList::iterator it = list.begin(); it != list.end(); it++ )
				{
					uins.append((*it).toUInt());
				}
			}
			else
			{
				uins.append(uin.toUInt());
			}

			if (chat_manager->findChatByUins(uins))
			{
				chat_manager->findChatByUins(uins)->showMinimized();
			}
			else
			{
				tclMisc->setTclResult(interp, noSuchWin);
				return TCL_ERROR;
			}
			break;
		}
		case 10:
		{
			if (argc != 3)
			{
				Tcl_WrongNumArgs(interp, 0, NULL, "chat restore UIN/UINy");
				return TCL_ERROR;
			}
			uin = argv[2];
			if (uin.find(" ") > -1)
			{
				QStringList list = QStringList::split(" ", uin);
				for ( QStringList::iterator it = list.begin(); it != list.end(); it++ )
				{
					uins.append((*it).toUInt());
				}
			}
			else
			{
				uins.append(uin.toUInt());
			}
			if (chat_manager->findChatByUins(uins))
			{
				chat_manager->findChatByUins(uins)->showNormal();
			}
			else
			{
				tclMisc->setTclResult(interp, noSuchWin);
				return TCL_ERROR;
			}
			break;
		}
		case 11:
		{
			if (argc != 3)
			{
				Tcl_WrongNumArgs(interp, 0, NULL, "chat maximize UIN/UINy");
				return TCL_ERROR;
			}
			uin = argv[2];
			if (uin.find(" ") > -1)
			{
				QStringList list = QStringList::split(" ", uin);
				for ( QStringList::iterator it = list.begin(); it != list.end(); it++ )
				{
					uins.append((*it).toUInt());
				}
			}
			else
			{
				uins.append(uin.toUInt());
			}
			if (chat_manager->findChatByUins(uins))
			{
				chat_manager->findChatByUins(uins)->showMaximized();
			}
			else
			{
				tclMisc->setTclResult(interp, noSuchWin);
				return TCL_ERROR;
			}
			break;
		}
		case 12:
		{
			if (argc != 2)
			{
				Tcl_WrongNumArgs(interp, 0, NULL, "chat fixemotspath");
				return TCL_ERROR;
			}
			QString theme = emoticons->themePath(), tmp = "";
			for ( int i = 0; i < theme.contains("/"); i++ )
				tmp += "../";

			tclMisc->setTclResult(interp, tmp);
			break;
		}
		case 13:
		{
			if (argc != 3)
			{
				Tcl_WrongNumArgs(interp, 0, NULL, "chat clear UIN/UINy");
				return TCL_ERROR;
			}
			uin = argv[2];
			if (uin.find(" ") > -1)
			{
				QStringList list = QStringList::split(" ", uin);
				for ( QStringList::iterator it = list.begin(); it != list.end(); it++ )
				{
					uins.append((*it).toUInt());
				}
			}
			else
			{
				uins.append(uin.toUInt());
			}
			if (chat_manager->findChatByUins(uins))
			{
				chat_manager->findChatByUins(uins)->body->clear();
			}
			else
			{
				tclMisc->setTclResult(interp, noSuchWin);
				return TCL_ERROR;
			}
			break;
		}
		case 14:
		{
			if (argc != 3)
			{
				Tcl_WrongNumArgs(interp, 0, NULL, "chat getselection UIN/UINsList");
				return TCL_ERROR;
			}
			uin = argv[2];
			if (uin.find(" ") > -1)
			{
				QStringList list = QStringList::split(" ", uin);
				for ( QStringList::iterator it = list.begin(); it != list.end(); it++ )
				{
					uins.append((*it).toUInt());
				}
			}
			else
				uins.append(uin.toUInt());

			if (chat_manager->findChatByUins(uins))
			{
				QString str, tmp = chat_manager->findChatByUins(uins)->edit->selectedText();
				HtmlDocument doc;
				doc.parseHtml(tmp);
				for ( int i = 0; i < doc.countElements(); i++ )
					if (!doc.isTagElement(i))
						str = doc.elementText(i);

				tclMisc->setTclResult(interp, tmp);
			}
			else
			{
				tclMisc->setTclResult(interp, noSuchWin);
				return TCL_ERROR;
			}
			break;
		}
		default:
		{
			tclMisc->setTclResult(interp, QObject::tr("Command should be one of following:\n%1").arg(acts.join(", ")));
			return TCL_ERROR;
		}
	}
	return TCL_OK;
}

/* Tcl Command hint
SYNTAX: hint ?-icon ikona? ?-font czcionka? ?-fgcolor kolor? ?-bgcolor kolor? ?-timeout sekundy? tekst

DESCRIPTION:
Wywietla tzw. dymki obok zadokowanej ikony Kadu. Jako pierwszy argument powinien by podany
tekst ktry bdzie stanowi informacj zawart w dymku. Moe on zawiera znaczniki HTML.
Pozostae argumenty s opcjonalne.<br><br>
<DT>
<P><DT><B>-icon</B><DD>
definiuje, jaka ikona ma si pojawi po lewej stronie informacji tekstowej. Domylnie pusta.<br>
<P><DT><B>-font</B><DD>
definiuje czcionk, ktra bdzie uyta w dymku. Domylnie jest to helvetica, rozmiar 12.
Argument czcionki moe mie 2 formy: pojedyncze sowo - nazw czcionki, lub te dwa sowa
(jako jeden argument TCL! czyli w cudzysowiu lub nawiasach klamrowych), gdzie pierwsze
to nazwa czcionki, a drugie to jej rozmiar.<br>
<P><DT><B>-fgcolor</B><DD>
okrela kolor czcionki. Domylnie czarny (black).<br>
<P><DT><B>-bgcolor</B><DD>
okrela kolor ta. Domylnie szary (grey).<br>
<P><DT><B>-timeout</B><DD>
ustawia czas wywietlania dymka w sekundach. Domylnie 3 sekundy.<br>
<br>
<I>Kolory</I> mog by podane w postaci #RRGGBB lub jako anglo-jzycznej nazwy kolorw podstawowych, jak
black, white, green, grey, itd. Czcionka oznacza rodzaj uytej czcionki, ale mona te po spacji
(ale w tym samym argumencie!) poda jej rozmiar.
*/
int Tcl_CmdHint(void* clientData, Tcl_Interp *interp, int argc, const char *argv[])
{
	if (argc < 2 || (argc % 2))
	{
		Tcl_WrongNumArgs(interp, 0, NULL, "hint ?-icon icon? ?-font font? ?-fgcolor color? ?-bgcolor color? ?-timeout secs? text");
		return TCL_ERROR;
	}
	QString msg = QString::fromUtf8(argv[(argc-1)]), tmp;
	QMap<QString,QVariant> parms;
	QStringList fontList;
	bool ok;
	int size = 0;

	QStringList opts = QStringList::split(" ", "-icon -font -fgcolor -bgcolor -timeout");
	for ( int c = 1; c < (argc-1); c++ )
	{
		tmp = argv[c];
		c++;
		switch (opts.findIndex(tmp))
		{
			case 0:
			{
				parms["Pixmap"] = icons_manager.loadIcon(argv[c]);
				break;
			}
			case 1:
			{
				int _argc, code;
				const char **_argv;
				code = Tcl_SplitList(interp, argv[2], &_argc, &_argv);
				if (code == TCL_ERROR)
				{
					tclMisc->setTclResult(interp, QObject::tr("-font option takes proper TCL list as an artument."));
					return TCL_ERROR;
				}

				for ( int i = 0; i < _argc; i++ )
					fontList.append(_argv[i]);

				if (fontList.count() > 1)
				{
					size = fontList[1].toUInt(&ok);
					if (!ok)
					{
						tclMisc->setTclResult(interp, QObject::tr("second argument of option -font should be integer value."));
						return TCL_ERROR;
					}
				}
				parms["Font"] = QFont(fontList[0], size);
				break;
			}
			case 2:
			{
				parms["Foreground color"] = QString(argv[c]);
				break;
			}
			case 3:
			{
				parms["Background color"] = QString(argv[c]);
				break;
			}
			case 4:
			{
				parms["Timeout"] = QString(argv[c]).toUInt(&ok);
				if (!ok)
				{
					tclMisc->setTclResult(interp, QObject::tr("-timeout option accepts only integer values."));
					return TCL_ERROR;
				}
				break;
			}
			default:
			{
				Tcl_WrongNumArgs(interp, 0, NULL, "hint ?-icon icon? ?-font font? ?-fgcolor color? ?-bgcolor color? ?-timeout secs? text");
				return TCL_ERROR;
			}
		}
	}
	parms["ShowSource"] = bool(FALSE);
	notify->emitMessage("TCL", "Hints", msg, &parms);
	return TCL_OK;
}

/* Tcl Command timer
SYNTAX: timer ?-repeat? milisekundy kod
SYNTAX: timer -delete ID
SYNTAX: timer list

DESCRIPTION:
Skadnia przypomina t najczciej uywan z komendy after. Wykonuje
podany kod po upywie podanej liczby <I>milisekund</I>. Zwraca ID (identyfikator)
dla danego licznika, ktrego pniej mona uy z opcj -delete, aby zatrzyma odliczanie
i usun licznik. timer list zwraca list licznikw, w ktrej kady element jest dwywurazowy
- pierwszy wyraz to ID licznika, a drugi to przyporzdkowany mu kod. Opcja -repeat
oznacza, e podany kod bdzie wykonywany co podan liczb <I>milisekund</I>, a do momentu
usunicia czasomierza, co pozwala na budowanie ptli niekoniecznie rekurencyjnych.
*/
int Tcl_CmdTimer(void* clientData, Tcl_Interp *interp, int argc, const char *argv[])
{
	if (argc > 4 || argc < 2)
	{
		Tcl_WrongNumArgs(interp, 0, NULL, "timer ?\?-repeat? msecs code? ?-delete id? ?list?");
		return TCL_ERROR;
	}
	int time, id, arg, sessID = *((int*) clientData);
	bool ok, ss = TRUE;
	QString cmd = argv[1];
	TclSession *tsess = tclSess[sessID];

	if (!cmd.compare("-delete"))
	{
		if (argc != 3)
		{
			Tcl_WrongNumArgs(interp, 0, NULL, "timer -delete id");
			return TCL_ERROR;
		}
		id = QString(argv[2]).toInt(&ok);
		if (!ok)
		{
			Tcl_WrongNumArgs(interp, 0, NULL, "timer -delete id");
			return TCL_ERROR;
		}

		uint i = 0;
		while ( i < tsess->tclTimers.count() && tsess->tclTimers[i].id != id )
			i++;

		if ( i == tsess->tclTimers.count() )
		{
			tclMisc->setTclResult(interp, QObject::tr("Timer doesn't exists!"));
			return TCL_ERROR;
		}

		tsess->tclTimers[i].ptr->kill();

	}
	else if (!cmd.compare("list"))
	{
		if (argc != 2)
		{
			Tcl_WrongNumArgs(interp, 0, NULL, "timer list");
			return TCL_ERROR;
		}
		for ( uint i = 0; i < tsess->tclTimers.count(); i++ )
			Tcl_AppendElement(interp, (char*)QString::number(tsess->tclTimers[i].id).latin1());

		return TCL_OK;

	}
	else
	{
		if (argc == 3)
		{
			arg = 2;
			time = cmd.toInt(&ok);
		}
		else if (argc == 4 && !cmd.compare("-repeat") )
		{
			arg = 3;
			cmd = argv[2];
			time = cmd.toInt(&ok);
			ss = FALSE;
		}
		else
		{
			Tcl_WrongNumArgs(interp, 0, NULL, "timer ?-repeat? msecs code");
			return TCL_ERROR;
		}

		if (!ok)
		{
			Tcl_WrongNumArgs(interp, 0, NULL, "timer ?-repeat? msecs code");
			return TCL_ERROR;
		}

		TclTimer *timer = new TclTimer(tsess);
		timerID ID;

		timer->setCode(QString::fromUtf8(argv[arg]));

		// Rejestracja nowego ID
		timer->setSingleShot(ss);
		ID.ptr = timer;
		id = timer->start(time, ss);
		timer->currIdent(id);
		ID.id = id;
		tsess->tclTimers.append(ID);

		// Zwrocenie ID
		Tcl_SetResult(interp, (char*)QString::number(id).latin1(), TCL_VOLATILE);

	}

	return TCL_OK;
}

/* Tcl Command on
SYNTAX: on typ ?-delete? procedura
SYNTAX: on -list ?typ?

DESCRIPTION:
Komenda on suy do obslugi zdarze. Sposobem uycia przypomina nieco
komend 'bind' z IRC bota - Eggdropa. Kademu zdarzeniu
moe towarzyszy jedno lub wicej wywoa procedur. <I>Typ</I> podawany
jako pierwszy z argumentw okrela rodzaj zdarzenia, ktre chcemy obsuy.
Proba przyporzadkowania procedury do danego <I>typu</I>, ktra jest ju
przyporzdkowana zakoczy si zwrceniem bdu (dlatego w razie,
gdy nie ma pewnoci, warto to sprawdzi wczeniej komend
<code>[lsearch [on -list typ] procedura]</code>). Jeli zostanie uyty przecznik -delete,
to obsuga zdarzenia dla danego <I>typu</I> i <I>procedury</I> zostanie usunita
(o ile istnieje, bo jeli nie, to zwrcony zostanie bd). Jeli natomiast uyty
zostanie przecznik -list, mona nie podawa adnego argumentu, lub poda jedynie
<I>typ</I>. Jeli nie podamy <I>typu</I>, to zwrcona zostanie lista wszystkich
obsugiwanych zdarze. Jeli natomiast podamy <I>typ</I>, to zostan zwrcone
wszystkie procedury obsugujce zdarzenie tego typu.<br>

<center><B>Obsugiwane <I>typy</I> zdarze</B><br></center><br>
<font color=#555555>
<B>typ_zdarzenia</B> <I>opcjonalne_argumenty[<B>spodziewany_typ_danych</B>]</I><br>
Opis zdarzenia.<br>
</font>

EVENTS

DESCRIPTION:
Podane argumenty bd przekazywane do procedury, ktr
przyporzdkujemy do obsugi tego zdarzenia. W nawiasach kwadratowych podane s typy danych, jakich
naley si spodziewa w danym argumencie.<br>
Swego rodzaju trikiem jest uycie <I>procedury</I> wraz z argumentem ju w kodzie, tak jak
w drugim przykadnie. Wtedy argumenty wstawiane na poziomie samego kodu s przekazywane
do procedury jako pierwsze, dopiero po nich nastpuj argumenty zdarzenia.<br>

EXAMPLE:
on chstatus chstat

proc chstat {uin status descr ip port} {
	set st "Uytkownik o UINie $uin zmieni status na: $status"
	if {"$descr" != ""} {
		append st " ($descr)"
	}
	append st .
	puts $st
}
ENDOFEXAMPLE

<br>

EXAMPLE:
on chat0 "recvMsg false"
on chat1 "recvMsg true"

proc recvMsg {boolean uin text time} {
	if {$boolean} {
		puts "To zdarzenie zostao wywoane przez chat1."
	} else {
		puts "To zdarzenie zostao wywoane przez chat0."
	}
}
ENDOFEXAMPLE

Efekt bdzie widoczny w konsoli konsoli Tcl moduu.
*/
int Tcl_CmdEvent(void* clientData, Tcl_Interp *interp, int argc, const char *argv[])
{
	QString syntax = "on ?-list ?type?? ??-delete? proc?";
	if (argc < 2 || argc > 4)
	{
		Tcl_WrongNumArgs(interp, 0, NULL, syntax.latin1());
		return TCL_ERROR;
	}
	int sessID = *((int*) clientData), type;
	TclSession *tsess = tclSess[sessID];

	if (!QString::compare(argv[1], "-list"))
	{
		if (argc < 2 || argc > 3)
		{
			Tcl_WrongNumArgs(interp, 0, NULL, syntax.latin1());
			return TCL_ERROR;
		}
		if (argc == 3)
		{
			if (tclMisc->getEvents().findIndex(argv[2]) == -1)
			{
				tclMisc->setTclResult(interp, QObject::tr("Wrong event type: %1").arg(type));
				return TCL_ERROR;
			}
			type = tclMisc->getEventId(argv[2]);
			TclString* str = new TclString();
			TclEvent::Iterator it;
			for( it = tsess->tclEventList[type].begin(); it != tsess->tclEventList[type].end(); it++ )
				str->append(*it);

			str->setResult(interp);
			delete str;
		}
		else
		{
			TclString* str = new TclString();
			QStringList::Iterator it;
			for( it = tclMisc->getEvents().begin(); it != tclMisc->getEvents().end(); it++ )
				str->append(*it);

			str->setResult(interp);
			delete str;
		}
	}
	else if (!QString::compare(argv[2], "-delete"))
	{
		if (argc != 4)
		{
			Tcl_WrongNumArgs(interp, 0, NULL, syntax.latin1());
			return TCL_ERROR;
		}
		if (tclMisc->getEvents().findIndex(argv[1]) == -1)
		{
			tclMisc->setTclResult(interp, QObject::tr("Wrong event type: %1").arg(type));
			return TCL_ERROR;
		}
		type = tclMisc->getEventId(argv[1]);
		if (tsess->tclEventList[type].findIndex(argv[3]) > -1)
		{
			tsess->tclEventList[type].remove(argv[3]);
		}
		else
		{
			tclMisc->setTclResult(interp, QObject::tr("Event handler for \'%1\' with command \'%2\' doesn't exist.").arg(argv[1]).arg(argv[3]));
			return TCL_ERROR;
		}
	}
	else
	{
		if (argc != 3)
		{
			Tcl_WrongNumArgs(interp, 0, NULL, syntax.latin1());
			return TCL_ERROR;
		}
		if (tclMisc->getEvents().findIndex(argv[1]) == -1)
		{
			tclMisc->setTclResult(interp, QObject::tr("Wrong event type: %1").arg(type));
			return TCL_ERROR;
		}
		type = tclMisc->getEventId(argv[1]);
		if (tsess->tclEventList[type].findIndex(argv[2]) > -1)
		{
			tclMisc->setTclResult(interp, QObject::tr("Event handler for \'%1\' with command \'%2\' already exists.").arg(argv[1]).arg(argv[2]));
			return TCL_ERROR;
		}
		tsess->tclEventList[type].append(argv[2]);
	}
	return TCL_OK;
}

/* Tcl Command msg
SYNTAX: msg ?-html? UINy wiadomo
DESCRIPTION:
Wysya wiadomo do kontaktu(-w) o podanym(-ych) <I>UIN'ie(-ach)</I> o treci
wiadomo. Wywoanie tej komendy nie powoduje ani otwarcia okna rozmowy z dan osob,
ani te nie dodaje tej wiadomoci do otwartego okna. Opcja <b>-html</b> pozwala na uywanie
znacznikw formatujcych <code>&lt;span style="..."&gt;</code>. Zwraca liczb, ktora stanowi identyfikator wiadomoci,
ktry mona potem wykorzysta przy zdarzeniu <B>ack</B> (patrz komenda <a href="on.html"><B>on</B></a>).
*/
int Tcl_CmdMsg(void* clientData, Tcl_Interp *interp, int argc, const char *argv[])
{
	if (argc < 3 || argc > 4)
	{
		Tcl_WrongNumArgs(interp, 0, NULL, "msg ?-html? UINs message");
		return TCL_ERROR;
	}
	if (gadu->status().isOffline())
	{
		tclMisc->setTclResult(interp, QObject::tr("Application encountered network error."));
		return TCL_ERROR;
	}
	QString rcpt, msg, myLastMessage, mesg;
	bool conv, rich = FALSE;
	uint myLastFormatsLength;
	uint uin;
	void* myLastFormats;

	if (argc == 4)
	{
		if (!QString::compare(argv[1], "-html"))
		{
			rich = TRUE;
			rcpt = QString::fromUtf8(argv[2]);
			msg = QString::fromUtf8(argv[3]);
		}
		else
		{
			Tcl_WrongNumArgs(interp, 0, NULL, "msg ?-html? UINs message");
			return TCL_ERROR;
		}
	}
	else
	{
		rcpt = QString::fromUtf8(argv[1]);
		msg = QString::fromUtf8(argv[2]);
	}

	UinsList uins;
	if (rcpt.find(" ") > -1)
	{
		QCString utf;
		QStringList list = QStringList::split(" ", rcpt);

		for ( uint i = 0; i < list.count(); i++ )
		{
			uin = list[i].toUInt(&conv);
			if (conv)
			{
				if (!uins.contains(uin))
				{
					uins.append(uin);
				}
			}
			else
			{
				tclMisc->setTclResult(interp, QObject::tr("User isn't registered: %1\nwhile executing:\nmsg %2 %3").arg(list[i]).arg(rcpt).arg(msg));
				return TCL_ERROR;
			}
		}
	}
	else
	{
		uint uin = rcpt.toUInt(&conv);
		if (conv)
		{
			uins.append(uin);
		}
		else
		{
			if (userlist.containsAltNick(rcpt))
			{
				uins.append(userlist.byAltNick(rcpt).uin());
			}
			else
			{
				tclMisc->setTclResult(interp, QObject::tr("User isn't registered: %1\nwhile executing:\nmsg %1 %2").arg(rcpt).arg(msg));
				return TCL_ERROR;
			}
		}
	}

	if (rich)
	{
		// Kod pozyczony z Kadu
		myLastMessage = msg;

		mesg = myLastMessage;
		mesg.replace(QRegExp("\n"), "\r\n");
		mesg = unformatGGMessage(mesg, myLastFormatsLength, myLastFormats);
		myLastMessage = mesg;
		if (myLastFormatsLength)
		{
			myLastMessage = formatGGMessage(myLastMessage, myLastFormatsLength - sizeof(struct gg_msg_richtext),
				(void *)((char *)(myLastFormats) + sizeof(struct gg_msg_richtext)),0);
		}
		else
		{
			HtmlDocument::escapeText(myLastMessage);
		}
		myLastMessage.replace(QRegExp("\r\n"), "\n");
		Tcl_SetResult(interp, (char*)QString::number(gadu->sendMessageRichText(uins, unicode2cp(mesg),
			(unsigned char *)myLastFormats, myLastFormatsLength)).latin1(), TCL_VOLATILE);
	}
	else
	{
		Tcl_SetResult(interp, (char*)QString::number(gadu->sendMessage(uins, unicode2cp(msg))).latin1(),
			TCL_VOLATILE);
	}
	return TCL_OK;
}

/* Tcl Command status
SYNTAX: status ?nowyStatus ?opis??
SYNTAX: status ?-showdesc?

DESCRIPTION:
Bez argumentw zwraca aktualny status lokalnego uytkownika. Mona poda opcje -showdesc, aby pobra
ewentualny opis do statusu. Jeli w takim wypadku uytkownika ma ustawiony opis, to zwrcony on bdzie jako
drugi wyraz listy (pierwszy to status). Jeli podany jest <I>nowy status</I>,
to zostanie on ustawiony. Jeli podany <I>status</I> wymaga opisu, to ostatnio ustawiany zostanie tutaj
uyty, chyba e uytkownik poda tutaj nowy opis.
*/
int Tcl_CmdStatus(void* clientData, Tcl_Interp *interp, int argc, const char *argv[])
{
	if (argc > 4)
	{
		Tcl_WrongNumArgs(interp, 0, NULL, "status ?-showdesc? ?newStatus ?description??");
		return TCL_ERROR;
	}
	if ((argc == 2 || argc == 3) && QString::compare(argv[1], "-showdesc"))
	{
		QString stat = argv[1], desc = "";
		int ggstat = tclMisc->TclStatusToGGStatus(stat);
		if (ggstat == -1)
		{
			tclMisc->setTclResult(interp, QObject::tr("valid statuses: %1").arg("notavailable available busy available+description busy+description notavailable+description invisible invisible+description"));
			return TCL_ERROR;
		}

		if (argc == 3)
			desc = QString::fromUtf8(argv[2]);

		tclMisc->setTclStatus(argv[1], desc);
	}
	else
	{
		int status = gadu->status().index();
		QString _status = tclMisc->KaduStatusToTclStatus(status);
		if (_status.isEmpty())
		{
			tclMisc->setTclResult(interp, QObject::tr("Unknown status type."));
			return TCL_ERROR;
		}

		Tcl_SetResult(interp, (char*)_status.latin1(), TCL_VOLATILE);

		if (!QString::compare(argv[1], "-showdesc") && gadu->status().hasDescription())
			tclMisc->appendTclResult(interp, gadu->status().description());
	}
	return TCL_OK;
}

/* Tcl Command console
SYNTAX: console
DESCRIPTION:
Otwiera okno konsoli TCL, gdzie mona wpisywa komendy na bierzco.
*/
int Tcl_CmdConsole(void* clientData, Tcl_Interp *interp, int argc, const char *argv[])
{
	if (argc > 1)
	{
		Tcl_WrongNumArgs(interp, 0, NULL, "console");
		return TCL_ERROR;
	}
	tclSess[0]->openConsole();
	return TCL_OK;
}

/* Tcl Command qlistbox
SYNTAX: qlistbox tytu ?wiadomo? lista

DESCRIPTION:
Wywietla okno o podanym <I>tytule</I>, z list sw podanych w <I>liscie</I>. Opcjonalnie
moe byc podana tre <I>wiadomoci</I>, ktra zostanie wywietlona tu nad list. W oknie
oprcz listy znajduje si pole edycji, oraz 4 przyciski: OK i CANCEL o dziaaniu wiadomym, oraz ADD i DELETE,
ktre odpowiednio: dodaj warto pola edycji i uswaj zaznaczony element z listy. Komenda
zwraca w zalenoci od potwierdzenia, bd anulowania operacji list: po dokonanych operacjach na liscie, bd
jej stan pierwotny, z jakim zostaa wywoana.
*/
int Tcl_CmdListBox(void* clientData, Tcl_Interp *interp, int argc, const char *argv[])
{
	if (argc > 4 || argc < 3)
	{
		Tcl_WrongNumArgs(interp, 0, NULL, "listbox caption ?message? list");
		return TCL_ERROR;
	}
	int _argc, code;
	const char **_argv;
	QString caption = QString::fromUtf8(argv[1]), message, tmp;
	QStringList list;
	QCString utf;
	Tcl_Obj* result;
	Tcl_DString ds;
	Tcl_DStringInit(&ds);

	if (argc == 3)
	{
		code = Tcl_SplitList(interp, argv[2], &_argc, &_argv);
	}
	else
	{
		code = Tcl_SplitList(interp, argv[3], &_argc, &_argv);
		message = QString::fromUtf8(argv[2]);
	}
	if (code != TCL_OK)
	{
		tclMisc->setTclResult(interp, QObject::tr("Invalid list argument for command listbox."));
		return code;
	}
	TclListBox *dialog;
	if (!message.isNull())
	{
		dialog = new TclListBox(TRUE);
		dialog->setMsg(message);
	}
	else
	{
		dialog = new TclListBox(FALSE);
	}
	dialog->setCaption(caption);
	for (int i = 0; i < _argc; i++ )
	{
		tmp = QString::fromUtf8(_argv[i]);
		dialog->list()->insertItem(tmp);
		list.append(tmp);
	}

	dialog->show();
	if (dialog->exec() == QDialog::Accepted)
	{
		int cnt = dialog->list()->count();
		for ( int i = 0; i < cnt; i++ )
		{
			utf = dialog->list()->text(i).utf8();
			result = Tcl_NewStringObj((const char* const)utf, utf.length());
			Tcl_DStringAppendElement(&ds, Tcl_GetString(result));
		}
	}
	else
	{
		for ( QStringList::iterator it = list.begin(); it != list.end(); it++ )
		{
			utf = QString(*it).utf8();
			result = Tcl_NewStringObj((const char* const)utf, utf.length());
			Tcl_DStringAppendElement(&ds, Tcl_GetString(result));
		}
	}
	Tcl_DStringResult(interp, &ds);
	Tcl_DStringFree(&ds);
	delete dialog;
	return TCL_OK;
}

/* Tcl Command qmenu
GLOBALSYNTAX: qmenu komenda nazwa ?argumenty?
DESCRIPTION:
Komenda do tworzenia i zarzdzania menu.

SUBSYNTAX: qmenu new nazwa
DESCRIPTION:
Tworzy nowe menu. Nazwa jest jednoczesnie identyfikatorem uzywanym w pozostaych podkomendach menu,
tytuem menu wywietlanego w oknie Kadu, jak i nazw menu popup, ktry zostanie przyporzdkowany do tego menu.
Popup menu musi by oczywicie uprzednio stworzone za pomoc komendy <B>popup</B>. Jeli menu o podanej
<I>nazwie</I> ju istnieje, zwraca bd. Uycie znaku <B>&amp;</B> gdzie w <I>nazwie</I> (ale nie na kocu)
spowoduje automatyczne przydzielenie skrtu klawiszowego Alt+&lt;litera znajdujca si tu za tym znakiem&gt;
do otwierania tego menu. W pniejszym uywaniu <I>nazwy</I> manu, naley uwzgldni uyty znak <B>&amp;</B>.

SUBSYNTAX: qmenu delete nazwa
DESCRIPTION:
Usuwa menu o podanej nazwie. Jeli menu o podanej <I>nazwie</I> nie istnieje, zwraca bd.

SUBSYNTAX: qmenu exists nazwa
DESCRIPTION:
Jeli menu o <I>nazwie</I> istnieje, zwraca 1, inaczej zwraca 0.

SUBSYNTAX: qmenu disable nazwa
DESCRIPTION:
Wycza menu. W takim stanie uytkownik nie moe na nim klikn, a kolor jego tytuu zmienia si na janiejszy.
Jeli menu o podanej <I>nazwie</I> nie istnieje, zwraca bd.

SUBSYNTAX: qmenu enable nazwa
DESCRIPTION:
Wcza menu. Jest to domylny tryb pracy menu zaraz po stworzeniu.
Jeli menu o podanej <I>nazwie</I> nie istnieje, zwraca bd.

SUBSYNTAX: qmenu isenabled nazwa
DESCRIPTION:
Zwraca 1, jeli menu o podanej <I>nazwie</I> jest wczone, inaczej zwraca 0.
*/
int Tcl_CmdMenu(void* clientData, Tcl_Interp *interp, int argc, const char *argv[])
{
	if (argc < 2)
	{
		Tcl_WrongNumArgs(interp, 0, NULL, "qmenu command name ?arguments?");
		return TCL_ERROR;
	}
	QStringList cmds = QStringList::split(" ", "new delete disable enable exists isenabled");
	QString command = argv[1], err, icon, name = QString::fromUtf8(argv[2]);
	QCString utf;
	int sessID = *((int*) clientData);
	TclSession *tsess = tclSess[sessID];

	switch (cmds.findIndex(command))
	{
		case 0:
		{
			if (argc != 3)
			{
				Tcl_WrongNumArgs(interp, 0, NULL, "qmenu new name");
				return TCL_ERROR;
			}
			if (tsess->tclPopups.contains(name))
			{
				tsess->tclMenuBar[name] = kadu->menuBar()->insertItem(name, tsess->tclPopups[name].menu);
			}
			else
			{
				tclMisc->setTclResult(interp, QObject::tr("Popup %1 doesn't exist.").arg(name));
				return TCL_ERROR;
			}
			break;
		}
		case 1:
		{
			if (argc != 3)
			{
				Tcl_WrongNumArgs(interp, 0, NULL, "qmenu delete name");
				return TCL_ERROR;
			}
			if (tsess->tclMenuBar.contains(name))
			{
				kadu->menuBar()->removeItem(tsess->tclMenuBar[name]);
			}
			else
			{
				tclMisc->setTclResult(interp, QObject::tr("Menu %1 doesn't exist.").arg(name));
				return TCL_ERROR;
			}
			break;
		}
		case 2:
		{
			if (argc != 3)
			{
				Tcl_WrongNumArgs(interp, 0, NULL, "qmenu disable name");
				return TCL_ERROR;
			}
			if (tsess->tclMenuBar.contains(name))
			{
				if (kadu->menuBar()->isItemEnabled(tsess->tclMenuBar[name]))
				{
					kadu->menuBar()->setItemEnabled(tsess->tclMenuBar[name], FALSE);
				}
				else
				{
					tclMisc->setTclResult(interp, QObject::tr("Menu '%1' is already disabled.").arg(name));
					return TCL_ERROR;
				}
			}
			else
			{
				tclMisc->setTclResult(interp, QObject::tr("Menu %1 doesn't exist.").arg(name));
				return TCL_ERROR;
			}
			break;
		}
		case 3:
		{
			if (argc != 3)
			{
				Tcl_WrongNumArgs(interp, 0, NULL, "qmenu enable name");
				return TCL_ERROR;
			}
			if (tsess->tclMenuBar.contains(name))
			{
				if (!kadu->menuBar()->isItemEnabled(tsess->tclMenuBar[name]))
				{
					kadu->menuBar()->setItemEnabled(tsess->tclMenuBar[name], TRUE);
				}
				else
				{
					tclMisc->setTclResult(interp, QObject::tr("Menu '%1' is already enabled.").arg(name));
					return TCL_ERROR;
				}
			}
			else
			{
				tclMisc->setTclResult(interp, QObject::tr("Menu %1 doesn't exist.").arg(name));
				return TCL_ERROR;
			}
			break;
		}
		case 4:
		{
			if (argc != 3)
			{
				Tcl_WrongNumArgs(interp, 0, NULL, "qmenu exists name");
				return TCL_ERROR;
			}
			if (tsess->tclMenuBar.contains(name))
			{
				Tcl_SetResult(interp, "1", TCL_STATIC);
			}
			else
			{
				Tcl_SetResult(interp, "0", TCL_STATIC);
			}
			break;
		}
		case 5:
		{
			if (argc != 3)
			{
				Tcl_WrongNumArgs(interp, 0, NULL, "qmenu isenabled name");
				return TCL_ERROR;
			}
			if (tsess->tclMenuBar.contains(name))
			{
				if (kadu->menuBar()->isItemEnabled(tsess->tclMenuBar[name]))
				{
					Tcl_SetResult(interp, "1", TCL_STATIC);
				}
				else
				{
					Tcl_SetResult(interp, "0", TCL_STATIC);
				}
			}
			else
			{
				tclMisc->setTclResult(interp, QObject::tr("Menu %1 doesn't exist.").arg(name));
				return TCL_ERROR;
			}
			break;
		}
		default:
			tclMisc->setTclResult(interp, QObject::tr("Command should be one of following:\n%1").arg(cmds.join(", ")));
			return TCL_ERROR;
	}
	return TCL_OK;
}

/* Tcl Command config
GLOBALSYNTAX: config komenda argumenty
DESCRIPTION:
Tworzy nowe wpisy w oknie konfiguracyjnym. Kazdy taki wpis zapisuje jednoczesnie swoja wartosc w pliku
konfiguracyjnym, tak wiec wartosci tworzonych widgetow nalezy odczytywac z pliku konfiguracyjnego za pomoca
komendy configfile (gdzie nazw grupy bdzie nazwa zakadki z okna konfiguracyjnego,
a nazw zmiennej bdzie stanowi ta podana w config add).<br>
<b>UWAGA!</b> Aktualnie modu TCL nie obsuguje polskich znakw w konfiguracji Kadu, ze wzgldu na specyfik
tego okna oraz zasad dziaania moduu.

SUBSYNTAX: config newtab nazwa
DESCRIPTION:
Dodaje now zakadk o podanej <I>nazwie</I> do okna konfoguracji. Niestety nie sprawdza,
czy zakadka o tej <I>nazwie</I> ju istnieje, wic podczas pierwszego tworzenia tej zakadki,
trzeba doda te chocia jedn kontrolk (patrz niej), eby mona byo potem uy komendy
config exists odnonie tej kontrolki, co zarazem bdzie wiadczy o istnieniu danej zakadki.

SUBSYNTAX: config add opcje
DESCRIPTION:
Dodaje now kontrolk.
Opcje:<br>
<DT>
<P><DT><B>-tab</B> <I>zakadka</I><DD>
Dodaje kontrolk w podanej <I>zakadce</I>.<br>

<DT>
<P><DT><B>-parent</B> <I>kontrolka</I><DD>
Dodaje kontrolk w podanej <I>kontrolce</I>-rodzicu.<br>

<DT>
<P><DT><B>-caption</B> <I>tytu</I><DD>
Dodawana kontorlka bdzie mie podany <I>tytu</I>.<br>

<DT>
<P><DT><B>-type</B> <I>typ</I><DD>
Okrela typ kontrolki. Wszystkie typy kontrolek opisane s niej.<br>

<DT>
<P><DT><B>-cols</B> <I>kolumny</I><DD>
Okrela liczb kolumn dla typu <B>grid</B>.<br>

<DT>
<P><DT><B>-default</B> <I>domylnaWarto</I><DD>
Okrela domyln warto dla kontrolek przyjmujcych opcj <B>-var</b>.<br>

<DT>
<P><DT><B>-var</B> <I>nazwaZmiennej</I><DD>
Podana <I>nazwaZmiennej</I> bdzie uywana do zapisywania wartoci kontrolki w pliku konfiguracyjnym, w postaci:<br>
<code>[zakadka]<br>
nazwaZmiennej=warto</code><br>

<DT>
<P><DT><B>-hint</B> <I>podpowied</I><DD>
Dodawanej kontrolce zostanie przyporzdkowana dana <I>podpowied</I> (w chmurce).<br>

<DT>
<P><DT><B>-options</B> <I>listaOpcji</I><DD>
Lista dostpnych wartoci symbolicznych, widocznych dla uytkownika, dla takich kontrolek jak <B>combobox</B>, <B>vradiogroup</B>, czy <B>hradiogroup</B>.<br>

<DT>
<P><DT><B>-values</B> <I>listaWartoci</I><DD>
Lista rzeczywistych wartoci odpowiadajcych symbolicznym wartociom opdanym w opcji <B>-options</B>. Lista elementw dla obu opcji (tej i powyszej) musi si zgadza.<br>

<DT>
<P><DT><B>-step</B> <I>warto</I><DD>
Okrela o ile ma nastpi skok wartoci dla takich kontrolek jak <B>slider</B> lub <B>spinbox</B>. Jest to warto liczbowa.<br>

<DT>
<P><DT><B>-min</B> <I>warto</I><DD>
Okrela minimaln warto dla takich kontrolek jak <B>slider</B> lub <B>spinbox</B>. Jest to warto liczbowa.<br>

<DT>
<P><DT><B>-max</B> <I>warto</I><DD>
Okrela maksymaln warto dla takich kontrolek jak <B>slider</B> lub <B>spinbox</B>. Jest to warto liczbowa.<br>
<br>

Typy kontrolek:<br>
<DT>
<P><DT><B>label</B><DD>
Wymaga nastpujcych opcji: <B>-tab</B>, <B>-type</B>, <B>-caption</B><br>
Dodatkowo obsuguje opcje:<br>
Standardowa etykieta.<br>

<DT>
<P><DT><B>checkbox</B><DD>
Wymaga nastpujcych opcji: <B>-tab</B>, <B>-type</B>, <B>-caption</B>, <B>-var</B><br>
Dodatkowo obsuguje opcje: <B>-default</B>, <B>-parent</B><br>
Kontrolka o wartoci binarnej: zaznaczona, lub odznaczona.<br>

<DT>
<P><DT><B>combobox</B><DD>
Wymaga nastpujcych opcji: <B>-tab</B>, <B>-type</B>, <B>-caption</B>, <B>-var</B>, <B>-options</B>, <B>-values</B><br>
Dodatkowo obsuguje opcje: <B>-default</B>, <B>-parent</B><br>
Kontrolka wyboru jednego elementu z listy. Uzytkownik wybiera element spord podanych do opcji <B>-options</B>, a warto zmiennej podanej przy opcji <B>-var</B> przyjmuje warto odpowiadajc z listy podanej przy opcji <B>-values</B>.<br>

<DT>
<P><DT><B>hradiogroup</B><DD>
Wymaga nastpujcych opcji: <B>-tab</B>, <B>-type</B>, <B>-caption</B>, <B>-var</B>, <B>-options</B>, <B>-values</B><br>
Dodatkowo obsuguje opcje: <B>-default</B>, <B>-parent</B><br>
Grupa kontrolek o wartociach logicznych - zaznaczona, lub odznaczona. Tylko 1 z tych kontrolek moe by w danym momencie wybrana. Kontrolki s uoone obok siebie w poziomie. Jest to rodzaj alternatywny dla typu <B>combobox</B>.<br>

<DT>
<P><DT><B>vradiogroup</B><DD>
Wymaga nastpujcych opcji: <B>-tab</B>, <B>-type</B>, <B>-caption</B>, <B>-var</B>, <B>-options</B>, <B>-values</B><br>
Dodatkowo obsuguje opcje: <B>-default</B>, <B>-parent</B><br>
Wieloliniowe pole edycji tekstu.<br>
Grupa kontrolek o wartociach logicznych - zaznaczona, lub odznaczona. Tylko 1 z tych kontrolek moe by w danym momencie wybrana. Kontrolki s uoone w pionie. Jest to rodzaj alternatywny dla typu <B>combobox</B>.<br>

<DT>
<P><DT><B>lineedit</B><DD>
Wymaga nastpujcych opcji: <B>-tab</B>, <B>-type</B>, <B>-caption</B>, <B>-var</B><br>
Dodatkowo obsuguje opcje: <B>-default</B>, <B>-parent</B><br>
Jednoliniowe pole edycji tekstu.<br>

<DT>
<P><DT><B>text</B><DD>
Wymaga nastpujcych opcji: <B>-tab</B>, <B>-type</B>, <B>-caption</B>, <B>-var</B><br>
Dodatkowo obsuguje opcje: <B>-default</B>, <B>-parent</B><br>
Wieloliniowe pole edycji tekstu.<br>

<DT>
<P><DT><B>colorbutton</B><DD>
Wymaga nastpujcych opcji: <B>-tab</B>, <B>-type</B>, <B>-caption</B>, <B>-var</B>, <B>-default</B><br>
Dodatkowo obsuguje opcje: <B>-parent</B><br>
Przycisk wyboru koloru. Warto dla opcji <B>-default</B> przybiera nazw koloru, lub warto #RRGGBB.<br>

<DT>
<P><DT><B>fontselect</B><DD>
Wymaga nastpujcych opcji: <B>-tab</B>, <B>-type</B>, <B>-caption</B>, <B>-var</B><br>
Dodatkowo obsuguje opcje: <B>-default</B>, <B>-parent</B><br>
Kontrolka wyboru czcionki.<br>

<DT>
<P><DT><B>hotkey</B><DD>
Wymaga nastpujcych opcji: <B>-tab</B>, <B>-type</B>, <B>-caption</B>, <B>-var</B><br>
Dodatkowo obsuguje opcje: <B>-default</B>, <B>-parent</B><br>
Kontrolka wyboru skrtu klawiszowego.<br>

<DT>
<P><DT><B>slider</B><DD>
Wymaga nastpujcych opcji: <B>-tab</B>, <B>-type</B>, <B>-caption</B>, <B>-var</B><br>
Dodatkowo obsuguje opcje: <B>-default</B>, <B>-min</B>, <B>-max</B>, <B>-step</B>, <B>-parent</B><br>
Pasek doboru wartoci liczbowej przez przesuwanie suwaka. Mona okreli od jakiej minimalnej wartoci zaczyna si pasek, a na jakiej maksymalnej koczy. Mona te okreli co jak warto ma nastpowa skok przy przesuwie.<br>

<DT>
<P><DT><B>spinbox</B><DD>
Wymaga nastpujcych opcji: <B>-tab</B>, <B>-type</B>, <B>-caption</B>, <B>-var</B><br>
Dodatkowo obsuguje opcje: <B>-default</B>, <B>-min</B>, <B>-max</B>, <B>-step</B>, <B>-parent</B><br>
Pole edycji wartoci liczbowej przez wpisywanie liczby z klawiatury, lub inkrementacje bd dekrementacje jej za pomoc bocznych przyciskw. Mona okreli minimaln warto edytowanej liczby, oraz jej maksymaln warto. Mona te okreli co jak warto ma nastpowac dekrementacja bd inkrementacja.<br>

<DT>
<P><DT><B>vbox</B><DD>
Wymaga nastpujcych opcji: <B>-tab</B>, <B>-type</B>, <B>-caption</B><br>
Dodatkowo obsuguje opcje: <B>-parent</B><br>
Kontrolka uywana do ustalania ustawienia kontrolek wzgldem siebie. Kontrolki w VBox'ie bd ustawiane jedna pod drug, ponadto zostan one otoczone ramk i opatrzone etykieta podan w opcji <B>-caption</B>.<br>

<DT>
<P><DT><B>hbox</B><DD>
Wymaga nastpujcych opcji: <B>-tab</B>, <B>-type</B>, <B>-caption</B><br>
Dodatkowo obsuguje opcje: <B>-parent</B><br>
Kontrolka uywana do ustalania ustawienia kontrolek wzgldem siebie. Kontrolki w HBox'ie bd ustawiane jedna obok drugiej, ponadto zostan one otoczone ramk i opatrzone etykieta podan w opcji <B>-caption</B>.<br>

<DT>
<P><DT><B>grid</B><DD>
Wymaga nastpujcych opcji: <B>-tab</B>, <B>-type</B>, <B>-caption</B><br>
Dodatkowo obsuguje opcje: <B>-parent</B>, <B>-cols</B><br>
Kontrolka uywana do ustalania ustawienia kontrolek wzgldem siebie. Kontrolki w HBox'ie bd ustawiane kolejno w kadym wierszu, a do wypenienia wszystkich kolumn. Domylna liczba kolumn to 3.<br>

SUBSYNTAX: config delete nazwaZakadki ?tytu?
DESCRIPTION:
Usuwa kontrolk z danej <I>zakadki</I> o podanym <I>tytule</I>. Jeli tytu nie jest podany,
usuwana jest zakadka o podanej <I>nazwie</I>. Aby usun zakadk, wczeniej trzeba usun wszystkie
kontrolki w niej zawarte.

SUBSYNTAX: config exists nazwaZakadki tytu
DESCRIPTION:
Sprawdza czy kontrolka o <I>tytule</I> w zakadce o <I>nazwie</I> istnieje. Jeli tak, zwraca 1, jeli nie, zwraca 0.
*/
int Tcl_CmdConfig(void* clientData, Tcl_Interp *interp, int argc, const char *argv[])
{
	if (argc < 2)
	{
		Tcl_WrongNumArgs(interp, 0, NULL, "config command arguments");
		return TCL_ERROR;
	}
	QStringList cmds = QStringList::split(" ", "newtab add exists delete"),
				types = QStringList::split(" ", "checkbox label lineedit vbox hbox grid colorbutton combobox hotkey hradiobox vradiobox selectfont slider spinbox text"),
				_opts = QStringList::split(" ", "-type -tab -caption -parent -var -default -hint -cols -options -values -min -max -step");
	QString command = argv[1], grp, cap, var, err, type, strDef, parent, hint, tmp, syntax, cols, opts, vals, min, max, step;
	int res, boolDef, id, _cols;
	QCString utf;

	switch (cmds.findIndex(command))
	{
		case 0:
		{
			if (argc < 3 || argc > 4)
			{
				Tcl_WrongNumArgs(interp, 0, NULL, "config newtab tabName ?icon?");
				return TCL_ERROR;
			}
			grp = QString::fromUtf8(argv[2]);
			if (ConfigDialog::tabExists(grp))
			{
				tclMisc->setTclResult(interp, QObject::tr("Tab %1 already exists").arg(grp));
				return TCL_ERROR;
			}
			if (argc == 4)
			{
				ConfigDialog::addTab(grp, QString::fromUtf8(argv[3]));
			}
			else
			{
				ConfigDialog::addTab(grp);
			}
			break;
		}
		case 1:
		{
			syntax = "config add -type type -tab tabName -caption caption ?-parent parent? ?-var varName? ?-default value? ?-hint text? ?-cols number? ?-min minimalValue? ?-max maximalValue? ?-step value? ?-options list? ?-values list?";
			res = argc % 2;
			if (argc < 8 || res)
			{
				Tcl_WrongNumArgs(interp, 0, NULL, syntax.local8Bit());
				return TCL_ERROR;
			}
			for ( int i = 2; i < argc; i++ )
			{
				id = _opts.findIndex(argv[i]);
				i++;
				tmp = QString::fromUtf8(argv[i]);
				switch (id)
				{
					case 0:
						type = tmp;
						break;
					case 1:
						grp = tmp;
						break;
					case 2:
						cap = tmp;
						break;
					case 3:
						parent = tmp;
						break;
					case 4:
						var = tmp;
						break;
					case 5:
						strDef = tmp;
						break;
					case 6:
						hint = tmp;
						break;
					case 7:
						cols = tmp;
						break;
					case 8:
						opts = tmp;
						break;
					case 9:
						vals = tmp;
						break;
					case 10:
						min = tmp;
						break;
					case 11:
						max = tmp;
						break;
					case 12:
						step = tmp;
						break;
					default:
						Tcl_WrongNumArgs(interp, 0, NULL, syntax.local8Bit());
						return TCL_ERROR;
				}
			}
			if (type.isNull() || grp.isNull() || cap.isNull())
			{
				Tcl_WrongNumArgs(interp, 0, NULL, syntax.local8Bit());
				return TCL_ERROR;
			}
			if (!ConfigDialog::tabExists(grp))
			{
				tclMisc->setTclResult(interp, QObject::tr("Tab %1 doesn't exists").arg(grp));
				return TCL_ERROR;
			}

			switch (types.findIndex(type))
			{
				case 0:
				{
					if (!strDef.isNull())
					{
						res = Tcl_GetBoolean(interp, strDef.latin1(), &boolDef);
						if (res == TCL_ERROR)
						{
							tclMisc->setTclResult(interp, QObject::tr("Default value for checkbox should be boolean type."));
							return TCL_ERROR;
						}
					}
					else
					{
						boolDef = FALSE;
					}
					if (var.isNull())
					{
						tclMisc->setTclResult(interp, QObject::tr("Checkbox type needs a variable name to be given."));
						return TCL_ERROR;
					}
					if (parent.isNull())
					{
						parent = grp;
					}
					if (!ConfigDialog::controlExists(grp, cap))
					{
						ConfigDialog::addCheckBox(grp, parent, cap, var, boolDef, hint);
					}
					else
					{
						tclMisc->setTclResult(interp, QObject::tr("checkbox %1 in tab %2 already exists.").arg(cap).arg(grp));
						return TCL_ERROR;
					}
					break;
				}
				case 1:
				{
					if (parent.isNull())
					{
						parent = grp;
					}
					if (!ConfigDialog::controlExists(grp, cap))
					{
						ConfigDialog::addLabel(grp, parent, cap);
					}
					else
					{
						tclMisc->setTclResult(interp, QObject::tr("label %1 in tab %2 already exists.").arg(cap).arg(grp));
						return TCL_ERROR;
					}
					break;
				}
				case 2:
				{
					if (var.isNull())
					{
						Tcl_WrongNumArgs(interp, 0, NULL, "Lineedit type needs a variable name to be given.");
						return TCL_ERROR;
					}
					if (parent.isNull())
					{
						parent = grp;
					}
					if (!ConfigDialog::controlExists(grp, cap))
					{
						ConfigDialog::addLineEdit(grp, parent, cap, var, strDef, hint);
					}
					else
					{
						tclMisc->setTclResult(interp, QObject::tr("Lineedit %1 in tab %2 already exists.").arg(cap).arg(grp));
						return TCL_ERROR;
					}
					break;
				}
				case 3:
				{
					if (parent.isNull())
					{
						parent = grp;
					}
					if (!ConfigDialog::controlExists(grp, cap))
					{
						ConfigDialog::addVGroupBox(grp, parent, cap);
					}
					else
					{
						tclMisc->setTclResult(interp, QObject::tr("vbox %1 in tab %2 already exists.").arg(cap).arg(grp));
						return TCL_ERROR;
					}
					break;
				}
				case 4:
				{
					if (parent.isNull())
					{
						parent = grp;
					}
					if (!ConfigDialog::controlExists(grp, cap))
					{
						ConfigDialog::addHGroupBox(grp, parent, cap);
					}
					else
					{
						tclMisc->setTclResult(interp, QObject::tr("hbox %1 in tab %2 already exists.").arg(cap).arg(grp));
						return TCL_ERROR;
					}
					break;
				}
				case 5:
				{
					if (!cols.isNull())
					{
						res = Tcl_GetInt(interp, cols.latin1(), &_cols);
						if (res == TCL_ERROR)
						{
							Tcl_WrongNumArgs(interp, 0, NULL, "Columns number should be an integer value.");
							return TCL_ERROR;
						}
					}
					else
					{
						_cols = 3;
					}
					if (parent.isNull())
					{
						parent = grp;
					}
					if (!ConfigDialog::controlExists(grp, cap))
					{
						ConfigDialog::addGrid(grp, parent, cap, _cols);
					}
					else
					{
						tclMisc->setTclResult(interp, QObject::tr("grid %1 in tab %2 already exists.").arg(cap).arg(grp));
						return TCL_ERROR;
					}
					break;
				}
				case 6:
				{
					if (parent.isNull())
					{
						parent = grp;
					}
					if (var.isNull())
					{
						tclMisc->setTclResult(interp, QObject::tr("Colorbutton type needs a variable name to be given."));
						return TCL_ERROR;
					}
					QColor color(strDef);
					if (!color.isValid())
					{
						tclMisc->setTclResult(interp, QObject::tr("-default option argument for colorbutton type needs to be valid color name or #RRGGBB value."));
						return TCL_ERROR;
					}
					if (!ConfigDialog::controlExists(grp, cap))
					{
						ConfigDialog::addColorButton(grp, parent, cap, var, color, hint);
					}
					else
					{
						tclMisc->setTclResult(interp, QObject::tr("Colorbutton %1 in tab %2 already exists.").arg(cap).arg(grp));
						return TCL_ERROR;
					}
					break;
				}
				case 7:
				{
					if (parent.isNull())
					{
						parent = grp;
					}
					if (var.isNull())
					{
						tclMisc->setTclResult(interp, QObject::tr("Combobox type needs a variable name to be given."));
						return TCL_ERROR;
					}
					if (opts.isNull())
					{
						tclMisc->setTclResult(interp, QObject::tr("Combobox type needs options to be given."));
						return TCL_ERROR;
					}
					if (vals.isNull())
					{
						tclMisc->setTclResult(interp, QObject::tr("Combobox type needs values to be given."));
						return TCL_ERROR;
					}
					QStringList options( tclMisc->tclListToQtList(opts) ),
								values( tclMisc->tclListToQtList(vals) );

					if (options.count() == 0)
					{
						tclMisc->setTclResult(interp, QObject::tr("Combobox type needs options to be given with at least one element.\nOptions argument should be proper TCL list."));
						return TCL_ERROR;
					}
					if (values.count() == 0)
					{
						tclMisc->setTclResult(interp, QObject::tr("Combobox type needs values to be given with at least one element.\nValues argument should be proper TCL list."));
						return TCL_ERROR;
					}
					if (options.count() != values.count())
					{
						tclMisc->setTclResult(interp, QObject::tr("Options and values list for combobox type should contain same number of elements."));
						return TCL_ERROR;
					}
					if (strDef.isNull())
					{
						strDef = values[0];
					}
					else if (values.findIndex(strDef) == -1)
					{
						tclMisc->setTclResult(interp, QObject::tr("Default value for combobox type should be also included in values list."));
						return TCL_ERROR;
					}

					if (!ConfigDialog::controlExists(grp, cap))
					{
						ConfigDialog::addComboBox(grp, parent, cap, var, options, values, strDef, hint);
					}
					else
					{
						tclMisc->setTclResult(interp, QObject::tr("ComboBox %1 in tab %2 already exists.").arg(cap).arg(grp));
						return TCL_ERROR;
					}
					break;
				}
				case 8:
				{
					if (parent.isNull())
					{
						parent = grp;
					}
					if (var.isNull())
					{
						tclMisc->setTclResult(interp, QObject::tr("Hotkey type needs a variable name to be given."));
						return TCL_ERROR;
					}
					if (strDef.isNull())
					{
						tclMisc->setTclResult(interp, QObject::tr("Hotkey type needs a default value to be given."));
						return TCL_ERROR;
					}
					if (!ConfigDialog::controlExists(grp, cap))
					{
						ConfigDialog::addHotKeyEdit(grp, parent, cap, var, strDef, hint);
					}
					else
					{
						tclMisc->setTclResult(interp, QObject::tr("Hotkey %1 in tab %2 already exists.").arg(cap).arg(grp));
						return TCL_ERROR;
					}
					break;
				}
				case 9:
				{
					if (parent.isNull())
					{
						parent = grp;
					}
					if (var.isNull())
					{
						tclMisc->setTclResult(interp, QObject::tr("Hradiogroup type needs a variable name to be given."));
						return TCL_ERROR;
					}
					if (opts.isNull())
					{
						tclMisc->setTclResult(interp, QObject::tr("Hradiogroup type needs options to be given."));
						return TCL_ERROR;
					}
					if (vals.isNull())
					{
						tclMisc->setTclResult(interp, QObject::tr("Hradiogroup type needs values to be given."));
						return TCL_ERROR;
					}
					QStringList options( tclMisc->tclListToQtList(opts) ),
								values( tclMisc->tclListToQtList(vals) );

					if (options.count() == 0)
					{
						tclMisc->setTclResult(interp, QObject::tr("Hradiogroup type needs options to be given with at least one element.\nOptions argument should be proper TCL list."));
						return TCL_ERROR;
					}
					if (values.count() == 0)
					{
						tclMisc->setTclResult(interp, QObject::tr("Hradiogroup type needs values to be given with at least one element.\nValues argument should be proper TCL list."));
						return TCL_ERROR;
					}
					if (options.count() != values.count())
					{
						tclMisc->setTclResult(interp, QObject::tr("Options and values list for hradiobox type should contain same number of elements."));
						return TCL_ERROR;
					}
					if (values.findIndex(strDef) == -1)
					{
						tclMisc->setTclResult(interp, QObject::tr("Default value for hradiogroup type should be also included in values list."));
						return TCL_ERROR;
					}

					if (!ConfigDialog::controlExists(grp, cap))
					{
						ConfigDialog::addHRadioGroup(grp, parent, cap, var, options, values, strDef, hint);
					}
					else
					{
						tclMisc->setTclResult(interp, QObject::tr("Hradiogroup %1 in tab %2 already exists.").arg(cap).arg(grp));
						return TCL_ERROR;
					}
					break;
				}
				case 10:
				{
					if (parent.isNull())
					{
						parent = grp;
					}
					if (var.isNull())
					{
						tclMisc->setTclResult(interp, QObject::tr("Vradiogroup type needs a variable name to be given."));
						return TCL_ERROR;
					}
					if (opts.isNull())
					{
						tclMisc->setTclResult(interp, QObject::tr("Vradiogroup type needs options to be given."));
						return TCL_ERROR;
					}
					if (vals.isNull())
					{
						tclMisc->setTclResult(interp, QObject::tr("Vradiogroup type needs values to be given."));
						return TCL_ERROR;
					}
					QStringList options( tclMisc->tclListToQtList(opts) ),
								values( tclMisc->tclListToQtList(vals) );

					if (options.count() == 0)
					{
						tclMisc->setTclResult(interp, QObject::tr("Vradiogroup type needs options to be given with at least one element.\nOptions argument should be proper TCL list."));
						return TCL_ERROR;
					}
					if (values.count() == 0)
					{
						tclMisc->setTclResult(interp, QObject::tr("Vradiogroup type needs values to be given with at least one element.\nValues argument should be proper TCL list."));
						return TCL_ERROR;
					}
					if (options.count() != values.count())
					{
						tclMisc->setTclResult(interp, QObject::tr("Options and values list for vradiobox type should contain same number of elements."));
						return TCL_ERROR;
					}
					if (values.findIndex(strDef) == -1)
					{
						tclMisc->setTclResult(interp, QObject::tr("Default value for vradiogroup type should be also included in values list."));
						return TCL_ERROR;
					}

					if (!ConfigDialog::controlExists(grp, cap))
					{
						ConfigDialog::addVRadioGroup(grp, parent, cap, var, options, values, strDef, hint);
					}
					else
					{
						tclMisc->setTclResult(interp, QObject::tr("Vradiogroup %1 in tab %2 already exists.").arg(cap).arg(grp));
						return TCL_ERROR;
					}
					break;
				}
				case 11:
				{
					if (parent.isNull())
					{
						parent = grp;
					}
					if (var.isNull())
					{
						tclMisc->setTclResult(interp, QObject::tr("Selectfont type needs a variable name to be given."));
						return TCL_ERROR;
					}
					if (strDef.isNull())
					{
						tclMisc->setTclResult(interp, QObject::tr("Selectfont type needs a default value to be given."));
						return TCL_ERROR;
					}
					if (!ConfigDialog::controlExists(grp, cap))
					{
						ConfigDialog::addSelectFont(grp, parent, cap, var, strDef, hint);
					}
					else
					{
						tclMisc->setTclResult(interp, QObject::tr("Selectfont %1 in tab %2 already exists.").arg(cap).arg(grp));
						return TCL_ERROR;
					}
					break;
				}
				case 12:
				{
					if (parent.isNull())
					{
						parent = grp;
					}
					if (var.isNull())
					{
						tclMisc->setTclResult(interp, QObject::tr("Slider type needs a variable name to be given."));
						return TCL_ERROR;
					}

					int val, i_min, i_max, i_step;

					if (strDef.isNull())
					{
						val = 50;
					}
					else
					{
						if (Tcl_GetInt(interp, strDef.latin1(), &val) == TCL_ERROR)
						{
							tclMisc->setTclResult(interp, QObject::tr("Slider default value has to be an integer."));
							return TCL_ERROR;
						}
					}

					if (min.isNull())
					{
						i_min = 0;
					}
					else
					{
						if (Tcl_GetInt(interp, min.latin1(), &i_min) == TCL_ERROR)
						{
							tclMisc->setTclResult(interp, QObject::tr("Slider minimal value has to be an integer."));
							return TCL_ERROR;
						}
					}

					if (max.isNull())
					{
						i_max = 100;
					}
					else
					{
						if (Tcl_GetInt(interp, max.latin1(), &i_max) == TCL_ERROR)
						{
							tclMisc->setTclResult(interp, QObject::tr("Slider maximal value has to be an integer."));
							return TCL_ERROR;
						}
					}

					if (step.isNull())
					{
						i_step = 1;
					}
					else
					{
						if (Tcl_GetInt(interp, step.latin1(), &i_step) == TCL_ERROR)
						{
							tclMisc->setTclResult(interp, QObject::tr("Slider step value has to be an integer."));
							return TCL_ERROR;
						}
					}

					if (!ConfigDialog::controlExists(grp, cap))
					{
						ConfigDialog::addSlider(grp, parent, cap, var, i_min, i_max, i_step, val, hint);
					}
					else
					{
						tclMisc->setTclResult(interp, QObject::tr("Slider %1 in tab %2 already exists.").arg(cap).arg(grp));
						return TCL_ERROR;
					}
					break;
				}
				case 13:
				{
					if (parent.isNull())
					{
						parent = grp;
					}
					if (var.isNull())
					{
						tclMisc->setTclResult(interp, QObject::tr("Spinbox type needs a variable name to be given."));
						return TCL_ERROR;
					}

					int val, i_min, i_max, i_step;

					if (strDef.isNull())
					{
						val = 50;
					}
					else
					{
						if (Tcl_GetInt(interp, strDef.latin1(), &val) == TCL_ERROR)
						{
							tclMisc->setTclResult(interp, QObject::tr("Spinbox default value has to be an integer."));
							return TCL_ERROR;
						}
					}

					if (min.isNull())
					{
						i_min = 0;
					}
					else
					{
						if (Tcl_GetInt(interp, min.latin1(), &i_min) == TCL_ERROR)
						{
							tclMisc->setTclResult(interp, QObject::tr("Spinbox minimal value has to be an integer."));
							return TCL_ERROR;
						}
					}

					if (max.isNull())
					{
						i_max = 100;
					}
					else
					{
						if (Tcl_GetInt(interp, max.latin1(), &i_max) == TCL_ERROR)
						{
							tclMisc->setTclResult(interp, QObject::tr("Spinbox maximal value has to be an integer."));
							return TCL_ERROR;
						}
					}

					if (step.isNull())
					{
						i_step = 1;
					}
					else
					{
						if (Tcl_GetInt(interp, step.latin1(), &i_step) == TCL_ERROR)
						{
							tclMisc->setTclResult(interp, QObject::tr("Spinbox step value has to be an integer."));
							return TCL_ERROR;
						}
					}

					if (!ConfigDialog::controlExists(grp, cap))
					{
						ConfigDialog::addSpinBox(grp, parent, cap, var, i_min, i_max, i_step, val, hint);
					}
					else
					{
						tclMisc->setTclResult(interp, QObject::tr("Slider %1 in tab %2 already exists.").arg(cap).arg(grp));
						return TCL_ERROR;
					}
					break;
				}
				case 14:
				{
					if (parent.isNull())
					{
						parent = grp;
					}
					if (var.isNull())
					{
						tclMisc->setTclResult(interp, QObject::tr("Text type needs a variable name to be given."));
						return TCL_ERROR;
					}
					if (strDef.isNull())
					{
						tclMisc->setTclResult(interp, QObject::tr("Text type needs a default value to be given."));
						return TCL_ERROR;
					}
					if (!ConfigDialog::controlExists(grp, cap))
					{
						ConfigDialog::addTextEdit(grp, parent, cap, var, strDef, hint);
					}
					else
					{
						tclMisc->setTclResult(interp, QObject::tr("Text %1 in tab %2 already exists.").arg(cap).arg(grp));
						return TCL_ERROR;
					}
					break;
				}
				default:
					tclMisc->setTclResult(interp, QObject::tr("Type should be one of following:\n%1").arg(types.join(", ")));
					return TCL_ERROR;
			}
			break;
		}
		case 2:
		{
			if (argc > 4 || argc < 3)
			{
				Tcl_WrongNumArgs(interp, 0, NULL, "config exists tabName ?caption?");
				return TCL_ERROR;
			}
			grp = QString::fromUtf8(argv[2]);
			if (argc == 4)
			{
				cap = QString::fromUtf8(argv[3]);
				if (ConfigDialog::controlExists(grp, cap))
				{
					Tcl_SetResult(interp, "1", TCL_STATIC);
				}
				else
				{
					Tcl_SetResult(interp, "0", TCL_STATIC);
				}
			}
			else
			{
				if (ConfigDialog::tabExists(grp))
				{
					Tcl_SetResult(interp, "1", TCL_STATIC);
				}
				else
				{
					Tcl_SetResult(interp, "0", TCL_STATIC);
				}
			}
			break;
		}
		case 3:
		{
			if (argc > 4 || argc < 3)
			{
				Tcl_WrongNumArgs(interp, 0, NULL, "config delete tabName ?caption?");
				return TCL_ERROR;
			}
			grp = QString::fromUtf8(argv[2]);
			if (argc == 4)
			{
				cap = QString::fromUtf8(argv[3]);
				if (!ConfigDialog::controlExists(grp, cap))
				{
					err = QObject::tr("Widget %1 (in %2) doesn't exist.").arg(cap).arg(grp);
					tclMisc->setTclResult(interp, err);
					return TCL_ERROR;
				}
				ConfigDialog::removeControl(grp, cap);
			}
			else
			{
				if (!ConfigDialog::tabExists(grp))
				{
					tclMisc->setTclResult(interp, QObject::tr("Tab %1 doesn't exists").arg(grp));
					return TCL_ERROR;
				}
				ConfigDialog::removeTab(grp);
			}
			break;
		}
		default:
			tclMisc->setTclResult(interp, QObject::tr("Command should be one of following:\n%1").arg(cmds.join(", ")));
			return TCL_ERROR;
	}
	return TCL_OK;
}
/*
Nie mozna zidentyfikowac po sygnale, ktory button zostal wcisniety, wiec nie mozna przyporzadkowac
odpowiedniego do niego kodu.

int Tcl_CmdToolBar(void* clientData, Tcl_Interp *interp, int argc, const char *argv[])
{
	if (argc < 2)
	{
		Tcl_WrongNumArgs(interp, 0, NULL, "config command arguments");
		return TCL_ERROR;
	}
	QStringList cmds = QStringList::split(" ", "add delete exists");
	switch (cmds.findIndex(argv[1]))
	{
		case 0:
		{
			if (argc < 2)
			{
				Tcl_WrongNumArgs(interp, 0, NULL, "config add ");
				return TCL_ERROR;
			}
			Kadu::addToolButton();
			break;
		}
		case 1:
		{
			break;
		}
		case 2:
		{
			break;
		}
		default:
		{
		}
	}
	return TCL_OK;
}
*/

/* Tcl Command usermenu
GLOBALSYNTAX: usermenu komenda ?argumenty?
DESCRIPTION:
Komenda do zarzdzania menu kontekstowym listy kontaktw.

SUBSYNTAX: usermenu add nazwa ?komenda? ?ikona ?pozycja??
DESCRIPTION:
Dodaje nowy element do menu o podanej <I>nazwie</I>, ktremu przypisana zostanie
podana komenda. Opcjonalnie mona poda <I>ikon</I>, oraz <I>pozycj<I> (liczc od 0)
na ktrej powinien znale si nowy element. Element moe by rwnierz nazw
popup menu, utowrzonego wczeniej komend <B>popup</B>. Wtedy argument <I>komendy</I>
zostaje pominity.

SUBSYNTAX: usermenu delete nazwa
DESCRIPTION:
Usuwa element menu o podanej nazwie. Jeli menu o podanej <I>nazwie</I> nie istnieje, zwraca bd.<br>

SUBSYNTAX: usermenu exists nazwa
DESCRIPTION:
Jeli element menu o <I>nazwie</I> istnieje, zwraca 1, inaczej zwraca 0.

SUBSYNTAX: usermenu disable nazwa
DESCRIPTION:
Wycza element menu. W takim stanie uytkownik nie moe na nim klikn, a kolor jego tytuu zmienia si na janiejszy.
Jeli element o podanej <I>nazwie</I> nie istnieje, zwraca bd.<br>

SUBSYNTAX: usermenu enable nazwa
DESCRIPTION:
Wcza element menu. Jest to domylny tryb pracy menu zaraz po stworzeniu.
Jeli element o podanej <I>nazwie</I> nie istnieje, zwraca bd.<br>

SUBSYNTAX: usermenu isenabled nazwa
DESCRIPTION:
Zwraca 1, jeli element menu o podanej <I>nazwie</I> jest wczone, inaczej zwraca 0.

SUBSYNTAX: usermenu listitems
DESCRIPTION:
Zwraca nazwy wszystkich elementw z menu w postaci listy TCL.

SUBSYNTAX: usermenu selected ?-altname?
DESCRIPTION:
Zwraca UIN, lub list UIN'w wybranych przez uytkownika na licie kontaktw. Jeli podana zostanie
opcja -altname, zwracane elementy to nazwy alternatywne (wywietlane), zamiast UIN'w.
*/
int Tcl_CmdUserMenu(void* clientData, Tcl_Interp *interp, int argc, const char *argv[])
{
	if (argc < 2)
	{
		Tcl_WrongNumArgs(interp, 0, NULL, "usermenu command ?arguments?");
		return TCL_ERROR;
	}
	QStringList cmds = QStringList::split(" ", "add delete exists disable enable list selected");
	QString name = QString::fromUtf8(argv[2]), err, icon;
	int id = UserBox::userboxmenu->getItem(name), pos = -1;
	QCString utf;
	Tcl_Obj* result;
	int sessID = *((int*) clientData);
	TclSession *tsess = tclSess[sessID];

	switch (cmds.findIndex(argv[1]))
	{
		case 0:
		{
			if (tsess->tclPopups.contains(name))
			{
				if (argc < 3 || argc > 5)
				{
					Tcl_WrongNumArgs(interp, 0, NULL, "usermenu add name ?command? ?icon ?position??");
					return TCL_ERROR;
				}
				if (argc > 3)
				{
					icon = QString::fromUtf8(argv[3]);
					if (argc == 5)
					{
						bool ok;
						pos = QString(argv[4]).toInt(&ok);
						if (!ok)
						{
							if (!QString::compare(argv[4], "end"))
							{
								pos = -1;
							}
							else
								tclMisc->setTclResult(interp, QObject::tr("Position argument in 'usermenu add' command should be an integer value."));
						}
					}
				}
				id = UserBox::userboxmenu->insertItem(icons_manager.loadIcon(icon), name, tsess->tclPopups[name].menu, -1, pos);
			}
			else
			{
				if (argc < 4 || argc > 6)
				{
					Tcl_WrongNumArgs(interp, 0, NULL, "usermenu add name ?command? ?icon ?position??");
					return TCL_ERROR;
				}
				if (argc > 4)
				{
					icon = QString::fromUtf8(argv[4]);
					if (argc == 6)
					{
						bool ok;
						pos = QString(argv[5]).toInt(&ok);
						if (!ok)
						{
							if (!QString::compare(argv[5], "end"))
							{
								pos = -1;
							}
							else
								tclMisc->setTclResult(interp, QObject::tr("Position argument in 'usermenu add' command should be an integer value."));
						}
					}
				}
				id = UserBox::userboxmenu->insertItem(icons_manager.loadIcon(icon), QString::fromUtf8(argv[2]), tsess, SLOT(handleUserMenu(int)), 0, -1, pos);
				tsess->tclMenuCmds[id] = QString::fromUtf8(argv[3]);
			}
			break;
		}
		case 1:
		{
			if (argc != 3)
			{
				Tcl_WrongNumArgs(interp, 0, NULL, "usermenu delete name");
				return TCL_ERROR;
			}
			if (id != -1)
			{
				UserBox::userboxmenu->removeItem(id);
				tsess->tclMenuCmds.remove(id);
			}
			else
			{
				tclMisc->setTclResult(interp, QObject::tr("User menu item '%1' doesn't exist.").arg(name));
				return TCL_ERROR;
			}
			break;
		}
		case 2:
		{
			if (argc != 3)
			{
				Tcl_WrongNumArgs(interp, 0, NULL, "usermenu exists name");
				return TCL_ERROR;
			}
			if (id != -1)
			{
				Tcl_SetResult(interp, "1", TCL_STATIC);
			}
			else
			{
				Tcl_SetResult(interp, "0", TCL_STATIC);
			}
			return TCL_OK;
			break;
		}
		case 3:
		{
			if (argc != 3)
			{
				Tcl_WrongNumArgs(interp, 0, NULL, "usermenu disable name");
				return TCL_ERROR;
			}
			if (id != -1)
			{
				if (UserBox::userboxmenu->isItemEnabled(id))
				{
					UserBox::userboxmenu->setItemEnabled(id, FALSE);
				}
				else
				{
					tclMisc->setTclResult(interp, QObject::tr("User menu item '%1' is already disabled.").arg(name));
					return TCL_ERROR;
				}
			}
			else
			{
				tclMisc->setTclResult(interp, QObject::tr("User menu item '%1' doesn't exist.").arg(name));
				return TCL_ERROR;
			}
			break;
		}
		case 4:
		{
			if (argc != 3)
			{
				Tcl_WrongNumArgs(interp, 0, NULL, "usermenu enable name");
				return TCL_ERROR;
			}
			if (id != -1)
			{
				if (!UserBox::userboxmenu->isItemEnabled(id))
				{
					UserBox::userboxmenu->setItemEnabled(id, TRUE);
				}
				else
				{
					tclMisc->setTclResult(interp, QObject::tr("User menu item '%1' is already enabled.").arg(name));
					return TCL_ERROR;
				}
			}
			else
			{
				tclMisc->setTclResult(interp, QObject::tr("User menu item '%1' doesn't exist.").arg(name));
				return TCL_ERROR;
			}
			break;
		}
		case 5:
		{
			if (argc != 2)
			{
				Tcl_WrongNumArgs(interp, 0, NULL, "usermenu list");
				return TCL_ERROR;
			}
			for ( uint i = 0; i <= UserBox::userboxmenu->count(); i++ )
			{
				utf = UserBox::userboxmenu->text(UserBox::userboxmenu->idAt(i)).utf8();
				result = Tcl_NewStringObj((const char* const)utf, utf.length());
				if (utf.length())
					Tcl_AppendElement(interp, Tcl_GetString(result));
			}
			return TCL_OK;
			break;
		}
		case 6:
		{
			if (argc > 3)
			{
				Tcl_WrongNumArgs(interp, 0, NULL, "usermenu selected ?-altname?");
				return TCL_ERROR;
			}
			bool alt = FALSE;
			if (argc == 3)
			{
				if (!QString::compare("-altname", argv[2]))
					alt = TRUE;
			}
			if (kadu->userbox()->getSelectedUins().count())
			{
				if (alt)
				{
					QStringList uins = kadu->userbox()->getSelectedAltNicks();
					for ( uint i = 0; i < uins.count(); i++ )
						tclMisc->appendTclResult(interp, uins[i]);
				}
				else
				{
					UinsList uins = kadu->userbox()->getSelectedUins();
					for ( uint i = 0; i < uins.count(); i++ )
						Tcl_AppendElement(interp, (char*)QString::number(uins[i]).latin1());
				}
			}
			return TCL_OK;
		}
		case 7:
		{
			if (argc != 3)
			{
				Tcl_WrongNumArgs(interp, 0, NULL, "usermenu deleteindex index");
				return TCL_ERROR;
			}
			bool ok;
			int idx = QString(argv[2]).toInt(&ok);
			if (!ok)
			{
				tclMisc->setTclResult(interp, QObject::tr("User menu item with index '%1' doesn't exist.").arg(argv[2]));
				return TCL_ERROR;
			}
			int ident = UserBox::userboxmenu->idAt(idx);
			UserBox::userboxmenu->removeItem(ident);
			tsess->tclMenuCmds.remove(ident);
			break;
		}
		default:
			tclMisc->setTclResult(interp, QObject::tr("Command should be one of following:\n%1").arg(cmds.join(", ")));
			return TCL_ERROR;
	}
	return TCL_OK;
}

/* Tcl Command popup
GLOBALSYNTAX: popup komenda ?nazwa? ?argumenty?
DESCRIPTION:
popup suy do tworzenia gotowego do uycia menu w pamici. Takowe menu moe zosta
uyte w dowolnym miejscu (w ktrym wchodzi w gre menu), jak na belce gwnego okna Kadu,
lub w menu kontekstowym listy kontaktw. Jeli chce si go uy w wielu miejscach jednoczenie,
nic nie stoi na przeszkodzie, ani te nie trzeba budowa go od nowa. Istnieje ono cay czas w pamici,
a do usunicia komend popup <B>delete</B>.

SUBSYNTAX: popup new nazwa
DESCRIPTION:
Tworzy nowe menu o podanej <I>nazwie</I>.

SUBSYNTAX: popup delete nazwa
Niszczy menu i zwalnia pami. Wczeniej naley usun wszelkie wystpienia tego menu
w aplikacji.

SUBSYNTAX: popup exists nazwa
DESCRIPTION:
Zwraca 1 jeli menu o podanej <I>nazwie</I> istnieje, inaczej zwraca 0.

SUBSYNTAX: popup list
DESCRIPTION:
Zwraca list nazw menu istniejcych w pamici.

SUBSYNTAX: popup listitems nazwa
DESCRIPTION:
Zwraca list elementw zawartych w menu o podanej <I>nazwie</I>.

SUBSYNTAX: popup additem nazwa tytu komenda ?ikona ?pozycja ?skrt???
DESCRIPTION:
Dodaje nowy element o danym <I>tytule</I> do menu o <I>nazwie</I>. Po wybraniu tego elementu
przez uytkownika, wykonana zostanie komenda. Opcjonalnie mona poda nazw ikony dla elementu,
pozycj na ktrej ma znajdowa si nowoutworzony element, oraz skrt klawiszowy.

SUBSYNTAX: popup addseparator nazwa
DESCRIPTION:
Dodaje belk oddzielajc do menu o podanej <I>nazwie</I>.

SUBSYNTAX: popup delitem nazwa tytu
DESCRIPTION:
Usuwa element o podanym <I>tytule</I> z menu o podanej <I>nazwie</I>.

SUBSYNTAX: popup delindex nazwa indeks
DESCRIPTION:
Usuwa element o numerze podanym jako indeks, liczc od 0, z menu o podanej <I>nazwie</I>.

SUBSYNTAX: popup disableitem nazwa tytu
DESCRIPTION:
Wycza element o <I>tytule</I> w menu o <I>nazwie</I>.

SUBSYNTAX: popup enableitem nazwa tytu
DESCRIPTION:
Wcza element o <I>tytule</I> w menu o <I>nazwie</I>.

SUBSYNTAX: popup isitemenabled nazwa tytu
DESCRIPTION:
Zwraca 1, jeli element o podanym <I>tytule</I> z menu o podanej <I>nazwie</I> jest wczony,
inaczej zwraca 0.

SUBSYNTAX: popup itemconf nazwa tytu ?-newname nowyTytu? ?-cmd komenda? ?-icon ikona? ?-accel skrt?
DESCRIPTION:
Rekonfiguruje ustawienia elementu tytu z menu o podanej <I>nazwie</I>, wg. uytych opcji.
Jeli adna z opcji nie jest podana, zwraca list zawierajc tytu elementu i komend do niego
przyporzdkowan w takiej wasnie kolejnoci.

SUBSYNTAX: popup addpopup nazwa nazwaPopupa ?ikona?
DESCRIPTION:
Dodaje do menu o podanej <I>nazwie</I> inne menu o <I>nazwie popupa</I>, w postaci jednego elementu.
Opcjonalnie mona poda </I>ikon<I>.
*/
int Tcl_CmdPopup(void* clientData, Tcl_Interp *interp, int argc, const char *argv[])
{
	if (argc < 2)
	{
		Tcl_WrongNumArgs(interp, 0, NULL, "popup command ?name? ?arguments?");
		return TCL_ERROR;
	}
	QStringList cmds = QStringList::split(" ", "new delete exists list listitems additem addseparator delitem disableitem enableitem isitemenabled delindex addpopup itemconf");
	QString name = QString::fromUtf8(argv[2]), err, icon, name2;
	QCString utf;
	Tcl_Obj* result;
	int id, itemId = -1, sessID = *((int*) clientData);
	QPopupMenu *menu;
	TclSession *tsess = tclSess[sessID];

	if (argc > 2)
	{
		if (tsess->tclPopups.contains(name))
		{
			for ( uint i = 0; i <= tsess->tclPopups[name].menu->count(); i++ )
			{
				if ( ! tsess->tclPopups[name].menu->text(tsess->tclPopups[name].menu->idAt(i)).compare(QString::fromUtf8(argv[3])) )
				{
					itemId = tsess->tclPopups[name].menu->idAt(i);
					break;
				}
			}
		}
	}

	switch (cmds.findIndex(argv[1]))
	{
		case 0:
		{
			if (argc != 3)
			{
				Tcl_WrongNumArgs(interp, 0, NULL, "popup new name");
				return TCL_ERROR;
			}
			menu = new QPopupMenu(kadu);
			if (tsess->tclPopups.contains(name))
			{
				tclMisc->setTclResult(interp, QObject::tr("Popup %1 already exists.").arg(name));
				return TCL_ERROR;
			}
			tsess->tclPopups[name].menu = menu;
			QObject::connect(tsess->tclPopups[name].menu, SIGNAL(activated(int)), tsess, SLOT(handlePopup(int)));
			break;
		}
		case 1:
		{
			if (argc != 3)
			{
				Tcl_WrongNumArgs(interp, 0, NULL, "popup delete name");
				return TCL_ERROR;
			}
			if (!tsess->tclPopups.contains(name))
			{
				tclMisc->setTclResult(interp, QObject::tr("Popup %1 doesn't exist.").arg(name));
				return TCL_ERROR;
			}
			QObject::disconnect(tsess->tclPopups[name].menu, SIGNAL(activated(int)), tsess, SLOT(handlePopup(int)));
			delete tsess->tclPopups[name].menu;
			tsess->tclPopups.remove(name);
			break;
		}
		case 2:
		{
			if (argc != 3)
			{
				Tcl_WrongNumArgs(interp, 0, NULL, "popup exists name");
				return TCL_ERROR;
			}
			if (tsess->tclPopups.contains(name))
			{
				Tcl_SetResult(interp, "1", TCL_STATIC);
			}
			else
			{
				Tcl_SetResult(interp, "0", TCL_STATIC);
			}
			break;
		}
		case 3:
		{
			if (argc != 2)
			{
				Tcl_WrongNumArgs(interp, 0, NULL, "popup list");
				return TCL_ERROR;
			}
			for ( uint i = 0; i < tsess->tclPopups.keys().count(); i++ )
			{
				utf = tsess->tclPopups.keys()[i];
				result = Tcl_NewStringObj((const char* const)utf, utf.length());
				if (utf.length())
					Tcl_AppendElement(interp, Tcl_GetString(result));
			}
			break;
		}
		case 4:
		{
			if (argc != 3)
			{
				Tcl_WrongNumArgs(interp, 0, NULL, "popup listitems name");
				return TCL_ERROR;
			}
			if (tsess->tclPopups.contains(name))
			{
				for ( uint i = 0; i <= tsess->tclPopups[name].menu->count(); i++ )
				{
					utf = tsess->tclPopups[name].menu->text(tsess->tclPopups[name].menu->idAt(i)).utf8();
					result = Tcl_NewStringObj((const char* const)utf, utf.length());
					if (utf.length())
						Tcl_AppendElement(interp, Tcl_GetString(result));
				}
				return TCL_OK;
			}
			else
			{
				tclMisc->setTclResult(interp, QObject::tr("Popup %1 doesn't exist.").arg(name));
				return TCL_ERROR;
			}
			break;
		}
		case 5:
		{
			if (argc < 5 || argc > 8)
			{
				Tcl_WrongNumArgs(interp, 0, NULL, "popup additem name itemName itemCommand ?icon ?position ?accelerator???");
				return TCL_ERROR;
			}
			if (tsess->tclPopups.contains(name))
			{
				int pos = -1;
				if (argc > 5)
				{
					icon = QString::fromUtf8(argv[5]);
					if (argc == 7)
					{
						bool ok;
						pos = QString(argv[6]).toUInt(&ok);
						if (!ok)
						{
							if (!QString::compare(argv[6], "end"))
							{
								pos = -1;
							}
							else
								tclMisc->setTclResult(interp, QObject::tr("Position argument in 'popup' command should be an integer value."));
						}
					}
				}
				id = tsess->tclPopups[name].menu->insertItem(icons_manager.loadIcon(icon), QString::fromUtf8(argv[3]), -1, pos);
				if (argc == 8)
					tsess->tclPopups[name].menu->setAccel(QKeySequence(argv[7]), id);

				tsess->tclPopupCmds[id] = QString::fromUtf8(argv[4]);
			}
			else
			{
				tclMisc->setTclResult(interp, QObject::tr("Popup %1 doesn't exist.").arg(name));
				return TCL_ERROR;
			}
			break;
		}
		case 6:
		{
			if (argc != 3)
			{
				Tcl_WrongNumArgs(interp, 0, NULL, "popup addseparator name");
				return TCL_ERROR;
			}
			if (tsess->tclPopups.contains(name))
			{
				tsess->tclPopups[name].menu->insertSeparator();
			}
			else
			{
				tclMisc->setTclResult(interp, QObject::tr("Popup %1 doesn't exist.").arg(name));
				return TCL_ERROR;
			}
			break;
		}
		case 7:
		{
			if (argc != 4)
			{
				Tcl_WrongNumArgs(interp, 0, NULL, "popup delitem name itemName");
				return TCL_ERROR;
			}
			if (tsess->tclPopups.contains(name))
			{
				if (itemId == -1)
				{
					tclMisc->setTclResult(interp, QObject::tr("Popup item '%1' doesn't exist.").arg(QString::fromUtf8(argv[3])));
					return TCL_ERROR;
				}
				tsess->tclPopups[name].menu->removeItem(itemId);
				tsess->tclPopupCmds[id].remove(itemId);
			}
			else
			{
				tclMisc->setTclResult(interp, QObject::tr("Popup %1 doesn't exist.").arg(name));
				return TCL_ERROR;
			}
			break;
		}
		case 8:
		{
			if (argc != 4)
			{
				Tcl_WrongNumArgs(interp, 0, NULL, "popup disableitem name itemName");
				return TCL_ERROR;
			}
			if (tsess->tclPopups.contains(name))
			{
				if (itemId == -1)
				{
					tclMisc->setTclResult(interp, QObject::tr("Popup item '%1' doesn't exist.").arg(QString::fromUtf8(argv[3])));
					return TCL_ERROR;
				}
				if (tsess->tclPopups[name].menu->isItemEnabled(itemId))
				{
					tsess->tclPopups[name].menu->setItemEnabled(itemId, FALSE);
				}
				else
				{
					tclMisc->setTclResult(interp, QObject::tr("Popup item '%1' is already disabled.").arg(name));
					return TCL_ERROR;
				}
			}
			else
			{
				tclMisc->setTclResult(interp, QObject::tr("Popup %1 doesn't exist.").arg(name));
				return TCL_ERROR;
			}
			break;
		}
		case 9:
		{
			if (argc != 4)
			{
				Tcl_WrongNumArgs(interp, 0, NULL, "popup enableitem name itemName");
				return TCL_ERROR;
			}
			if (tsess->tclPopups.contains(name))
			{
				if (itemId == -1)
				{
					tclMisc->setTclResult(interp, QObject::tr("Popup item '%1' doesn't exist.").arg(QString::fromUtf8(argv[3])));
					return TCL_ERROR;
				}
				if (!tsess->tclPopups[name].menu->isItemEnabled(itemId))
				{
					tsess->tclPopups[name].menu->setItemEnabled(itemId, TRUE);
				}
				else
				{
					tclMisc->setTclResult(interp, QObject::tr("Popup item '%1' is already enabled.").arg(name));
					return TCL_ERROR;
				}
			}
			else
			{
				tclMisc->setTclResult(interp, QObject::tr("Popup %1 doesn't exist.").arg(name));
				return TCL_ERROR;
			}
			break;
		}
		case 10:
		{
			if (argc != 4)
			{
				Tcl_WrongNumArgs(interp, 0, NULL, "popup isitemenabled name itemName");
				return TCL_ERROR;
			}
			if (tsess->tclPopups.contains(name))
			{
				if (itemId == -1)
				{
					tclMisc->setTclResult(interp, QObject::tr("Popup item '%1' doesn't exist.").arg(QString::fromUtf8(argv[3])));
					return TCL_ERROR;
				}
				if (tsess->tclPopups[name].menu->isItemEnabled(itemId))
				{
					Tcl_SetResult(interp, "1", TCL_STATIC);
				}
				else
				{
					Tcl_SetResult(interp, "0", TCL_STATIC);
				}
			}
			else
			{
				tclMisc->setTclResult(interp, QObject::tr("Popup %1 doesn't exist.").arg(name));
				return TCL_ERROR;
			}
			break;
		}
		case 11:
		{
			if (argc != 4)
			{
				Tcl_WrongNumArgs(interp, 0, NULL, "popup delindex name index");
				return TCL_ERROR;
			}
			if (tsess->tclPopups.contains(name))
			{
				int idxID;
				if (Tcl_GetInt(interp, argv[3], &idxID))
					return TCL_ERROR;

				tsess->tclPopups[name].menu->removeItem( tsess->tclPopups[name].menu->idAt(idxID) );
				tsess->tclPopupCmds[id].remove(idxID);
			}
			else
			{
				tclMisc->setTclResult(interp, QObject::tr("Popup %1 doesn't exist.").arg(name));
				return TCL_ERROR;
			}
			break;
		}
		case 12:
		{
			if (argc < 4 || argc > 5)
			{
				Tcl_WrongNumArgs(interp, 0, NULL, "popup addpopup name popupToAdd ?icon?");
				return TCL_ERROR;
			}
			name2 = QString::fromUtf8(argv[3]);
			if (tsess->tclPopups.contains(name))
			{
				if (tsess->tclPopups.contains(name2))
				{
					if (argc == 5)
					{
						icon = QString::fromUtf8(argv[4]);
					}
					tsess->tclPopups[name].menu->insertItem(icons_manager.loadIcon(icon), name2, tsess->tclPopups[name2].menu);
				}
				else
				{
					tclMisc->setTclResult(interp, QObject::tr("Popup %1 doesn't exist.").arg(name2));
					return TCL_ERROR;
				}
			}
			else
			{
				tclMisc->setTclResult(interp, QObject::tr("Popup %1 doesn't exist.").arg(name));
				return TCL_ERROR;
			}
			break;
		}
		case 13:
		{
			if (argc < 4)
			{
				Tcl_WrongNumArgs(interp, 0, NULL, "popup itemconf name itemName ?-newname itemName? ?-cmd command? ?-icon icon? ?-accel accelerator?");
				return TCL_ERROR;
			}
			if (tsess->tclPopups.contains(name))
			{
				if (argc > 4)
				{
					bool rest = argc % 2;
					if (rest)
					{
						Tcl_WrongNumArgs(interp, 0, NULL, "popup itemconf name itemName ?-newname itemName? ?-cmd command? ?-icon icon? ?-accel accelerator?");
						return TCL_ERROR;
					}
					QStringList opts = QStringList::split(" ", "-newname -cmd -icon -accel");
					QString opt, arg;
					for ( int i = 4; i < argc; i++ )
					{
						opt = argv[i];
						i++;
						arg = QString::fromUtf8(argv[i]);
						switch (opts.findIndex(opt))
						{
							case 0:
								tsess->tclPopups[name].menu->changeItem(itemId, arg);
								break;
							case 1:
								tsess->tclPopupCmds[itemId] = arg;
								break;
							case 2:
								tsess->tclPopups[name].menu->changeItem(itemId, icons_manager.loadIcon(arg), tsess->tclPopups[name].menu->text(itemId));
								break;
							case 3:
								tsess->tclPopups[name].menu->setAccel(QKeySequence(arg), itemId);
								break;
							default:
								tclMisc->setTclResult(interp, QObject::tr("Unknown option: %1.").arg(opt));
								return TCL_ERROR;
						}
					}
				}
				else
				{
					tclMisc->appendTclResult(interp, tsess->tclPopups[name].menu->text(itemId));
					tclMisc->appendTclResult(interp, tsess->tclPopupCmds[itemId]);
				}
			}
			else
			{
				tclMisc->setTclResult(interp, QObject::tr("Popup %1 doesn't exist.").arg(name));
				return TCL_ERROR;
			}
			break;
		}
		default:
			tclMisc->setTclResult(interp, QObject::tr("Command should be one of following:\n%1").arg(cmds.join(", ")));
			return TCL_ERROR;
	}
	return TCL_OK;
}

/*

Templorary removed Tcl Command trayicon

SYNTAX: trayicon ikona
DESCRIPTION:
Komenda suca ustawianiu <I>ikony</I> zadokowanego Kadu.
*/
/*
int Tcl_CmdTrayIcon(void* clientData, Tcl_Interp *interp, int argc, const char *argv[])
{
//	if (!trayicon)
//		return TCL_OK;

	if (argc != 2)
	{
		Tcl_WrongNumArgs(interp, 0, NULL, "trayicon icon");
		return TCL_ERROR;
	}
//	QPixmap p = icons_manager.loadIcon(argv[1]);
//	trayicon->setType(p);
	return TCL_OK;
}
*/

/* Tcl Command parse
SYNTAX: parse UIN tekst
DESCRIPTION:
Podmienia w podanym <I>tekscie</I> specjalne znaki, na odpowiednie dane dotyczce podanego
<I>UINa</I>. Podmieniane znaki to:<br>
<table>
<tr><td><B>%s</B></td><td>- status</td></tr>
<tr><td><B>%d</B></td><td>- opis</td></tr>
<tr><td><B>%i</B></td><td>- adres ip</td></tr>
<tr><td><B>%v</B></td><td>- adres dns</td></tr>
<tr><td><B>%n</B></td><td>- pseudonim</td></tr>
<tr><td><B>%a</B></td><td>- wywietlany pseudonim</td></tr>
<tr><td><B>%f</B></td><td>- imi</td></tr>
<tr><td><B>%r</B></td><td>- nazwisko</td></tr>
<tr><td><B>%m</B></td><td>- numer telefonu</td></tr>
<tr><td><B>%u</B></td><td>- UIN</td></tr>
<tr><td><B>%g</B></td><td>- grupa</td></tr>
<tr><td><B>%o</B></td><td>- port (gdy za NAT'em)</td></tr>
<tr><td><B>%p</B></td><td>- port (z zewnetrznym IP)</td></tr>
<tr><td><B>%e</B></td><td>- e-mail</td></tr>
</table>
Ponadto, obsuguje ona wszystkie inne znaczniki dostpne w parserze HTML Kadu,
jak <B>{}</B>, <B>`'</B>, etc.
*/
int Tcl_CmdParse(void* clientData, Tcl_Interp *interp, int argc, const char *argv[])
{
	if (argc != 3)
	{
		Tcl_WrongNumArgs(interp, 0, NULL, "parse UIN text");
		return TCL_ERROR;
	}
	UserListElement u;
	bool conv, ok = TRUE;
	QString altName = QString::fromUtf8(argv[1]);
	int uin = altName.toUInt(&conv);

	if (conv)
	  {
		if (userlist.containsUin(uin))
		{
			 u = userlist.byUin(uin);
		}
		else
		{
			ok = FALSE;
		}
	}
	else
	{
		if (userlist.containsAltNick(altName))
		{
			 u = userlist.byAltNick(altName);
		}
		else
		{
			ok = FALSE;
		}
	}
	if (!ok)
	{
		tclMisc->setTclResult(interp, QObject::tr("User %1 isn't registered on the list.").arg(altName));
		return TCL_ERROR;
	}
	QString ret = parse( QString::fromUtf8(argv[2]).replace("'", "&apos;") , u, FALSE);
	tclMisc->setTclResult(interp, ret.replace("&apos;", "'") );
	return TCL_OK;
}

/* Tcl Command descriptions
SYNTAX: descriptions command ?arguments?
DESCRIPTION:
Komenda suca do manipulowania histori opisw Kadu. S to te opisy, ktre mona zobaczy,
gdy wybierze si status z opisem i pojawi si okno dialogowe, w ktrym wpisuje si swj opis.
Jest tam rozwijana lista, zawiera ona list opisw, do ktrych mona mie dostp za pomoc
tej komendy.

SUBSYNTAX: descriptions get ?numer?
DESCRIPTION:
Komenda <B>descriptsions get</B> pobiera wszystkie opisy i zwraca je w postaci
listy TCL, chyba e zostanie podany opcjonalny argument - liczba z zakresu skonfigurowanego
w oknie konfiguracji Kadu, ktra oznacza, ktry z kolei opis chcesz pobra.

SUBSYNTAX: descriptions set numer tekst
DESCRIPTION:
Komenda <B>descriptions set</B> pozwala na ustawianie tych opisw
wedug wasnych potrzeb. Wymagane s do niej 2 argumenty - liczba z zakresu skonfigurowanego
w oknie konfiguracji Kadu , do okrelenia, ktry wpis chce si zmieni, oraz tekst,
ktry bdzie stanowi nowy opis.

SUBSYNTAX: descriptions max
DESCRIPTION:
Zwraca liczb okrelajc maksymaln liczb wpisw do historii opisw Kadu
(liczba ta moe si zmienia, poniewa jest konfigurowalna w oknie konfiguracji Kadu).
*/
int Tcl_CmdDescriptions(void* clientData, Tcl_Interp *interp, int argc, const char *argv[])
{
	if (argc > 4 || argc < 2)
	{
		Tcl_WrongNumArgs(interp, 0, NULL, "descriptions command ?arguments?");
		return TCL_ERROR;
	}
	uint max = config_file.readUnsignedNumEntry("General", "NumberOfDescriptions");
	QString cmd = argv[1];
	if (!cmd.compare("get"))
	{
		if (argc > 3)
		{
			Tcl_WrongNumArgs(interp, 0, NULL, "descriptions get ?number?");
			return TCL_ERROR;
		}
		else if (argc == 3)
		{
			bool conv;
			uint cnt = QString(argv[2]).toUInt(&conv);
			if (!conv)
			{
				Tcl_WrongNumArgs(interp, 0, NULL, "descriptions get ?number?");
				return TCL_ERROR;
			}
			if (cnt > max)
			{
				tclMisc->setTclResult(interp, QObject::tr("Kadu descriptions history supports currently %1 descriptions. You ware trying to manage description for %2 entry in history.").arg(QString::number(max)).arg(argv[2]));
				return TCL_ERROR;
			}
			tclMisc->setTclResult(interp, defaultdescriptions[cnt]);
		}
		else
		{
			TclString* str = new TclString();
			for ( uint i = 0; i < defaultdescriptions.count(); i++ )
				str->appendElement(defaultdescriptions[i]);

			str->setResult(interp);
			delete str;
		}
	}
	else if (!cmd.compare("set"))
	{
		if (argc != 4)
		{
			Tcl_WrongNumArgs(interp, 0, NULL, "descriptions set number text");
			return TCL_ERROR;
		}
		bool conv;
		uint cnt = QString(argv[2]).toUInt(&conv);
		if (!conv)
		{
			Tcl_WrongNumArgs(interp, 0, NULL, "descriptions set number text");
			return TCL_ERROR;
		}
		if (cnt > max)
		{
			tclMisc->setTclResult(interp, QObject::tr("Kadu descriptions history supports currently %1 descriptions. You ware trying to manage description for %2 entry in history.").arg(QString::number(max)).arg(argv[2]));
			return TCL_ERROR;
		}
		defaultdescriptions[cnt] = QString::fromUtf8(argv[3]);
	}
	else if (!cmd.compare("max"))
	{
		tclMisc->setTclResult(interp, QString::number(max));
	}
	else
	{
		tclMisc->setTclResult(interp, QObject::tr("Command should be one of following: %1").arg("get, set"));
		return TCL_ERROR;
	}
	return TCL_OK;
}


/* Tcl Command search
SYNTAX: search ?opcje?
DESCRIPTION:
Komenda suca gwnie przeszukiwaniu katalogu publicznego, ale moe te otworzy okno
dialogowe do tego suace, aby uytkownik zrobi to wasnorcznie. Operacje przeszukiwania
powinny opiera si dodatkowo na zdarzeniach <B>pubdir</B>, <B>pubdirnext</B>, oraz <B>searchdone</B>,
ktre posiadaj w argumentach identyfikator przeszukiwania, ten za jest zwracany przez komend search.
Jest to potrzebne, gdy chcemy okreli, czy dane zdarzenie (jedno z trzech ww.) jest wywokonywane wanie
dla naszego wywoania search.

SUBSYNTAX: search
DESCRIPTION:
Bez adnych argumentw otwiera po prostu znane z Kadu okno dialogowe do przeszukiwania katalogu
publicznego.

SUBSYNTAX: search -uin UIN
DESCRIPTION:
Szuka w katalogu publicznym podanego UIN'u. Zwraca identyfikator przeszukiwania.

SUBSYNTAX: search -nick pseudonim -first imi -last nazwisko -city miasto -born rok_urodzenia -sex pe -from pozycja -available
DESCRIPTION:
Najczciej spotykana posta przeszkukiwania katalogu publicznego. Dane personalne poszukiwanej osoby
podaje si jako argumenty odpowiednich opcji. Na uwag zasuguj 4 opcje:<br>
<B>-born</B> - naley poda rok urodzenia w formie 19xx (lub 20xx)<br>
<B>-sex</B> - pe mona okreli na kilka sposobw: <B>m</B>, <B>male</B>, <B>man</B> - oznacza mczyzne,
natomiast <B>f</B>, <B>female</B>, <B>woman</B> - oznacza kobiet.
<B>-from</B> - jest to pozycja od ktrej naley rozpocz przeszukiwanie. Opcja ta jest zazwyczaj uywana
jedynie w przypadku, gdy liczba odpowiedzi pasujcych do zapytania przekracza 20. Wtedy wywoywane jest
zdarzenie <B>pubdirnext</B> ktre zawiera pozycj, od ktrej naley rozpocz dalsze poszukiwania tak,
aby nie powtrzyy si te odpowiedzi, ktre ju mamy. <B>UWAGA!</B> Naley uwaa z automatyzacj
przeszukiwania katalogu a do znalezienia wszystkich moliwych odpowiedzi, gdy w niektrych moe
ich by na prawd duo i serwer GG moe zablokowa dostp do katalogu publicznego naszemu numerowi
na jaki czas.<br>
<B>-available</B> - jedyna opcja nie wymagajca argumentu. Jej uycie powoduje otrzymanie odpowiedzi
jedynie o kontaktach koystajcych w tym momencie z GG.<br><br>
Jedno z dwch zdarze musi by zwrcone po wywoaniu komendy <B>search</B> (prcz zdarzenia <B>pubdir</B>):
<B>pubdirnext</B> lub <B>searchdone</B>.

EXAMPLE:
set mojID ""
proc a {id uin nick name born city status} {
	global mojID
	if {$mojID == $id} {
		puts "Znaleziono: <$uin> Imi $name, ksywka $nick, miasto $city."
	}
}
proc b {id position} {
	global mojID
	if {$mojID == $id} {
		set mojID [search -born 1984 -available -from $position]
	}
}
proc c {id} {
	global mojID
	if {$mojID == $id} {
		puts "Koniec szukania."
		set mojID ""
	}
}

on results 1 a
on searchnext 1 b
on searchdone 1 c

set mojID [search -born 1984 -available]
ENDOFEXAMPLE
*/
int Tcl_CmdSearch(void* clientData, Tcl_Interp *interp, int argc, const char *argv[])
{
	if (argc > 16)
	{
		Tcl_WrongNumArgs(interp, 0, NULL, "search ?options?");
		return TCL_ERROR;
	}
	QStringList options = QStringList::split(" ", "-nick -last -first -city -born -bornto -sex -from -available");
	if (argc > 1)
	{
		if (gadu->status().isOffline())
		{
			tclMisc->setTclResult(interp, QObject::tr("Cannot search in public directory while you're offline."));
			return TCL_ERROR;
		}
		int from = 0;
		QString uin;
		SearchRecord* rec = new SearchRecord();
		if (!QString::compare(argv[1], "-uin"))
		{
			if (argc != 3)
			{
				Tcl_WrongNumArgs(interp, 0, NULL, "search -uin UIN");
				return TCL_ERROR;
			}
			uin = argv[2];
			bool conv;
			uin.toUInt(&conv);
			if (!conv)
			{
				Tcl_WrongNumArgs(interp, 0, NULL, "search -uin UIN");
				return TCL_ERROR;
			}
			rec->reqUin(uin);
		}
		else
		{
			int idx;
			for ( int i = 1; i < argc; i++ )
			{
				idx = options.findIndex(argv[i]);
				if (idx < 7 && idx > -1)
					i++;

				switch (idx)
				{
					case 0:
						rec->reqNickName(QString::fromUtf8(argv[i]));
						break;
					case 1:
						rec->reqLastName(QString::fromUtf8(argv[i]));
						break;
					case 2:
						rec->reqFirstName(QString::fromUtf8(argv[i]));
						break;
					case 3:
						rec->reqCity(QString::fromUtf8(argv[i]));
						break;
					case 4:
					{
						QString arg, arg2;
						arg = QString::fromUtf8(argv[i]);
						if (arg.find(" ") > -1)
						{
							arg2 = QStringList::split(" ", arg)[1];
							arg = QStringList::split(" ", arg)[0];
						}
						else
							arg2 = arg;

						bool conv1, conv2;
						arg.toUInt(&conv1);
						arg2.toUInt(&conv2);
						if (!conv1 || !conv2)
						{
							tclMisc->setTclResult(interp, QObject::tr("-born option takes one or two integers as argument"));
							return TCL_ERROR;
						}

						rec->reqBirthYear(arg, arg2);
						break;
					}
					case 5:
					{
						QString arg;
						QStringList male = QStringList::split(" ", "m male man"),
									female =  QStringList::split(" ", "f female woman");
						if (male.findIndex(arg) > -1)
						{
							rec->reqGender(false);
							continue;
						}
						else if (female.findIndex(arg) > -1)
						{
							rec->reqGender(true);
							continue;
						}
						else
						{
							tclMisc->setTclResult(interp, QObject::tr("Unknown gender: %1").arg(arg));
							return TCL_ERROR;
						}
						break;
					}
					case 6:
					{
						bool conv;
						QString arg = QString::fromUtf8(argv[i]);
						from = arg.toInt(&conv);
						if (!conv)
						{
							tclMisc->setTclResult(interp, QObject::tr("%1 is not a number. Option -from needs a number as argument.").arg(arg));
							return TCL_ERROR;
						}
						break;
					}
					case 7:
					{
						rec->reqActive();
						break;
					}
					default:
					{
						tclMisc->setTclResult(interp, QObject::tr("Option should be one of following: %1").arg(options.join(", ")));
						return TCL_ERROR;
					}
				}
			}
		}
		rec->FromUin = from;
		gadu->searchNextInPubdir(*rec);
		tclPubDirList[rec->Seq] = rec;
		Tcl_SetResult(interp, (char*)QString::number(rec->Seq).latin1(), TCL_VOLATILE);
		return TCL_OK;
	}
	else
	{
		SearchDialog* win = new SearchDialog();
		win->show();
		return TCL_OK;
	}
}

/* Tcl Command hist
SYNTAX: hist komenda ?opcje?
DESCRIPTION:
Pozwala na niektre z operacji na historii rozmw.

SUBSYNTAX: hist append ?opcje?
DESCRIPTION:
Dodaje odpowiedni wpis do historii. Nastpujce opcje s poprawne:
<DT>
<P><DT><B>-type</B><DD>
okrela typ wpisu. Moe to by <B>msg</B>, <B>sms</B>, lub <B>status</B>. Ta opcja musi by zawsze uyta.<br>
<P><DT><B>-text</B><DD>
tre dodawanego wpisu. Opcja wymagana dla wszystkich typw wpisw.<br>
<P><DT><B>-from</B><DD>
okrela jaki UIN wysa wiadomo. Jeli UIN bdzie naszym, to wiadomo automatycznie zostanie rozpoznana jako wysana przez nas. Opcja wymagana przez typ wpisu <B>msg</B>.<br>
<P><DT><B>-uins</B><DD>
lista UINw okrelajca plik historii do ktrego zostanie dodany wpis. Dla typu <B>status</B> oznacza jednoczenie UIN ktry zmienia status (dla tego typu tylko 1 wyraz z listy jest brany pod uwag). Opcja wymagana dla typu wpisu <B>msg</B>, oraz <B>status</B>.<br>
<P><DT><B>-status</B><DD>
okrela na jaki status dany UIN zmieni status. Opcja wymagana dla typu wpisu <B>status</B>.<br>
<P><DT><B>-description</B><DD>
okrela opis do statusu z opisem. Opcja wymagana dla typu wpisu <B>status</B>.<br>
<P><DT><B>-to</B><DD>
okrela numer telefonu komrkowego na ktry wysano SMS. Opcja wymagana dla typu wpisu <B>sms</B>.<br>

SUBSYNTAX: hist get ?opcje?
DESCRIPTION:
Zwraca wpis(y) z historii w postaci listy. Kady element listy zawiera kilka podelementw,
odpowiednich dla danego typu wpisu. Nastpujce opcje s poprawne:
<P><DT><B>-uins</B><DD>
lista UINw okrelajca plik historii z ktrego bd czytane wpisy. Ta opcja jest wymagana.
<P><DT><B>-type</B><DD>
typ wpisw ktre bierzemy pod uwag. Poprawne wartoci: <B>sendmsg</B> (wysane wiadomoci),
<B>rcvmsg</B> (odebrane wiadomoci), <B>msg</B> (wszystkie wiadomoci), <B>status</B> (zmiany statusw),
<B>sms</B> (wysane sms'y), <B>all</B> (wszystkie wpisy). Domylna warto to <B>all</B>.
<P><DT><B>-from</B><DD>
okrela od ktrego wpisy naley rozpocz czytanie. Ta opcja jest wymagana.
<P><DT><B>-count</B><DD>
okrela ile wpisw chcesz odczyta (domylnie 1)<br><br>

Dla typu <B>sendmsg</B> elementy listy maj posta<br>
{{<I>tre wiadomoci</I>} <I>czas</I>}<br><br>
Dla typu <B>rcvmsg</B> elementy listy maj posta<br>
{<I>UIN</I> {<I>tre wiadomoci</I>} <I>czas</I>}<br><br>
Dla typu <B>msg</B> elementy listy maj posta<br>
{<I>UIN</I> {<I>tre wiadomoci</I>} <I>czas</I>}<br><br>
Dla typu <B>status</B> elementy listy maj posta<br>
{<I>UIN</I> <I>nowy_status</I> {<I>opis</I>} <I>czas</I>}<br><br>
Dla typu <B>sms</B> elementy listy maj posta<br>
{<I>numer_telefonu</I> {<I>tre wiadomoci</I>} <I>czas</I>}<br><br>
<br>
Dla typue <B>all</B> kady element listy ma posta odopwiedni dla jego
indywidualnego typu.

SUBSYNTAX: hist count UINy
DESCRIPTION:
Zwraca liczb wszystkich wpisw dla historii o podanych <I>UINach</I>.
*/

int Tcl_CmdHistory(void* clientData, Tcl_Interp *interp, int argc, const char *argv[])
{
	if (argc < 2)
	{
		Tcl_WrongNumArgs(interp, 0, NULL, "hist command ?options?");
		return TCL_ERROR;
	}
	QStringList cmds = QStringList::split(" ", "append get count");
	switch (cmds.findIndex(argv[1]))
	{
		case 0:
		{
			QStringList opts = QStringList::split(" ", "-type -text -time -uins -description -status -to -from");
			QString type = QString::null,
					text = QString::null,
					description = QString::null,
					to = QString::null,
					arg, sender;
			UinsList uins;
			UinType from = 0;
			int status = -1;
			time_t time = QDateTime::currentDateTime().toTime_t();
			for ( int c = 2; c < argc; c++ )
			{
				arg = argv[c];
				c++;
				switch (opts.findIndex(arg))
				{
					case 0:
					{
						type = argv[c];
						break;
					}
					case 1:
					{
						text = QString::fromUtf8(argv[c]);
						break;
					}
					case 2:
					{
						// Time in UTC format
						bool conv;
						int tmp = QString(argv[c]).toUInt(&conv);
						if (!conv)
						{
							tclMisc->setTclResult(interp, QObject::tr("-time option takes an integer as argument"));
							return TCL_ERROR;
						}
						QDateTime _time;
						_time.setTime_t(tmp);
						time = _time.toTime_t();
						break;
					}
					case 3:
					{
						QStringList _uins = QStringList::split(" ", argv[c]);
						uint tmp;
						bool conv;
						for ( uint i = 0; i < _uins.count(); i++ )
						{
							tmp = _uins[i].toUInt(&conv);
							if (!conv)
							{
								tclMisc->setTclResult(interp, QObject::tr("-uins option takes a list of integer as argument"));
								return TCL_ERROR;
							}
							uins.append(tmp);
						}
						break;
					}
					case 4:
					{
						description = QString::fromUtf8(argv[c]);
						break;
					}
					case 5:
					{
						status = tclMisc->TclStatusToKaduStatus(argv[c]);
						break;
					}
					case 6:
					{
						to = argv[c];
						bool conv;
						to.toInt(&conv);
						if (!conv)
						{
							tclMisc->setTclResult(interp, QObject::tr("Mobile phone number has to be an integer."));
							return TCL_ERROR;
						}
					}
					case 7:
					{
						bool conv;
						sender = argv[c];
						from = (UinType)sender.toUInt(&conv);
						if (!conv)
						{
							tclMisc->setTclResult(interp, QObject::tr("-from option takes an integer as argument."));
							return TCL_ERROR;
						}
						break;
					}
					default:
					{
						tclMisc->setTclResult(interp, QObject::tr("Option should be one of following: %1").arg(opts.join(", ")));
						return TCL_ERROR;
					}
				}
			}

			if (type == QString::null || text == QString::null)
			{
				tclMisc->setTclResult(interp, QObject::tr("Both of -text and -type options need to be specified."));
				return TCL_ERROR;
			}

			QStringList types = QStringList::split(" ", "msg sms status");
			switch (types.findIndex(type))
			{
				case 0:
				{
					if (uins.count() < 1)
					{
						tclMisc->setTclResult(interp, QObject::tr("-uins option needs to be specified for 'msg' history type."));
						return TCL_ERROR;
					}
					if (from == 0)
					{
						tclMisc->setTclResult(interp, QObject::tr("-from option needs to be specified for 'msg' history type."));
						return TCL_ERROR;
					}
					QString myUin = config_file.readEntry("General", "UIN");
					bool own = FALSE;
					if (!sender.compare(myUin))
						own = TRUE;

					history.appendMessage(uins, from, text, own, time);
					break;
				}
				case 1:
				{
					if (to == QString::null)
					{
						tclMisc->setTclResult(interp, QObject::tr("-to option needs to be specified for 'sms' history type."));
						return TCL_ERROR;
					}
					history.appendSms(to, text);
					break;
				}
				case 2:
				{
					if (status == -1)
					{
						tclMisc->setTclResult(interp, QObject::tr("-status option needs to be specified for 'status' history type."));
						return TCL_ERROR;
					}
					if (tclMisc->isDescStatus(status) && description == QString::null)
					{
						tclMisc->setTclResult(interp, QObject::tr("Descriptive status needs -description option to be specified."));
						return TCL_ERROR;
					}
					UserStatus *_status = new GaduStatus();
					_status->setIndex(status, description);
					history.appendStatus(uins[0], *_status);
					delete _status;
					break;
				}
				default:
					tclMisc->setTclResult(interp, QObject::tr("Type should be one of following: %1").arg(types.join(", ")));
					return TCL_ERROR;
			}
			break;
		}
		case 1:
		{
			QStringList opts = QStringList::split(" ", "-uins -type -from -count");
			UinsList uins;
			int from = -1, count = 1;
			int mask = HISTORYMANAGER_ENTRY_ALL;
			QString type = "all", arg;

			for ( int c = 2; c < argc; c++ )
			{
				arg = argv[c];
				c++;
				switch (opts.findIndex(arg))
				{
					case 0:
					{
						QStringList _uins = QStringList::split(" ", argv[c]);
						uint tmp;
						bool conv;
						for ( uint i = 0; i < _uins.count(); i++ )
						{
							tmp = _uins[i].toUInt(&conv);
							if (!conv)
							{
								tclMisc->setTclResult(interp, QObject::tr("-uins option takes a list of integer as argument"));
								return TCL_ERROR;
							}
							uins.append(tmp);
						}
						break;
					}
					case 1:
					{
						type = argv[c];
						break;
					}
					case 2:
					{
						bool conv;
						from = QString(argv[c]).toInt(&conv);
						if (!conv)
						{
							tclMisc->setTclResult(interp, QObject::tr("-from option takes an integer as argument."));
							return TCL_ERROR;
						}
						break;
					}
					case 3:
					{
						bool conv;
						count = QString(argv[c]).toInt(&conv);
						if (!conv)
						{
							tclMisc->setTclResult(interp, QObject::tr("-count option takes an integer as argument."));
							return TCL_ERROR;
						}
						break;
					}
					default:
					{
						tclMisc->setTclResult(interp, QObject::tr("Option should be one of following: %1").arg(opts.join(", ")));
						return TCL_ERROR;
					}
				}
			}
			if (uins.count() < 1)
			{
				tclMisc->setTclResult(interp, QObject::tr("-uins option needs to be specified for 'get' command."));
				return TCL_ERROR;
			}
			if (from == -1)
			{
				tclMisc->setTclResult(interp, QObject::tr("-from option needs to be specified for 'get' command."));
				return TCL_ERROR;
			}

			QValueList<HistoryEntry> entries;
			QStringList types = QStringList::split(" ", "sendmsg rcvmsg msg sms status all");
			int idx = types.findIndex(type);
			switch (idx)
			{
				case 0:
					mask = HISTORYMANAGER_ENTRY_CHATSEND;
					break;
				case 1:
					mask = HISTORYMANAGER_ENTRY_CHATRCV;
					break;
				case 2:
					mask = HISTORYMANAGER_ENTRY_ALL_MSGS;
					break;
				case 3:
					mask = HISTORYMANAGER_ENTRY_SMSSEND;
					break;
				case 4:
					mask = HISTORYMANAGER_ENTRY_STATUS;
					break;
				case 5:
					mask = HISTORYMANAGER_ENTRY_ALL;
					break;
			}

			entries = history.getHistoryEntries(uins, from, count, mask);
			QString tmp;
			Tcl_DString ds;
			Tcl_DStringInit(&ds);
			TclString *str = new TclString();

			switch (idx)
			{
				case 0:
				{
					for ( QValueList<HistoryEntry>::Iterator it = entries.begin(); it != entries.end(); ++it )
					{
						str->listBegin();
						str->appendElement( (*it).message );
						str->appendElement( QString::number( (*it).date.toTime_t() ) );
						str->listEnd();
					}
					break;
				}
				case 1:
				{
					for ( QValueList<HistoryEntry>::Iterator it = entries.begin(); it != entries.end(); ++it )
					{
						str->listBegin();
						str->appendElement( QString::number((*it).uin) );
						str->appendElement( (*it).message );
						str->appendElement( QString::number( (*it).date.toTime_t() ) );
						str->listEnd();
					}
					break;
				}
				case 2:
				{
					for ( QValueList<HistoryEntry>::Iterator it = entries.begin(); it != entries.end(); ++it )
					{
						str->listBegin();
						str->appendElement( QString::number((*it).uin) );
						str->appendElement( (*it).message );
						str->appendElement( QString::number( (*it).date.toTime_t() ) );
						str->listEnd();
					}
					break;
				}
				case 3:
				{
					for ( QValueList<HistoryEntry>::Iterator it = entries.begin(); it != entries.end(); ++it )
					{
						str->listBegin();
						str->appendElement( (*it).mobile );
						str->appendElement( (*it).message );
						str->appendElement( QString::number( (*it).date.toTime_t() ) );
						str->listEnd();
					}
					break;
				}
				case 4:
				{
					for ( QValueList<HistoryEntry>::Iterator it = entries.begin(); it != entries.end(); ++it )
					{
						str->listBegin();
						str->appendElement( QString::number((*it).uin) );
						str->appendElement( tclMisc->GGStatusToTclStatus((*it).status) );
						str->appendElement( (*it).description );
						str->appendElement( QString::number( (*it).date.toTime_t() ) );
						str->listEnd();
					}
					break;
				}
				case 5:
				{
					for ( QValueList<HistoryEntry>::Iterator it = entries.begin(); it != entries.end(); ++it )
					{
						str->listBegin();
						if ((*it).type != HISTORYMANAGER_ENTRY_CHATSEND)
							str->appendElement( QString::number((*it).uin) );

						if ((*it).type == HISTORYMANAGER_ENTRY_STATUS)
						{
							str->appendElement( tclMisc->GGStatusToTclStatus((*it).status) );
							str->appendElement( (*it).description );
						}
						if ((*it).type == HISTORYMANAGER_ENTRY_SMSSEND)
							str->appendElement( (*it).mobile );

						if ((*it).type == HISTORYMANAGER_ENTRY_ALL_MSGS)
							str->appendElement( (*it).message );

						str->appendElement( QString::number( (*it).date.toTime_t() ) );
						str->listEnd();
					}
					break;
				}
			}

			str->setResult(interp);
			delete str;
			break;
		}
		case 2:
		{
			if (argc != 3)
			{
				Tcl_WrongNumArgs(interp, 0, NULL, "hist count UINs");
				return TCL_ERROR;
			}
			UinsList uins;
			bool conv;
			QStringList _uins = QStringList::split(" ", argv[2]);
			for ( uint i = 0; i < _uins.count(); i++ )
			{
				uins.append(_uins[i].toInt(&conv));
				if (!conv)
				{
					tclMisc->setTclResult(interp, QObject::tr("UINs list should be a list of integers."));
					return TCL_ERROR;
				}
			}
			tclMisc->setTclResult(interp, QString::number(history.getHistoryEntriesCount(uins)));
			break;
		}
		default:
			tclMisc->setTclResult(interp, QObject::tr("Command should be one of following: %1").arg(cmds.join(", ")));
			return TCL_ERROR;
	}
	return TCL_OK;
}

/*
int Tcl_CmdXmms(void* clientData, Tcl_Interp *interp, int argc, const char *argv[])
{
	if (argc < 2)
	{
		Tcl_WrongNumArgs(interp, 0, NULL, "hist command ?options?");
		return TCL_ERROR;
	}
}
*/
